@@ -18,16 +18,18 @@ use std::path::Path;
 use std::time::SystemTime;
 
 use async_trait::async_trait;
-use jj_cli::cli_util::{CliRunner, CommandError, CommandHelper};
+use futures::stream::BoxStream;
+use jj_cli::cli_util::{CliRunner, CommandHelper};
+use jj_cli::command_error::CommandError;
 use jj_cli::ui::Ui;
 use jj_lib::backend::{
     Backend, BackendInitError, BackendLoadError, BackendResult, ChangeId, Commit, CommitId,
-    Conflict, ConflictId, FileId, SigningFn, SymlinkId, Tree, TreeId,
+    Conflict, ConflictId, CopyRecord, FileId, SigningFn, SymlinkId, Tree, TreeId,
 };
 use jj_lib::git_backend::GitBackend;
 use jj_lib::index::Index;
 use jj_lib::repo::StoreFactories;
-use jj_lib::repo_path::RepoPath;
+use jj_lib::repo_path::{RepoPath, RepoPathBuf};
 use jj_lib::settings::UserSettings;
 use jj_lib::signing::Signer;
 use jj_lib::workspace::{Workspace, WorkspaceInitError};
@@ -39,7 +41,7 @@ enum CustomCommand {
 }
 
 fn create_store_factories() -> StoreFactories {
-    let mut store_factories = StoreFactories::default();
+    let mut store_factories = StoreFactories::empty();
     // Register the backend so it can be loaded when the repo is loaded. The name
     // must match `Backend::name()`.
     store_factories.add_backend(
@@ -72,7 +74,7 @@ fn run_custom_command(
 
 fn main() -> std::process::ExitCode {
     CliRunner::init()
-        .set_store_factories(create_store_factories())
+        .add_store_factories(create_store_factories())
         .add_subcommand(run_custom_command)
         .run()
 }
@@ -173,6 +175,15 @@ impl Backend for JitBackend {
         self.inner.write_commit(contents, sign_with)
     }
 
+    fn get_copy_records(
+        &self,
+        paths: &[RepoPathBuf],
+        roots: &[CommitId],
+        heads: &[CommitId],
+    ) -> BackendResult<BoxStream<BackendResult<CopyRecord>>> {
+        self.inner.get_copy_records(paths, roots, heads)
+    }
+
     fn gc(&self, index: &dyn Index, keep_newer: SystemTime) -> BackendResult<()> {
         self.inner.gc(index, keep_newer)
     }
