@@ -17,13 +17,13 @@ use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
 use itertools::Itertools;
-use jj_cli::cli_util::{CliRunner, CommandError, CommandHelper};
+use jj_cli::cli_util::{CliRunner, CommandHelper};
+use jj_cli::command_error::CommandError;
 use jj_cli::ui::Ui;
 use jj_lib::backend::{Backend, MergedTreeId};
 use jj_lib::commit::Commit;
 use jj_lib::git_backend::GitBackend;
 use jj_lib::local_working_copy::LocalWorkingCopy;
-use jj_lib::merged_tree::MergedTree;
 use jj_lib::op_store::{OperationId, WorkspaceId};
 use jj_lib::repo::ReadonlyRepo;
 use jj_lib::repo_path::RepoPathBuf;
@@ -32,11 +32,9 @@ use jj_lib::signing::Signer;
 use jj_lib::store::Store;
 use jj_lib::working_copy::{
     CheckoutError, CheckoutStats, LockedWorkingCopy, ResetError, SnapshotError, SnapshotOptions,
-    WorkingCopy, WorkingCopyStateError,
-};
-use jj_lib::workspace::{
-    default_working_copy_factories, WorkingCopyInitializer, Workspace, WorkspaceInitError,
+    WorkingCopy, WorkingCopyFactory, WorkingCopyStateError,
 };
+use jj_lib::workspace::{WorkingCopyFactories, Workspace, WorkspaceInitError};
 
 #[derive(clap::Parser, Clone, Debug)]
 enum CustomCommand {
@@ -67,7 +65,7 @@ fn run_custom_command(
                 &ReadonlyRepo::default_op_heads_store_initializer(),
                 &ReadonlyRepo::default_index_store_initializer(),
                 &ReadonlyRepo::default_submodule_store_initializer(),
-                &ConflictsWorkingCopy::initializer(),
+                &ConflictsWorkingCopyFactory {},
                 WorkspaceId::default(),
             )?;
             Ok(())
@@ -76,19 +74,13 @@ fn run_custom_command(
 }
 
 fn main() -> std::process::ExitCode {
-    let mut working_copy_factories = default_working_copy_factories();
+    let mut working_copy_factories = WorkingCopyFactories::new();
     working_copy_factories.insert(
         ConflictsWorkingCopy::name().to_owned(),
-        Box::new(|store, working_copy_path, state_path| {
-            Box::new(ConflictsWorkingCopy::load(
-                store.clone(),
-                working_copy_path.to_owned(),
-                state_path.to_owned(),
-            ))
-        }),
+        Box::new(ConflictsWorkingCopyFactory {}),
     );
     CliRunner::init()
-        .set_working_copy_factories(working_copy_factories)
+        .add_working_copy_factories(working_copy_factories)
         .add_subcommand(run_custom_command)
         .run()
 }
@@ -113,8 +105,8 @@ impl ConflictsWorkingCopy {
         store: Arc<Store>,
         working_copy_path: PathBuf,
         state_path: PathBuf,
-        workspace_id: WorkspaceId,
         operation_id: OperationId,
+        workspace_id: WorkspaceId,
     ) -> Result<Self, WorkingCopyStateError> {
         let inner = LocalWorkingCopy::init(
             store,
@@ -128,21 +120,6 @@ impl ConflictsWorkingCopy {
         })
     }
 
-    fn initializer() -> Box<WorkingCopyInitializer<'static>> {
-        Box::new(
-            |store, working_copy_path, state_path, workspace_id, operation_id| {
-                let wc = Self::init(
-                    store,
-                    working_copy_path,
-                    state_path,
-                    workspace_id,
-                    operation_id,
-                )?;
-                Ok(Box::new(wc))
-            },
-        )
-    }
-
     fn load(store: Arc<Store>, working_copy_path: PathBuf, state_path: PathBuf) -> Self {
         let inner = LocalWorkingCopy::load(store, working_copy_path, state_path);
         ConflictsWorkingCopy {
@@ -189,6 +166,40 @@ impl WorkingCopy for ConflictsWorkingCopy {
     }
 }
 
+struct ConflictsWorkingCopyFactory {}
+
+impl WorkingCopyFactory for ConflictsWorkingCopyFactory {
+    fn init_working_copy(
+        &self,
+        store: Arc<Store>,
+        working_copy_path: PathBuf,
+        state_path: PathBuf,
+        operation_id: OperationId,
+        workspace_id: WorkspaceId,
+    ) -> Result<Box<dyn WorkingCopy>, WorkingCopyStateError> {
+        Ok(Box::new(ConflictsWorkingCopy::init(
+            store,
+            working_copy_path,
+            state_path,
+            operation_id,
+            workspace_id,
+        )?))
+    }
+
+    fn load_working_copy(
+        &self,
+        store: Arc<Store>,
+        working_copy_path: PathBuf,
+        state_path: PathBuf,
+    ) -> Result<Box<dyn WorkingCopy>, WorkingCopyStateError> {
+        Ok(Box::new(ConflictsWorkingCopy::load(
+            store,
+            working_copy_path,
+            state_path,
+        )))
+    }
+}
+
 struct LockedConflictsWorkingCopy {
     wc_path: PathBuf,
     inner: Box<dyn LockedWorkingCopy>,
@@ -212,7 +223,7 @@ impl LockedWorkingCopy for LockedConflictsWorkingCopy {
     }
 
     fn snapshot(&mut self, mut options: SnapshotOptions) -> Result<MergedTreeId, SnapshotError> {
-        options.base_ignores = options.base_ignores.chain("", "/.conflicts".as_bytes());
+        options.base_ignores = options.base_ignores.chain("", "/.conflicts".as_bytes())?;
         self.inner.snapshot(options)
     }
 
@@ -226,8 +237,12 @@ impl LockedWorkingCopy for LockedConflictsWorkingCopy {
         self.inner.check_out(commit)
     }
 
-    fn reset(&mut self, new_tree: &MergedTree) -> Result<(), ResetError> {
-        self.inner.reset(new_tree)
+    fn reset(&mut self, commit: &Commit) -> Result<(), ResetError> {
+        self.inner.reset(commit)
+    }
+
+    fn recover(&mut self, commit: &Commit) -> Result<(), ResetError> {
+        self.inner.recover(commit)
     }
 
     fn sparse_patterns(&self) -> Result<&[RepoPathBuf], WorkingCopyStateError> {
