@@ -22,8 +22,9 @@ use jj_lib::str_util::StringPattern;
 use jj_lib::workspace::Workspace;
 
 use crate::cli_util::{CommandHelper, WorkspaceCommandHelper};
-use crate::command_error::{user_error, user_error_with_message, CommandError};
+use crate::command_error::{cli_error, user_error, user_error_with_message, CommandError};
 use crate::commands::git::{map_git_error, maybe_add_gitignore};
+use crate::config::{write_config_value_to_file, ConfigNamePathBuf};
 use crate::git_util::{get_git_repo, print_git_import_stats, with_remote_git_callbacks};
 use crate::ui::Ui;
 
@@ -35,7 +36,10 @@ pub struct GitCloneArgs {
     /// URL or path of the Git repo to clone
     #[arg(value_hint = clap::ValueHint::DirPath)]
     source: String,
-    /// The directory to write the Jujutsu repo to
+    /// Specifies the target directory for the Jujutsu repository clone.
+    /// If not provided, defaults to a directory named after the last component
+    /// of the source URL. The full directory path will be created if it
+    /// doesn't exist.
     #[arg(value_hint = clap::ValueHint::DirPath)]
     destination: Option<String>,
     /// Whether or not to colocate the Jujutsu repo with the git repo
@@ -81,6 +85,9 @@ pub fn cmd_git_clone(
     command: &CommandHelper,
     args: &GitCloneArgs,
 ) -> Result<(), CommandError> {
+    if command.global_args().at_operation.is_some() {
+        return Err(cli_error("--at-op is not respected"));
+    }
     let remote_name = "origin";
     let source = absolute_git_source(command.cwd(), &args.source);
     let wc_path_str = args
@@ -89,22 +96,18 @@ pub fn cmd_git_clone(
         .or_else(|| clone_destination_for_source(&source))
         .ok_or_else(|| user_error("No destination specified and wasn't able to guess it"))?;
     let wc_path = command.cwd().join(wc_path_str);
-    let wc_path_existed = match fs::create_dir(&wc_path) {
-        Ok(()) => false,
-        Err(err) if err.kind() == io::ErrorKind::AlreadyExists => true,
-        Err(err) => {
-            return Err(user_error_with_message(
-                format!("Failed to create {wc_path_str}"),
-                err,
-            ));
-        }
-    };
+
+    let wc_path_existed = wc_path.exists();
     if wc_path_existed && !is_empty_dir(&wc_path) {
         return Err(user_error(
             "Destination path exists and is not an empty directory",
         ));
     }
 
+    // will create a tree dir in case if was deleted after last check
+    fs::create_dir_all(&wc_path)
+        .map_err(|err| user_error_with_message(format!("Failed to create {wc_path_str}"), err))?;
+
     // Canonicalize because fs::remove_dir_all() doesn't seem to like e.g.
     // `/some/path/.`
     let canonical_wc_path: PathBuf = wc_path
@@ -142,6 +145,18 @@ pub fn cmd_git_clone(
 
     let (mut workspace_command, stats) = clone_result?;
     if let Some(default_branch) = &stats.default_branch {
+        // Set repository level `trunk()` alias to the default remote branch.
+        let config_path = workspace_command.repo().repo_path().join("config.toml");
+        write_config_value_to_file(
+            &ConfigNamePathBuf::from_iter(["revset-aliases", "trunk()"]),
+            &format!("{default_branch}@{remote_name}"),
+            &config_path,
+        )?;
+        writeln!(
+            ui.status(),
+            "Setting the revset alias \"trunk()\" to \"{default_branch}@{remote_name}\""
+        )?;
+
         let default_branch_remote_ref = workspace_command
             .repo()
             .view()
@@ -180,7 +195,7 @@ fn do_git_clone(
         r#"Fetching into new repo in "{}""#,
         wc_path.display()
     )?;
-    let mut workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
+    let mut workspace_command = command.for_workable_repo(ui, workspace, repo)?;
     maybe_add_gitignore(&workspace_command)?;
     git_repo.remote(remote_name, source).unwrap();
     let mut fetch_tx = workspace_command.start_transaction();
