@@ -15,7 +15,8 @@
 use tracing::instrument;
 
 use super::new;
-use crate::cli_util::{CommandError, CommandHelper};
+use crate::cli_util::CommandHelper;
+use crate::command_error::{cli_error, CommandError};
 use crate::ui::Ui;
 
 #[instrument(skip_all)]
@@ -25,17 +26,15 @@ pub(crate) fn cmd_merge(
     args: &new::NewArgs,
 ) -> Result<(), CommandError> {
     writeln!(
-        ui.warning(),
-        "warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent"
+        ui.warning_default(),
+        "`jj merge` is deprecated; use `jj new` instead, which is equivalent"
     )?;
     writeln!(
-        ui.warning(),
-        "warning: `jj merge` will be removed in a future version, and this will be a hard error"
+        ui.warning_default(),
+        "`jj merge` will be removed in a future version, and this will be a hard error"
     )?;
     if args.revisions.len() < 2 {
-        return Err(CommandError::CliError(String::from(
-            "Merge requires at least two revisions",
-        )));
+        return Err(cli_error("Merge requires at least two revisions"));
     }
     new::cmd_new(ui, command, args)
 }
