@@ -113,7 +113,21 @@ pub(crate) fn cmd_obslog(
     let mut commits = topo_order_reverse_ok(
         vec![Ok(start_commit)],
         |commit: &Commit| commit.id().clone(),
-        |commit: &Commit| commit.predecessors().collect_vec(),
+        |commit: &Commit| {
+            let mut predecessors = commit.predecessors().collect_vec();
+            // Predecessors don't need to follow any defined order. However in
+            // practice, if there are multiple predecessors, then usually the
+            // first predecessor is the previous version of the same change, and
+            // the other predecessors are commits that were squashed into it. If
+            // multiple commits are squashed at once, then they are usually
+            // recorded in chronological order. We want to show squashed commits
+            // in reverse chronological order, and we also want to show squashed
+            // commits before the squash destination (since the destination's
+            // subgraph may contain earlier squashed commits as well), so we
+            // visit the predecessors in reverse order.
+            predecessors.reverse();
+            predecessors
+        },
     )?;
     if args.deprecated_limit.is_some() {
         writeln!(
@@ -131,18 +145,20 @@ pub(crate) fn cmd_obslog(
             for predecessor in commit.predecessors() {
                 edges.push(Edge::Direct(predecessor?.id().clone()));
             }
+            let graph_width = || graph.width(commit.id(), &edges);
             let mut buffer = vec![];
             with_content_format.write_graph_text(
                 ui.new_formatter(&mut buffer).as_mut(),
                 |formatter| template.format(&commit, formatter),
-                || graph.width(commit.id(), &edges),
+                graph_width,
             )?;
             if !buffer.ends_with(b"\n") {
                 buffer.push(b'\n');
             }
             if let Some(renderer) = &diff_renderer {
                 let mut formatter = ui.new_formatter(&mut buffer);
-                show_predecessor_patch(ui, repo, renderer, formatter.as_mut(), &commit)?;
+                let width = usize::saturating_sub(ui.term_width(), graph_width());
+                show_predecessor_patch(ui, repo, renderer, formatter.as_mut(), &commit, width)?;
             }
             let node_symbol = format_template(ui, &Some(commit.clone()), &node_template);
             graph.add_node(
@@ -157,7 +173,8 @@ pub(crate) fn cmd_obslog(
             with_content_format
                 .write(formatter, |formatter| template.format(&commit, formatter))?;
             if let Some(renderer) = &diff_renderer {
-                show_predecessor_patch(ui, repo, renderer, formatter, &commit)?;
+                let width = ui.term_width();
+                show_predecessor_patch(ui, repo, renderer, formatter, &commit, width)?;
             }
         }
     }
@@ -171,14 +188,18 @@ fn show_predecessor_patch(
     renderer: &DiffRenderer,
     formatter: &mut dyn Formatter,
     commit: &Commit,
+    width: usize,
 ) -> Result<(), CommandError> {
-    let mut predecessors = commit.predecessors();
-    let predecessor = match predecessors.next() {
-        Some(predecessor) => predecessor?,
-        None => return Ok(()),
-    };
-    let predecessor_tree = rebase_to_dest_parent(repo, &predecessor, commit)?;
+    let predecessors: Vec<_> = commit.predecessors().try_collect()?;
+    let predecessor_tree = rebase_to_dest_parent(repo, &predecessors, commit)?;
     let tree = commit.tree()?;
-    renderer.show_diff(ui, formatter, &predecessor_tree, &tree, &EverythingMatcher)?;
+    renderer.show_diff(
+        ui,
+        formatter,
+        &predecessor_tree,
+        &tree,
+        &EverythingMatcher,
+        width,
+    )?;
     Ok(())
 }
