@@ -13,14 +13,18 @@
 // limitations under the License.
 
 mod abandon;
+mod diff;
 mod log;
 mod restore;
+mod show;
 pub mod undo;
 
 use abandon::{cmd_op_abandon, OperationAbandonArgs};
 use clap::Subcommand;
+use diff::{cmd_op_diff, OperationDiffArgs};
 use log::{cmd_op_log, OperationLogArgs};
 use restore::{cmd_op_restore, OperationRestoreArgs};
+use show::{cmd_op_show, OperationShowArgs};
 use undo::{cmd_op_undo, OperationUndoArgs};
 
 use crate::cli_util::CommandHelper;
@@ -34,8 +38,10 @@ use crate::ui::Ui;
 #[derive(Subcommand, Clone, Debug)]
 pub enum OperationCommand {
     Abandon(OperationAbandonArgs),
+    Diff(OperationDiffArgs),
     Log(OperationLogArgs),
     Restore(OperationRestoreArgs),
+    Show(OperationShowArgs),
     Undo(OperationUndoArgs),
 }
 
@@ -46,8 +52,10 @@ pub fn cmd_operation(
 ) -> Result<(), CommandError> {
     match subcommand {
         OperationCommand::Abandon(args) => cmd_op_abandon(ui, command, args),
+        OperationCommand::Diff(args) => cmd_op_diff(ui, command, args),
         OperationCommand::Log(args) => cmd_op_log(ui, command, args),
         OperationCommand::Restore(args) => cmd_op_restore(ui, command, args),
+        OperationCommand::Show(args) => cmd_op_show(ui, command, args),
         OperationCommand::Undo(args) => cmd_op_undo(ui, command, args),
     }
 }
