@@ -19,7 +19,7 @@ use tracing::instrument;
 
 use crate::cli_util::{CommandHelper, RevisionArg};
 use crate::command_error::{user_error_with_hint, CommandError};
-use crate::description_util::{description_template_for_commit, edit_description};
+use crate::description_util::{description_template, edit_description};
 use crate::ui::Ui;
 
 /// Split a revision in two
@@ -88,8 +88,9 @@ pub(crate) fn cmd_split(
     let mut tx = workspace_command.start_transaction();
     let end_tree = commit.tree()?;
     let base_tree = commit.parent_tree(tx.repo())?;
-    let instructions = format!(
-        "\
+    let format_instructions = || {
+        format!(
+            "\
 You are splitting a commit into two: {}
 
 The diff initially shows the changes in the commit you're splitting.
@@ -98,12 +99,13 @@ Adjust the right side until it shows the contents you want for the first commit.
 The remainder will be in the second commit. If you don't make any changes, then
 the operation will be aborted.
 ",
-        tx.format_commit_summary(&commit)
-    );
+            tx.format_commit_summary(&commit)
+        )
+    };
 
     // Prompt the user to select the changes they want for the first commit.
     let selected_tree_id =
-        diff_selector.select(&base_tree, &end_tree, matcher.as_ref(), Some(&instructions))?;
+        diff_selector.select(&base_tree, &end_tree, matcher.as_ref(), format_instructions)?;
     if &selected_tree_id == commit.tree_id() && diff_selector.is_interactive() {
         // The user selected everything from the original commit.
         writeln!(ui.status(), "Nothing changed.")?;
@@ -120,63 +122,68 @@ the operation will be aborted.
 
     // Create the first commit, which includes the changes selected by the user.
     let selected_tree = tx.repo().store().get_root_tree(&selected_tree_id)?;
-    let first_template = description_template_for_commit(
-        ui,
-        command.settings(),
-        tx.base_workspace_helper(),
-        "Enter a description for the first commit.",
-        commit.description(),
-        &base_tree,
-        &selected_tree,
-    )?;
-    let first_description = edit_description(tx.base_repo(), &first_template, command.settings())?;
-    let first_commit = tx
-        .mut_repo()
-        .rewrite_commit(command.settings(), &commit)
-        .set_tree_id(selected_tree_id)
-        .set_description(first_description)
-        .write()?;
+    let first_commit = {
+        let mut commit_builder = tx
+            .mut_repo()
+            .rewrite_commit(command.settings(), &commit)
+            .detach();
+        commit_builder.set_tree_id(selected_tree_id);
+        if commit_builder.description().is_empty() {
+            commit_builder.set_description(command.settings().default_description());
+        }
+        let temp_commit = commit_builder.write_hidden()?;
+        let template = description_template(
+            &tx,
+            "Enter a description for the first commit.",
+            &temp_commit,
+        )?;
+        let description = edit_description(tx.base_repo(), &template, command.settings())?;
+        commit_builder.set_description(description);
+        commit_builder.write(tx.mut_repo())?
+    };
 
     // Create the second commit, which includes everything the user didn't
     // select.
-    let (second_tree, second_base_tree) = if args.parallel {
-        // Merge the original commit tree with its parent using the tree
-        // containing the user selected changes as the base for the merge.
-        // This results in a tree with the changes the user didn't select.
-        (end_tree.merge(&selected_tree, &base_tree)?, &base_tree)
-    } else {
-        (end_tree, &selected_tree)
+    let second_commit = {
+        let new_tree = if args.parallel {
+            // Merge the original commit tree with its parent using the tree
+            // containing the user selected changes as the base for the merge.
+            // This results in a tree with the changes the user didn't select.
+            end_tree.merge(&selected_tree, &base_tree)?
+        } else {
+            end_tree
+        };
+        let parents = if args.parallel {
+            commit.parent_ids().to_vec()
+        } else {
+            vec![first_commit.id().clone()]
+        };
+        let mut commit_builder = tx
+            .mut_repo()
+            .rewrite_commit(command.settings(), &commit)
+            .detach();
+        commit_builder
+            .set_parents(parents)
+            .set_tree_id(new_tree.id())
+            // Generate a new change id so that the commit being split doesn't
+            // become divergent.
+            .generate_new_change_id();
+        let description = if commit.description().is_empty() {
+            // If there was no description before, don't ask for one for the
+            // second commit.
+            "".to_string()
+        } else {
+            let temp_commit = commit_builder.write_hidden()?;
+            let template = description_template(
+                &tx,
+                "Enter a description for the second commit.",
+                &temp_commit,
+            )?;
+            edit_description(tx.base_repo(), &template, command.settings())?
+        };
+        commit_builder.set_description(description);
+        commit_builder.write(tx.mut_repo())?
     };
-    let second_commit_parents = if args.parallel {
-        commit.parent_ids().to_vec()
-    } else {
-        vec![first_commit.id().clone()]
-    };
-    let second_description = if commit.description().is_empty() {
-        // If there was no description before, don't ask for one for the second commit.
-        "".to_string()
-    } else {
-        let second_template = description_template_for_commit(
-            ui,
-            command.settings(),
-            tx.base_workspace_helper(),
-            "Enter a description for the second commit.",
-            commit.description(),
-            second_base_tree,
-            &second_tree,
-        )?;
-        edit_description(tx.base_repo(), &second_template, command.settings())?
-    };
-    let second_commit = tx
-        .mut_repo()
-        .rewrite_commit(command.settings(), &commit)
-        .set_parents(second_commit_parents)
-        .set_tree_id(second_tree.id())
-        // Generate a new change id so that the commit being split doesn't
-        // become divergent.
-        .generate_new_change_id()
-        .set_description(second_description)
-        .write()?;
 
     // Mark the commit being split as rewritten to the second commit. As a
     // result, if @ points to the commit being split, it will point to the
