@@ -14,14 +14,15 @@
 
 use std::cmp::max;
 use std::collections::VecDeque;
-use std::io;
 use std::ops::Range;
+use std::path::{Path, PathBuf};
+use std::{io, mem};
 
-use futures::{try_join, Stream, StreamExt};
+use futures::StreamExt;
 use itertools::Itertools;
-use jj_lib::backend::{BackendError, BackendResult, TreeValue};
+use jj_lib::backend::{BackendError, TreeValue};
 use jj_lib::commit::Commit;
-use jj_lib::conflicts::{materialize_tree_value, MaterializedTreeValue};
+use jj_lib::conflicts::{materialized_diff_stream, MaterializedTreeValue};
 use jj_lib::diff::{Diff, DiffHunk};
 use jj_lib::files::DiffLine;
 use jj_lib::matchers::Matcher;
@@ -29,7 +30,7 @@ use jj_lib::merge::MergedTreeValue;
 use jj_lib::merged_tree::{MergedTree, TreeDiffStream};
 use jj_lib::object_id::ObjectId;
 use jj_lib::repo::Repo;
-use jj_lib::repo_path::{RepoPath, RepoPathBuf, RepoPathUiConverter};
+use jj_lib::repo_path::{RepoPath, RepoPathUiConverter};
 use jj_lib::settings::{ConfigResultExt as _, UserSettings};
 use jj_lib::store::Store;
 use jj_lib::{diff, files};
@@ -40,11 +41,14 @@ use unicode_width::UnicodeWidthStr as _;
 
 use crate::config::CommandNameAndArgs;
 use crate::formatter::Formatter;
-use crate::merge_tools::{self, DiffGenerateError, ExternalMergeTool};
+use crate::merge_tools::{
+    self, generate_diff, invoke_external_diff, new_utf8_temp_dir, DiffGenerateError, DiffToolMode,
+    ExternalMergeTool,
+};
 use crate::text_util;
 use crate::ui::Ui;
 
-const DEFAULT_CONTEXT_LINES: usize = 3;
+pub const DEFAULT_CONTEXT_LINES: usize = 3;
 
 #[derive(clap::Args, Clone, Debug)]
 #[command(next_help_heading = "Diff Formatting Options")]
@@ -241,8 +245,23 @@ impl<'a> DiffRenderer<'a> {
         from_tree: &MergedTree,
         to_tree: &MergedTree,
         matcher: &dyn Matcher,
+        width: usize,
     ) -> Result<(), DiffRenderError> {
-        let repo = self.repo;
+        formatter.with_label("diff", |formatter| {
+            self.show_diff_inner(ui, formatter, from_tree, to_tree, matcher, width)
+        })
+    }
+
+    fn show_diff_inner(
+        &self,
+        ui: &Ui,
+        formatter: &mut dyn Formatter,
+        from_tree: &MergedTree,
+        to_tree: &MergedTree,
+        matcher: &dyn Matcher,
+        width: usize,
+    ) -> Result<(), DiffRenderError> {
+        let store = self.repo.store();
         let path_converter = self.path_converter;
         for format in &self.formats {
             match format {
@@ -252,9 +271,7 @@ impl<'a> DiffRenderer<'a> {
                 }
                 DiffFormat::Stat => {
                     let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                    // TODO: In graph log, graph width should be subtracted
-                    let width = usize::from(ui.term_width().unwrap_or(80));
-                    show_diff_stat(repo, formatter, tree_diff, path_converter, width)?;
+                    show_diff_stat(formatter, store, tree_diff, path_converter, width)?;
                 }
                 DiffFormat::Types => {
                     let tree_diff = from_tree.diff_stream(to_tree, matcher);
@@ -266,22 +283,30 @@ impl<'a> DiffRenderer<'a> {
                 }
                 DiffFormat::Git { context } => {
                     let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                    show_git_diff(repo, formatter, *context, tree_diff)?;
+                    show_git_diff(formatter, store, tree_diff, *context)?;
                 }
                 DiffFormat::ColorWords { context } => {
                     let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                    show_color_words_diff(repo, formatter, *context, tree_diff, path_converter)?;
+                    show_color_words_diff(formatter, store, tree_diff, path_converter, *context)?;
                 }
                 DiffFormat::Tool(tool) => {
-                    merge_tools::generate_diff(
-                        ui,
-                        formatter.raw(),
-                        from_tree,
-                        to_tree,
-                        matcher,
-                        tool,
-                    )
-                    .map_err(DiffRenderError::DiffGenerate)?;
+                    match tool.diff_invocation_mode {
+                        DiffToolMode::FileByFile => {
+                            let tree_diff = from_tree.diff_stream(to_tree, matcher);
+                            show_file_by_file_diff(
+                                ui,
+                                formatter,
+                                store,
+                                tree_diff,
+                                path_converter,
+                                tool,
+                            )
+                        }
+                        DiffToolMode::Dir => {
+                            generate_diff(ui, formatter.raw(), from_tree, to_tree, matcher, tool)
+                                .map_err(DiffRenderError::DiffGenerate)
+                        }
+                    }?;
                 }
             }
         }
@@ -295,10 +320,11 @@ impl<'a> DiffRenderer<'a> {
         formatter: &mut dyn Formatter,
         commit: &Commit,
         matcher: &dyn Matcher,
+        width: usize,
     ) -> Result<(), DiffRenderError> {
         let from_tree = commit.parent_tree(self.repo)?;
         let to_tree = commit.tree()?;
-        self.show_diff(ui, formatter, &from_tree, &to_tree, matcher)
+        self.show_diff(ui, formatter, &from_tree, &to_tree, matcher, width)
     }
 }
 
@@ -515,14 +541,13 @@ fn basic_diff_file_type(value: &MaterializedTreeValue) -> &'static str {
 }
 
 pub fn show_color_words_diff(
-    repo: &dyn Repo,
     formatter: &mut dyn Formatter,
-    num_context_lines: usize,
+    store: &Store,
     tree_diff: TreeDiffStream,
     path_converter: &RepoPathUiConverter,
+    num_context_lines: usize,
 ) -> Result<(), DiffRenderError> {
-    formatter.push_label("diff")?;
-    let mut diff_stream = materialized_diff_stream(repo.store(), tree_diff);
+    let mut diff_stream = materialized_diff_stream(store, tree_diff);
     async {
         while let Some((path, diff)) = diff_stream.next().await {
             let ui_path = path_converter.format_file_path(&path);
@@ -641,29 +666,93 @@ pub fn show_color_words_diff(
                 }
             }
         }
+        Ok(())
+    }
+    .block_on()
+}
+
+pub fn show_file_by_file_diff(
+    ui: &Ui,
+    formatter: &mut dyn Formatter,
+    store: &Store,
+    tree_diff: TreeDiffStream,
+    path_converter: &RepoPathUiConverter,
+    tool: &ExternalMergeTool,
+) -> Result<(), DiffRenderError> {
+    fn create_file(
+        path: &RepoPath,
+        wc_dir: &Path,
+        value: MaterializedTreeValue,
+    ) -> Result<PathBuf, DiffRenderError> {
+        let fs_path = path.to_fs_path(wc_dir);
+        std::fs::create_dir_all(fs_path.parent().unwrap())?;
+        let content = diff_content(path, value)?;
+        std::fs::write(&fs_path, content.contents)?;
+        Ok(fs_path)
+    }
+
+    let temp_dir = new_utf8_temp_dir("jj-diff-")?;
+    let left_wc_dir = temp_dir.path().join("left");
+    let right_wc_dir = temp_dir.path().join("right");
+    let mut diff_stream = materialized_diff_stream(store, tree_diff);
+    async {
+        while let Some((path, diff)) = diff_stream.next().await {
+            let ui_path = path_converter.format_file_path(&path);
+            let (left_value, right_value) = diff?;
+
+            match (&left_value, &right_value) {
+                (_, MaterializedTreeValue::AccessDenied(source))
+                | (MaterializedTreeValue::AccessDenied(source), _) => {
+                    write!(
+                        formatter.labeled("access-denied"),
+                        "Access denied to {ui_path}:"
+                    )?;
+                    writeln!(formatter, " {source}")?;
+                    continue;
+                }
+                _ => {}
+            }
+            let left_path = create_file(&path, &left_wc_dir, left_value)?;
+            let right_path = create_file(&path, &right_wc_dir, right_value)?;
+
+            invoke_external_diff(
+                ui,
+                formatter.raw(),
+                tool,
+                &maplit::hashmap! {
+                    "left" => left_path.to_str().expect("temp_dir should be valid utf-8"),
+                    "right" => right_path.to_str().expect("temp_dir should be valid utf-8"),
+                },
+            )
+            .map_err(DiffRenderError::DiffGenerate)?;
+        }
         Ok::<(), DiffRenderError>(())
     }
-    .block_on()?;
-    formatter.pop_label()?;
-    Ok(())
+    .block_on()
 }
 
 struct GitDiffPart {
-    mode: String,
+    /// Octal mode string or `None` if the file is absent.
+    mode: Option<&'static str>,
     hash: String,
-    content: Vec<u8>,
+    content: FileContent,
 }
 
 fn git_diff_part(
     path: &RepoPath,
     value: MaterializedTreeValue,
 ) -> Result<GitDiffPart, DiffRenderError> {
+    const DUMMY_HASH: &str = "0000000000";
     let mode;
     let mut hash;
-    let mut contents: Vec<u8>;
+    let content;
     match value {
         MaterializedTreeValue::Absent => {
-            panic!("Absent path {path:?} in diff should have been handled by caller");
+            return Ok(GitDiffPart {
+                mode: None,
+                hash: DUMMY_HASH.to_owned(),
+                content: FileContent::empty(),
+            });
         }
         MaterializedTreeValue::AccessDenied(err) => {
             return Err(DiffRenderError::AccessDenied {
@@ -676,39 +765,36 @@ fn git_diff_part(
             executable,
             mut reader,
         } => {
-            mode = if executable {
-                "100755".to_string()
-            } else {
-                "100644".to_string()
-            };
+            mode = if executable { "100755" } else { "100644" };
             hash = id.hex();
-            // TODO: use `file_content_for_diff` instead of showing binary
-            contents = vec![];
-            reader.read_to_end(&mut contents)?;
+            content = file_content_for_diff(&mut reader)?;
         }
         MaterializedTreeValue::Symlink { id, target } => {
-            mode = "120000".to_string();
+            mode = "120000";
             hash = id.hex();
-            contents = target.into_bytes();
+            content = FileContent {
+                // Unix file paths can't contain null bytes.
+                is_binary: false,
+                contents: target.into_bytes(),
+            };
         }
         MaterializedTreeValue::GitSubmodule(id) => {
             // TODO: What should we actually do here?
-            mode = "040000".to_string();
+            mode = "040000";
             hash = id.hex();
-            contents = vec![];
+            content = FileContent::empty();
         }
         MaterializedTreeValue::Conflict {
             id: _,
-            contents: conflict_data,
+            contents,
             executable,
         } => {
-            mode = if executable {
-                "100755".to_string()
-            } else {
-                "100644".to_string()
+            mode = if executable { "100755" } else { "100644" };
+            hash = DUMMY_HASH.to_owned();
+            content = FileContent {
+                is_binary: false, // TODO: are we sure this is never binary?
+                contents,
             };
-            hash = "0000000000".to_string();
-            contents = conflict_data
         }
         MaterializedTreeValue::Tree(_) => {
             panic!("Unexpected tree in diff at path {path:?}");
@@ -716,42 +802,52 @@ fn git_diff_part(
     }
     hash.truncate(10);
     Ok(GitDiffPart {
-        mode,
+        mode: Some(mode),
         hash,
-        content: contents,
+        content,
     })
 }
 
-#[derive(PartialEq)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 enum DiffLineType {
     Context,
     Removed,
     Added,
 }
 
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+enum DiffTokenType {
+    Matching,
+    Different,
+}
+
+type DiffTokenVec<'content> = Vec<(DiffTokenType, &'content [u8])>;
+
 struct UnifiedDiffHunk<'content> {
     left_line_range: Range<usize>,
     right_line_range: Range<usize>,
-    lines: Vec<(DiffLineType, &'content [u8])>,
+    lines: Vec<(DiffLineType, DiffTokenVec<'content>)>,
 }
 
 impl<'content> UnifiedDiffHunk<'content> {
     fn extend_context_lines(&mut self, lines: impl IntoIterator<Item = &'content [u8]>) {
         let old_len = self.lines.len();
-        self.lines
-            .extend(lines.into_iter().map(|line| (DiffLineType::Context, line)));
+        self.lines.extend(lines.into_iter().map(|line| {
+            let tokens = vec![(DiffTokenType::Matching, line)];
+            (DiffLineType::Context, tokens)
+        }));
         self.left_line_range.end += self.lines.len() - old_len;
         self.right_line_range.end += self.lines.len() - old_len;
     }
 
-    fn extend_removed_lines(&mut self, lines: impl IntoIterator<Item = &'content [u8]>) {
+    fn extend_removed_lines(&mut self, lines: impl IntoIterator<Item = DiffTokenVec<'content>>) {
         let old_len = self.lines.len();
         self.lines
             .extend(lines.into_iter().map(|line| (DiffLineType::Removed, line)));
         self.left_line_range.end += self.lines.len() - old_len;
     }
 
-    fn extend_added_lines(&mut self, lines: impl IntoIterator<Item = &'content [u8]>) {
+    fn extend_added_lines(&mut self, lines: impl IntoIterator<Item = DiffTokenVec<'content>>) {
         let old_len = self.lines.len();
         self.lines
             .extend(lines.into_iter().map(|line| (DiffLineType::Added, line)));
@@ -770,7 +866,7 @@ fn unified_diff_hunks<'content>(
         right_line_range: 1..1,
         lines: vec![],
     };
-    let diff = Diff::for_tokenizer(&[left_content, right_content], diff::find_line_ranges);
+    let diff = Diff::by_line([left_content, right_content]);
     let mut diff_hunks = diff.hunks().peekable();
     while let Some(hunk) = diff_hunks.next() {
         match hunk {
@@ -801,9 +897,9 @@ fn unified_diff_hunks<'content>(
                 // The next hunk should be of DiffHunk::Different type if any.
                 current_hunk.extend_context_lines(before_lines.into_iter().rev());
             }
-            DiffHunk::Different(content) => {
-                let left_lines = content[0].split_inclusive(|b| *b == b'\n');
-                let right_lines = content[1].split_inclusive(|b| *b == b'\n');
+            DiffHunk::Different(contents) => {
+                let [left, right] = contents.try_into().unwrap();
+                let (left_lines, right_lines) = inline_diff_hunks(left, right);
                 current_hunk.extend_removed_lines(left_lines);
                 current_hunk.extend_added_lines(right_lines);
             }
@@ -815,6 +911,60 @@ fn unified_diff_hunks<'content>(
     hunks
 }
 
+/// Splits line-level hunks into word-level tokens. Returns lists of tokens per
+/// line.
+fn inline_diff_hunks<'content>(
+    left_content: &'content [u8],
+    right_content: &'content [u8],
+) -> (Vec<DiffTokenVec<'content>>, Vec<DiffTokenVec<'content>>) {
+    let mut left_lines: Vec<DiffTokenVec<'content>> = vec![];
+    let mut right_lines: Vec<DiffTokenVec<'content>> = vec![];
+    let mut left_tokens: DiffTokenVec<'content> = vec![];
+    let mut right_tokens: DiffTokenVec<'content> = vec![];
+
+    // Like Diff::default_refinement(), but doesn't try to match up contents by
+    // lines. We know left/right_contents have no matching lines.
+    let mut diff = Diff::for_tokenizer([left_content, right_content], diff::find_word_ranges);
+    diff.refine_changed_regions(diff::find_nonword_ranges);
+    for hunk in diff.hunks() {
+        match hunk {
+            DiffHunk::Matching(content) => {
+                for token in content.split_inclusive(|b| *b == b'\n') {
+                    left_tokens.push((DiffTokenType::Matching, token));
+                    right_tokens.push((DiffTokenType::Matching, token));
+                    if token.ends_with(b"\n") {
+                        left_lines.push(mem::take(&mut left_tokens));
+                        right_lines.push(mem::take(&mut right_tokens));
+                    }
+                }
+            }
+            DiffHunk::Different(contents) => {
+                let [left, right] = contents.try_into().unwrap();
+                for token in left.split_inclusive(|b| *b == b'\n') {
+                    left_tokens.push((DiffTokenType::Different, token));
+                    if token.ends_with(b"\n") {
+                        left_lines.push(mem::take(&mut left_tokens));
+                    }
+                }
+                for token in right.split_inclusive(|b| *b == b'\n') {
+                    right_tokens.push((DiffTokenType::Different, token));
+                    if token.ends_with(b"\n") {
+                        right_lines.push(mem::take(&mut right_tokens));
+                    }
+                }
+            }
+        }
+    }
+
+    if !left_tokens.is_empty() {
+        left_lines.push(left_tokens);
+    }
+    if !right_tokens.is_empty() {
+        right_lines.push(right_tokens);
+    }
+    (left_lines, right_lines)
+}
+
 fn show_unified_diff_hunks(
     formatter: &mut dyn Formatter,
     left_content: &[u8],
@@ -830,27 +980,24 @@ fn show_unified_diff_hunks(
             hunk.right_line_range.start,
             hunk.right_line_range.len()
         )?;
-        for (line_type, content) in hunk.lines {
-            match line_type {
-                DiffLineType::Context => {
-                    formatter.with_label("context", |formatter| {
-                        write!(formatter, " ")?;
-                        formatter.write_all(content)
-                    })?;
+        for (line_type, tokens) in &hunk.lines {
+            let (label, sigil) = match line_type {
+                DiffLineType::Context => ("context", " "),
+                DiffLineType::Removed => ("removed", "-"),
+                DiffLineType::Added => ("added", "+"),
+            };
+            formatter.with_label(label, |formatter| {
+                write!(formatter, "{sigil}")?;
+                for (token_type, content) in tokens {
+                    match token_type {
+                        DiffTokenType::Matching => formatter.write_all(content)?,
+                        DiffTokenType::Different => formatter
+                            .with_label("token", |formatter| formatter.write_all(content))?,
+                    }
                 }
-                DiffLineType::Removed => {
-                    formatter.with_label("removed", |formatter| {
-                        write!(formatter, "-")?;
-                        formatter.write_all(content)
-                    })?;
-                }
-                DiffLineType::Added => {
-                    formatter.with_label("added", |formatter| {
-                        write!(formatter, "+")?;
-                        formatter.write_all(content)
-                    })?;
-                }
-            }
+                io::Result::Ok(())
+            })?;
+            let (_, content) = tokens.last().expect("hunk line must not be empty");
             if !content.ends_with(b"\n") {
                 write!(formatter, "\n\\ No newline at end of file\n")?;
             }
@@ -859,100 +1006,83 @@ fn show_unified_diff_hunks(
     Ok(())
 }
 
-fn materialized_diff_stream<'a>(
-    store: &'a Store,
-    tree_diff: TreeDiffStream<'a>,
-) -> impl Stream<
-    Item = (
-        RepoPathBuf,
-        BackendResult<(MaterializedTreeValue, MaterializedTreeValue)>,
-    ),
-> + 'a {
-    tree_diff
-        .map(|(path, diff)| async {
-            match diff {
-                Err(err) => (path, Err(err)),
-                Ok((before, after)) => {
-                    let before_future = materialize_tree_value(store, &path, before);
-                    let after_future = materialize_tree_value(store, &path, after);
-                    let values = try_join!(before_future, after_future);
-                    (path, values)
-                }
-            }
-        })
-        .buffered((store.concurrency() / 2).max(1))
-}
-
 pub fn show_git_diff(
-    repo: &dyn Repo,
     formatter: &mut dyn Formatter,
-    num_context_lines: usize,
+    store: &Store,
     tree_diff: TreeDiffStream,
+    num_context_lines: usize,
 ) -> Result<(), DiffRenderError> {
-    formatter.push_label("diff")?;
-
-    let mut diff_stream = materialized_diff_stream(repo.store(), tree_diff);
+    let mut diff_stream = materialized_diff_stream(store, tree_diff);
     async {
         while let Some((path, diff)) = diff_stream.next().await {
             let path_string = path.as_internal_file_string();
             let (left_value, right_value) = diff?;
-            if left_value.is_absent() {
-                let right_part = git_diff_part(&path, right_value)?;
-                formatter.with_label("file_header", |formatter| {
-                    writeln!(formatter, "diff --git a/{path_string} b/{path_string}")?;
-                    writeln!(formatter, "new file mode {}", &right_part.mode)?;
-                    writeln!(formatter, "index 0000000000..{}", &right_part.hash)?;
-                    writeln!(formatter, "--- /dev/null")?;
-                    writeln!(formatter, "+++ b/{path_string}")
-                })?;
-                show_unified_diff_hunks(formatter, &[], &right_part.content, num_context_lines)?;
-            } else if right_value.is_present() {
-                let left_part = git_diff_part(&path, left_value)?;
-                let right_part = git_diff_part(&path, right_value)?;
-                formatter.with_label("file_header", |formatter| {
-                    writeln!(formatter, "diff --git a/{path_string} b/{path_string}")?;
-                    if left_part.mode != right_part.mode {
-                        writeln!(formatter, "old mode {}", &left_part.mode)?;
-                        writeln!(formatter, "new mode {}", &right_part.mode)?;
-                        if left_part.hash != right_part.hash {
-                            writeln!(formatter, "index {}...{}", &left_part.hash, right_part.hash)?;
+            let left_part = git_diff_part(&path, left_value)?;
+            let right_part = git_diff_part(&path, right_value)?;
+            formatter.with_label("file_header", |formatter| {
+                writeln!(formatter, "diff --git a/{path_string} b/{path_string}")?;
+                let left_hash = &left_part.hash;
+                let right_hash = &right_part.hash;
+                match (left_part.mode, right_part.mode) {
+                    (None, Some(right_mode)) => {
+                        writeln!(formatter, "new file mode {right_mode}")?;
+                        writeln!(formatter, "index {left_hash}..{right_hash}")?;
+                    }
+                    (Some(left_mode), None) => {
+                        writeln!(formatter, "deleted file mode {left_mode}")?;
+                        writeln!(formatter, "index {left_hash}..{right_hash}")?;
+                    }
+                    (Some(left_mode), Some(right_mode)) => {
+                        if left_mode != right_mode {
+                            writeln!(formatter, "old mode {left_mode}")?;
+                            writeln!(formatter, "new mode {right_mode}")?;
+                            if left_hash != right_hash {
+                                writeln!(formatter, "index {left_hash}..{right_hash}")?;
+                            }
+                        } else if left_hash != right_hash {
+                            writeln!(formatter, "index {left_hash}..{right_hash} {left_mode}")?;
                         }
-                    } else if left_part.hash != right_part.hash {
-                        writeln!(
-                            formatter,
-                            "index {}...{} {}",
-                            &left_part.hash, right_part.hash, left_part.mode
-                        )?;
                     }
-                    if left_part.content != right_part.content {
-                        writeln!(formatter, "--- a/{path_string}")?;
-                        writeln!(formatter, "+++ b/{path_string}")?;
-                    }
-                    Ok(())
+                    (None, None) => panic!("either left or right part should be present"),
+                }
+                io::Result::Ok(())
+            })?;
+
+            if left_part.content.contents == right_part.content.contents {
+                continue; // no content hunks
+            }
+
+            let left_path = match left_part.mode {
+                Some(_) => format!("a/{path_string}"),
+                None => "/dev/null".to_owned(),
+            };
+            let right_path = match right_part.mode {
+                Some(_) => format!("b/{path_string}"),
+                None => "/dev/null".to_owned(),
+            };
+            if left_part.content.is_binary || right_part.content.is_binary {
+                // TODO: add option to emit Git binary diff
+                writeln!(
+                    formatter,
+                    "Binary files {left_path} and {right_path} differ"
+                )?;
+            } else {
+                formatter.with_label("file_header", |formatter| {
+                    writeln!(formatter, "--- {left_path}")?;
+                    writeln!(formatter, "+++ {right_path}")?;
+                    io::Result::Ok(())
                 })?;
                 show_unified_diff_hunks(
                     formatter,
-                    &left_part.content,
-                    &right_part.content,
+                    &left_part.content.contents,
+                    &right_part.content.contents,
                     num_context_lines,
                 )?;
-            } else {
-                let left_part = git_diff_part(&path, left_value)?;
-                formatter.with_label("file_header", |formatter| {
-                    writeln!(formatter, "diff --git a/{path_string} b/{path_string}")?;
-                    writeln!(formatter, "deleted file mode {}", &left_part.mode)?;
-                    writeln!(formatter, "index {}..0000000000", &left_part.hash)?;
-                    writeln!(formatter, "--- a/{path_string}")?;
-                    writeln!(formatter, "+++ /dev/null")
-                })?;
-                show_unified_diff_hunks(formatter, &left_part.content, &[], num_context_lines)?;
             }
         }
-        Ok::<(), DiffRenderError>(())
+        Ok(())
     }
-    .block_on()?;
-    formatter.pop_label()?;
-    Ok(())
+    .block_on()
 }
 
 #[instrument(skip_all)]
@@ -961,24 +1091,22 @@ pub fn show_diff_summary(
     mut tree_diff: TreeDiffStream,
     path_converter: &RepoPathUiConverter,
 ) -> io::Result<()> {
-    formatter.with_label("diff", |formatter| -> io::Result<()> {
-        async {
-            while let Some((repo_path, diff)) = tree_diff.next().await {
-                let (before, after) = diff.unwrap();
-                let ui_path = path_converter.format_file_path(&repo_path);
-                if before.is_present() && after.is_present() {
-                    writeln!(formatter.labeled("modified"), "M {ui_path}")?;
-                } else if before.is_absent() {
-                    writeln!(formatter.labeled("added"), "A {ui_path}")?;
-                } else {
-                    // `R` could be interpreted as "renamed"
-                    writeln!(formatter.labeled("removed"), "D {ui_path}")?;
-                }
+    async {
+        while let Some((repo_path, diff)) = tree_diff.next().await {
+            let (before, after) = diff.unwrap();
+            let ui_path = path_converter.format_file_path(&repo_path);
+            if before.is_present() && after.is_present() {
+                writeln!(formatter.labeled("modified"), "M {ui_path}")?;
+            } else if before.is_absent() {
+                writeln!(formatter.labeled("added"), "A {ui_path}")?;
+            } else {
+                // `R` could be interpreted as "renamed"
+                writeln!(formatter.labeled("removed"), "D {ui_path}")?;
             }
-            Ok(())
         }
-        .block_on()
-    })
+        Ok(())
+    }
+    .block_on()
 }
 
 struct DiffStat {
@@ -994,11 +1122,8 @@ fn get_diff_stat(
 ) -> DiffStat {
     // TODO: this matches git's behavior, which is to count the number of newlines
     // in the file. but that behavior seems unhelpful; no one really cares how
-    // many `0xa0` characters are in an image.
-    let diff = Diff::for_tokenizer(
-        &[&left_content.contents, &right_content.contents],
-        diff::find_line_ranges,
-    );
+    // many `0x0a` characters are in an image.
+    let diff = Diff::by_line([&left_content.contents, &right_content.contents]);
     let mut added = 0;
     let mut removed = 0;
     for hunk in diff.hunks() {
@@ -1019,8 +1144,8 @@ fn get_diff_stat(
 }
 
 pub fn show_diff_stat(
-    repo: &dyn Repo,
     formatter: &mut dyn Formatter,
+    store: &Store,
     tree_diff: TreeDiffStream,
     path_converter: &RepoPathUiConverter,
     display_width: usize,
@@ -1029,7 +1154,7 @@ pub fn show_diff_stat(
     let mut max_path_width = 0;
     let mut max_diffs = 0;
 
-    let mut diff_stream = materialized_diff_stream(repo.store(), tree_diff);
+    let mut diff_stream = materialized_diff_stream(store, tree_diff);
     async {
         while let Some((repo_path, diff)) = diff_stream.next().await {
             let (left, right) = diff?;
@@ -1058,40 +1183,37 @@ pub fn show_diff_stat(
         max_bar_length as f64 / max_diffs as f64
     };
 
-    formatter.with_label("diff", |formatter| {
-        let mut total_added = 0;
-        let mut total_removed = 0;
-        let total_files = stats.len();
-        for stat in &stats {
-            total_added += stat.added;
-            total_removed += stat.removed;
-            let bar_added = (stat.added as f64 * factor).ceil() as usize;
-            let bar_removed = (stat.removed as f64 * factor).ceil() as usize;
-            // replace start of path with ellipsis if the path is too long
-            let (path, path_width) = text_util::elide_start(&stat.path, "...", max_path_width);
-            let path_pad_width = max_path_width - path_width;
-            write!(
-                formatter,
-                "{path}{:path_pad_width$} | {:>number_padding$}{}",
-                "", // pad to max_path_width
-                stat.added + stat.removed,
-                if bar_added + bar_removed > 0 { " " } else { "" },
-            )?;
-            write!(formatter.labeled("added"), "{}", "+".repeat(bar_added))?;
-            writeln!(formatter.labeled("removed"), "{}", "-".repeat(bar_removed))?;
-        }
-        writeln!(
-            formatter.labeled("stat-summary"),
-            "{} file{} changed, {} insertion{}(+), {} deletion{}(-)",
-            total_files,
-            if total_files == 1 { "" } else { "s" },
-            total_added,
-            if total_added == 1 { "" } else { "s" },
-            total_removed,
-            if total_removed == 1 { "" } else { "s" },
+    let mut total_added = 0;
+    let mut total_removed = 0;
+    let total_files = stats.len();
+    for stat in &stats {
+        total_added += stat.added;
+        total_removed += stat.removed;
+        let bar_added = (stat.added as f64 * factor).ceil() as usize;
+        let bar_removed = (stat.removed as f64 * factor).ceil() as usize;
+        // replace start of path with ellipsis if the path is too long
+        let (path, path_width) = text_util::elide_start(&stat.path, "...", max_path_width);
+        let path_pad_width = max_path_width - path_width;
+        write!(
+            formatter,
+            "{path}{:path_pad_width$} | {:>number_padding$}{}",
+            "", // pad to max_path_width
+            stat.added + stat.removed,
+            if bar_added + bar_removed > 0 { " " } else { "" },
         )?;
-        Ok(())
-    })?;
+        write!(formatter.labeled("added"), "{}", "+".repeat(bar_added))?;
+        writeln!(formatter.labeled("removed"), "{}", "-".repeat(bar_removed))?;
+    }
+    writeln!(
+        formatter.labeled("stat-summary"),
+        "{} file{} changed, {} insertion{}(+), {} deletion{}(-)",
+        total_files,
+        if total_files == 1 { "" } else { "s" },
+        total_added,
+        if total_added == 1 { "" } else { "s" },
+        total_removed,
+        if total_removed == 1 { "" } else { "s" },
+    )?;
     Ok(())
 }
 
@@ -1100,22 +1222,20 @@ pub fn show_types(
     mut tree_diff: TreeDiffStream,
     path_converter: &RepoPathUiConverter,
 ) -> io::Result<()> {
-    formatter.with_label("diff", |formatter| {
-        async {
-            while let Some((repo_path, diff)) = tree_diff.next().await {
-                let (before, after) = diff.unwrap();
-                writeln!(
-                    formatter.labeled("modified"),
-                    "{}{} {}",
-                    diff_summary_char(&before),
-                    diff_summary_char(&after),
-                    path_converter.format_file_path(&repo_path)
-                )?;
-            }
-            Ok(())
+    async {
+        while let Some((repo_path, diff)) = tree_diff.next().await {
+            let (before, after) = diff.unwrap();
+            writeln!(
+                formatter.labeled("modified"),
+                "{}{} {}",
+                diff_summary_char(&before),
+                diff_summary_char(&after),
+                path_converter.format_file_path(&repo_path)
+            )?;
         }
-        .block_on()
-    })
+        Ok(())
+    }
+    .block_on()
 }
 
 fn diff_summary_char(value: &MergedTreeValue) -> char {
