@@ -6,7 +6,7 @@ use futures::{StreamExt, TryFutureExt, TryStreamExt};
 use itertools::Itertools;
 use jj_lib::backend::{BackendError, BackendResult, FileId, MergedTreeId, TreeValue};
 use jj_lib::conflicts::{materialize_tree_value, MaterializedTreeValue};
-use jj_lib::diff::{find_line_ranges, Diff, DiffHunk};
+use jj_lib::diff::{Diff, DiffHunk};
 use jj_lib::files::{self, ContentHunk, MergeResult};
 use jj_lib::matchers::Matcher;
 use jj_lib::merge::Merge;
@@ -225,10 +225,7 @@ fn make_diff_sections(
     left_contents: &str,
     right_contents: &str,
 ) -> Result<Vec<scm_record::Section<'static>>, BuiltinToolError> {
-    let diff = Diff::for_tokenizer(
-        &[left_contents.as_bytes(), right_contents.as_bytes()],
-        find_line_ranges,
-    );
+    let diff = Diff::by_line([left_contents.as_bytes(), right_contents.as_bytes()]);
     let mut sections = Vec::new();
     for hunk in diff.hunks() {
         match hunk {
@@ -604,8 +601,7 @@ pub fn edit_merge_builtin(
     path: &RepoPath,
     content: Merge<ContentHunk>,
 ) -> Result<MergedTreeId, BuiltinToolError> {
-    let slices = content.map(|ContentHunk(v)| v.as_slice());
-    let merge_result = files::merge(&slices);
+    let merge_result = files::merge(&content);
     let sections = make_merge_sections(merge_result)?;
     let mut input = scm_record::helpers::CrosstermInput;
     let recorder = scm_record::Recorder::new(
@@ -1017,8 +1013,7 @@ mod tests {
         let content = extract_as_single_hunk(&merge, store, path)
             .block_on()
             .unwrap();
-        let slices = content.map(|ContentHunk(buf)| buf.as_slice());
-        let merge_result = files::merge(&slices);
+        let merge_result = files::merge(&content);
         let sections = make_merge_sections(merge_result).unwrap();
         insta::assert_debug_snapshot!(sections, @r###"
         [
