@@ -39,10 +39,6 @@ pub trait ListTemplate: Template {
         Self: 'a;
 }
 
-pub trait IntoTemplate<'a> {
-    fn into_template(self) -> Box<dyn Template + 'a>;
-}
-
 impl<T: Template + ?Sized> Template for &T {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         <T as Template>::format(self, formatter)
@@ -78,6 +74,11 @@ impl Template for Signature {
     }
 }
 
+// In template language, an integer value is represented as i64. However, we use
+// usize here because it's more convenient to guarantee that the lower value is
+// bounded to 0.
+pub type SizeHint = (usize, Option<usize>);
+
 impl Template for String {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         write!(formatter, "{self}")
@@ -352,6 +353,26 @@ pub trait TemplatePropertyExt: TemplateProperty {
     {
         TemplateFunction::new(self, move |value| Ok(function(value)))
     }
+
+    /// Translates to a property that will unwrap an extracted `Option` value
+    /// of the specified `type_name`, mapping `None` to `Err`.
+    fn try_unwrap<O>(self, type_name: &str) -> impl TemplateProperty<Output = O>
+    where
+        Self: TemplateProperty<Output = Option<O>> + Sized,
+    {
+        self.and_then(move |opt| {
+            opt.ok_or_else(|| TemplatePropertyError(format!("No {type_name} available").into()))
+        })
+    }
+
+    /// Converts this property into `Template`.
+    fn into_template<'a>(self) -> Box<dyn Template + 'a>
+    where
+        Self: Sized + 'a,
+        Self::Output: Template,
+    {
+        Box::new(FormattablePropertyTemplate::new(self))
+    }
 }
 
 impl<P: TemplateProperty + ?Sized> TemplatePropertyExt for P {}
@@ -401,15 +422,6 @@ where
     }
 }
 
-impl<'a, O> IntoTemplate<'a> for Box<dyn TemplateProperty<Output = O> + 'a>
-where
-    O: Template + 'a,
-{
-    fn into_template(self) -> Box<dyn Template + 'a> {
-        Box::new(FormattablePropertyTemplate::new(self))
-    }
-}
-
 /// Adapter to turn template back to string property.
 pub struct PlainTextFormattedProperty<T> {
     template: T,
