@@ -25,23 +25,25 @@ fn test_branch_multiple_names() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo", "bar"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Warning: Creating multiple branches: foo, bar
+    Created 2 branches pointing to qpvuntsm 230dd059 bar foo | (empty) (no description set)
+    Hint: Use -r to specify the target revision.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar foo 230dd059e1b0
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     test_env.jj_cmd_ok(&repo_path, &["new"]);
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "set", "foo", "bar"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Warning: Updating multiple branches: foo, bar
+    Moved 2 branches to zsuskuln 8bb159bc bar foo | (empty) (no description set)
+    Hint: Use -r to specify the target revision.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar foo 8bb159bc30a9
-    ◉   230dd059e1b0
-    ◉   000000000000
+    ○   230dd059e1b0
+    ◆   000000000000
     "###);
 
     let (stdout, stderr) =
@@ -52,8 +54,30 @@ fn test_branch_multiple_names() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @   8bb159bc30a9
-    ◉   230dd059e1b0
-    ◉   000000000000
+    ○   230dd059e1b0
+    ◆   000000000000
+    "###);
+
+    // Hint should be omitted if -r is specified
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["branch", "create", "-r@-", "foo", "bar"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Created 2 branches pointing to qpvuntsm 230dd059 bar foo | (empty) (no description set)
+    "###);
+
+    // Create and move with explicit -r
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "set", "-r@", "bar", "baz"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Created 1 branches pointing to zsuskuln 8bb159bc bar baz | (empty) (no description set)
+    Moved 1 branches to zsuskuln 8bb159bc bar baz | (empty) (no description set)
+    Hint: Consider using `jj branch move` if your intention was to move existing branches.
+    "###);
+
+    // Noop changes should not be included in the stats
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["branch", "set", "-r@", "foo", "bar", "baz"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to zsuskuln 8bb159bc bar baz foo | (empty) (no description set)
     "###);
 }
 
@@ -66,7 +90,9 @@ fn test_branch_at_root() {
     let (stdout, stderr) =
         test_env.jj_cmd_ok(&repo_path, &["branch", "create", "fred", "-r=root()"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Created 1 branches pointing to zzzzzzzz 00000000 fred | (empty) (no description set)
+    "###);
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["git", "export"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
@@ -118,7 +144,7 @@ fn test_branch_move() {
 
     let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "set", "foo"]);
     insta::assert_snapshot!(stderr, @r###"
-    Created branches: foo
+    Created 1 branches pointing to qpvuntsm 230dd059 foo | (empty) (no description set)
     Hint: Consider using `jj branch move` if your intention was to move existing branches.
     "###);
 
@@ -130,7 +156,9 @@ fn test_branch_move() {
     "###);
 
     let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "set", "foo"]);
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to mzvwutvl 167f90e7 foo | (empty) (no description set)
+    "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "set", "-r@-", "foo"]);
     insta::assert_snapshot!(stderr, @r###"
@@ -142,10 +170,14 @@ fn test_branch_move() {
         &repo_path,
         &["branch", "set", "-r@-", "--allow-backwards", "foo"],
     );
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to qpvuntsm 230dd059 foo | (empty) (no description set)
+    "###);
 
     let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "move", "foo"]);
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to mzvwutvl 167f90e7 foo | (empty) (no description set)
+    "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "move", "--to=@-", "foo"]);
     insta::assert_snapshot!(stderr, @r###"
@@ -157,7 +189,9 @@ fn test_branch_move() {
         &repo_path,
         &["branch", "move", "--to=@-", "--allow-backwards", "foo"],
     );
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to qpvuntsm 230dd059 foo | (empty) (no description set)
+    "###);
 
     // Delete branch locally, but is still tracking remote
     test_env.jj_cmd_ok(&repo_path, &["describe", "@-", "-mcommit"]);
@@ -177,7 +211,9 @@ fn test_branch_move() {
 
     // Restoring local target shouldn't invalidate tracking state
     let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "set", "foo"]);
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to mzvwutvl 66d48752 foo* | (empty) (no description set)
+    "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     foo: mzvwutvl 66d48752 (empty) (no description set)
       @origin (behind by 1 commits): qpvuntsm 1eb845f3 (empty) commit
@@ -187,7 +223,9 @@ fn test_branch_move() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "untrack", "foo@origin"]);
     test_env.jj_cmd_ok(&repo_path, &["branch", "delete", "foo"]);
     let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo"]);
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Created 1 branches pointing to mzvwutvl 66d48752 foo | (empty) (no description set)
+    "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     foo: mzvwutvl 66d48752 (empty) (no description set)
     foo@origin: qpvuntsm 1eb845f3 (empty) commit
@@ -209,12 +247,12 @@ fn test_branch_move_matching() {
     test_env.jj_cmd_ok(&repo_path, &["new", "-mhead2"]);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @   a2781dd9ee37
-    ◉  c1 f4f38657a3dd
-    ◉  b1 f652c32197cf
-    │ ◉   6b5e840ea72b
-    │ ◉  a1 a2 230dd059e1b0
+    ○  c1 f4f38657a3dd
+    ○  b1 f652c32197cf
+    │ ○   6b5e840ea72b
+    │ ○  a1 a2 230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // The default could be considered "--from=all() glob:*", but is disabled
@@ -243,41 +281,40 @@ fn test_branch_move_matching() {
     // Noop move
     let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "move", "--to=a1", "a2"]);
     insta::assert_snapshot!(stderr, @r###"
-    Nothing changed.
+    No branches to update.
     "###);
 
     // Move from multiple revisions
     let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "move", "--from=::@"]);
     insta::assert_snapshot!(stderr, @r###"
-    Warning: Updating multiple branches: b1, c1
-    Hint: Specify branch by name to update one.
+    Moved 2 branches to vruxwmqv a2781dd9 b1 c1 | (empty) head2
+    Hint: Specify branch by name to update just one of the branches.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  b1 c1 a2781dd9ee37
-    ◉   f4f38657a3dd
-    ◉   f652c32197cf
-    │ ◉   6b5e840ea72b
-    │ ◉  a1 a2 230dd059e1b0
+    ○   f4f38657a3dd
+    ○   f652c32197cf
+    │ ○   6b5e840ea72b
+    │ ○  a1 a2 230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
 
     // Try to move multiple branches, but one of them isn't fast-forward
     let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "move", "glob:?1"]);
     insta::assert_snapshot!(stderr, @r###"
-    Warning: Updating multiple branches: a1, b1, c1
     Error: Refusing to move branch backwards or sideways: a1
     Hint: Use --allow-backwards to allow it.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @   a2781dd9ee37
-    ◉  c1 f4f38657a3dd
-    ◉  b1 f652c32197cf
-    │ ◉   6b5e840ea72b
-    │ ◉  a1 a2 230dd059e1b0
+    ○  c1 f4f38657a3dd
+    ○  b1 f652c32197cf
+    │ ○   6b5e840ea72b
+    │ ○  a1 a2 230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // Select by revision and name
@@ -285,15 +322,17 @@ fn test_branch_move_matching() {
         &repo_path,
         &["branch", "move", "--from=::a1+", "--to=a1+", "glob:?1"],
     );
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to kkmpptxz 6b5e840e a1 | (empty) head1
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @   a2781dd9ee37
-    ◉  c1 f4f38657a3dd
-    ◉  b1 f652c32197cf
-    │ ◉  a1 6b5e840ea72b
-    │ ◉  a2 230dd059e1b0
+    ○  c1 f4f38657a3dd
+    ○  b1 f652c32197cf
+    │ ○  a1 6b5e840ea72b
+    │ ○  a2 230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
 }
 
@@ -324,12 +363,12 @@ fn test_branch_move_conflicting() {
     );
     insta::assert_snapshot!(get_log(), @r###"
     @  A1
-    ◉  A0 foo??
-    │ ◉  C0
+    ○  A0 foo??
+    │ ○  C0
     ├─╯
-    │ ◉  B0 foo??
+    │ ○  B0 foo??
     ├─╯
-    ◉
+    ◆
     "###);
 
     // Can't move the branch to C0 since it's sibling.
@@ -344,15 +383,17 @@ fn test_branch_move_conflicting() {
     // descendant of B0, though.
     let (_stdout, stderr) =
         test_env.jj_cmd_ok(&repo_path, &["branch", "set", "-rdescription(A1)", "foo"]);
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to mzvwutvl 9328d344 foo | (empty) A1
+    "###);
     insta::assert_snapshot!(get_log(), @r###"
     @  A1 foo
-    ◉  A0
-    │ ◉  C0
+    ○  A0
+    │ ○  C0
     ├─╯
-    │ ◉  B0
+    │ ○  B0
     ├─╯
-    ◉
+    ◆
     "###);
 }
 
@@ -407,6 +448,21 @@ fn test_branch_rename() {
     "###);
 }
 
+#[test]
+fn test_branch_rename_colocated() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo", "--colocate"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m=commit-0"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "blocal"]);
+
+    // Make sure that git tracking branches don't cause a warning
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["branch", "rename", "blocal", "blocal1"]);
+    insta::assert_snapshot!(stderr, @"");
+}
+
 #[test]
 fn test_branch_forget_glob() {
     let test_env = TestEnvironment::default();
@@ -420,7 +476,7 @@ fn test_branch_forget_glob() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar-2 foo-1 foo-3 foo-4 230dd059e1b0
-    ◉   000000000000
+    ◆   000000000000
     "###);
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "forget", "glob:foo-[1-3]"]);
     insta::assert_snapshot!(stdout, @"");
@@ -435,7 +491,7 @@ fn test_branch_forget_glob() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar-2 foo-4 230dd059e1b0
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // Forgetting a branch via both explicit name and glob pattern, or with
@@ -445,10 +501,12 @@ fn test_branch_forget_glob() {
         &["branch", "forget", "foo-4", "glob:foo-*", "glob:foo-*"],
     );
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Forgot 1 branches.
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar-2 230dd059e1b0
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // Malformed glob
@@ -497,7 +555,7 @@ fn test_branch_delete_glob() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar-2 foo-1 foo-3 foo-4 312a98d6f27b
-    ◉   000000000000
+    ◆   000000000000
     "###);
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "delete", "glob:foo-[1-3]"]);
     insta::assert_snapshot!(stdout, @"");
@@ -512,7 +570,7 @@ fn test_branch_delete_glob() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar-2 foo-1@origin foo-3@origin foo-4 312a98d6f27b
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // We get an error if none of the globs match live branches. Unlike `jj branch
@@ -529,10 +587,12 @@ fn test_branch_delete_glob() {
         &["branch", "delete", "foo-4", "glob:foo-*", "glob:foo-*"],
     );
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Deleted 1 branches.
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  bar-2 foo-1@origin foo-3@origin foo-4@origin 312a98d6f27b
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // The deleted branches are still there
@@ -561,7 +621,7 @@ fn test_branch_delete_glob() {
     error: invalid value 'whatever:branch' for '<NAMES>...': Invalid string pattern kind "whatever:"
 
     For more information, try '--help'.
-    Hint: Try prefixing with one of `exact:`, `glob:` or `substring:`
+    Hint: Try prefixing with one of `exact:`, `glob:`, `regex:`, or `substring:`
     "###);
 }
 
@@ -608,7 +668,9 @@ fn test_branch_forget_export() {
     insta::assert_snapshot!(stderr, @"");
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "forget", "foo"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Forgot 1 branches.
+    "###);
     // Forgetting a branch deletes local and remote-tracking branches including
     // the corresponding git-tracking branch.
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @"");
@@ -734,7 +796,9 @@ fn test_branch_forget_fetched_branch() {
         .unwrap();
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "forget", "feature1"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Forgot 1 branches.
+    "###);
 
     // Fetching a moved branch does not create a conflict
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["git", "fetch", "--remote=origin"]);
@@ -865,10 +929,10 @@ fn test_branch_track_untrack() {
     main@origin: sptzoqmo 7b33f629 commit 1
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    ◉  feature1@origin feature2@origin main@origin 7b33f6295eda
+    ◆  feature1@origin feature2@origin main@origin 7b33f6295eda
     │ @   230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // Track new branch. Local branch should be created.
@@ -913,10 +977,10 @@ fn test_branch_track_untrack() {
       @origin: sptzoqmo 7b33f629 commit 1
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    ◉  feature1 feature1@origin feature2@origin main 7b33f6295eda
+    ◆  feature1 feature1@origin feature2@origin main 7b33f6295eda
     │ @   230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // Fetch new commit. Only tracking branch "main" should be merged.
@@ -944,12 +1008,12 @@ fn test_branch_track_untrack() {
       @origin: mmqqkyyt 40dabdaf commit 2
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    ◉  feature1@origin feature2@origin main 40dabdaf4abe
-    │ ◉  feature1 7b33f6295eda
+    ◆  feature1@origin feature2@origin main 40dabdaf4abe
+    │ ○  feature1 7b33f6295eda
     ├─╯
     │ @   230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
 
     // Fetch new commit with auto tracking. Tracking branch "main" and new
@@ -983,12 +1047,12 @@ fn test_branch_track_untrack() {
       @origin: wwnpyzpo 3f0f86fa commit 3
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    ◉  feature1@origin feature2@origin feature3 main 3f0f86fa0e57
-    │ ◉  feature1 7b33f6295eda
+    ◆  feature1@origin feature2@origin feature3 main 3f0f86fa0e57
+    │ ○  feature1 7b33f6295eda
     ├─╯
     │ @   230dd059e1b0
     ├─╯
-    ◉   000000000000
+    ◆   000000000000
     "###);
 }
 
@@ -1014,6 +1078,7 @@ fn test_branch_track_conflict() {
     );
     let (_, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "track", "main@origin"]);
     insta::assert_snapshot!(stderr, @r###"
+    Started tracking 1 remote branches.
     main (conflicted):
       + qpvuntsm e802c4f8 (empty) b
       + qpvuntsm hidden 427890ea (empty) a
@@ -1129,6 +1194,7 @@ fn test_branch_track_untrack_patterns() {
     Warning: Git-tracking branch cannot be untracked: feature1@git
     Warning: Remote branch not tracked yet: feature2@origin
     Warning: Git-tracking branch cannot be untracked: main@git
+    Stopped tracking 1 remote branches.
     "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     feature1: omvolwpu 1336caed commit
@@ -1368,16 +1434,16 @@ fn test_branch_list_filtered() {
             &["log", "-r::(branches() | remote_branches())", "-T", template],
         ),
         @r###"
-    ◉  e31634b64294 remote-rewrite*
+    ○  e31634b64294 remote-rewrite*
     │ @  c7b4c09cd77c local-keep
     ├─╯
-    │ ◉  3e9a5af6ef15 remote-rewrite@origin (hidden)
+    │ ○  3e9a5af6ef15 remote-rewrite@origin (hidden)
     ├─╯
-    │ ◉  dad5f298ca57 remote-delete@origin
+    │ ○  dad5f298ca57 remote-delete@origin
     ├─╯
-    │ ◉  911e912015fb remote-keep
+    │ ○  911e912015fb remote-keep
     ├─╯
-    ◉  000000000000
+    ◆  000000000000
     "###);
 
     // All branches are listed by default.
