@@ -55,14 +55,14 @@ fn test_diff_basic() {
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--color=debug"]);
     insta::assert_snapshot!(stdout, @r###"
-    [38;5;3m<<diff header::Removed >><<diff header::regular file>><<diff header:: >><<diff header::file1>><<diff header:::>>[39m
-    [38;5;1m<<diff removed line_number:: >><<diff removed line_number:: >><<diff removed line_number:: >><<diff removed line_number::1>>[39m<<diff:: >><<diff::    : >>[4m[38;5;1m<<diff removed token::foo>>[24m[39m
-    [38;5;3m<<diff header::Modified regular file>><<diff header:: >><<diff header::file2>><<diff header:::>>[39m
-    [38;5;1m<<diff removed line_number:: >><<diff removed line_number:: >><<diff removed line_number:: >><<diff removed line_number::1>>[39m<<diff:: >>[38;5;2m<<diff added line_number:: >><<diff added line_number:: >><<diff added line_number:: >><<diff added line_number::1>>[39m<<diff::: >><<diff::foo>>
-    <<diff::     >>[38;5;2m<<diff added line_number:: >><<diff added line_number:: >><<diff added line_number:: >><<diff added line_number::2>>[39m<<diff::: >>[4m[38;5;2m<<diff added token::bar>>[24m[39m
-    [38;5;1m<<diff removed line_number:: >><<diff removed line_number:: >><<diff removed line_number:: >><<diff removed line_number::2>>[39m<<diff:: >>[38;5;2m<<diff added line_number:: >><<diff added line_number:: >><<diff added line_number:: >><<diff added line_number::3>>[39m<<diff::: >><<diff::baz >>[4m[38;5;1m<<diff removed token::qux>>[38;5;2m<<diff added token::quux>>[24m[39m<<diff::>>
-    [38;5;3m<<diff header::Added >><<diff header::regular file>><<diff header:: >><<diff header::file3>><<diff header:::>>[39m
-    <<diff::     >>[38;5;2m<<diff added line_number:: >><<diff added line_number:: >><<diff added line_number:: >><<diff added line_number::1>>[39m<<diff::: >>[4m[38;5;2m<<diff added token::foo>>[24m[39m
+    [38;5;3m<<diff header::Removed regular file file1:>>[39m
+    [38;5;1m<<diff removed line_number::   1>>[39m<<diff::     : >>[4m[38;5;1m<<diff removed token::foo>>[24m[39m
+    [38;5;3m<<diff header::Modified regular file file2:>>[39m
+    [38;5;1m<<diff removed line_number::   1>>[39m<<diff:: >>[38;5;2m<<diff added line_number::   1>>[39m<<diff::: foo>>
+    <<diff::     >>[38;5;2m<<diff added line_number::   2>>[39m<<diff::: >>[4m[38;5;2m<<diff added token::bar>>[24m[39m
+    [38;5;1m<<diff removed line_number::   2>>[39m<<diff:: >>[38;5;2m<<diff added line_number::   3>>[39m<<diff::: baz >>[4m[38;5;1m<<diff removed token::qux>>[38;5;2m<<diff added token::quux>>[24m[39m<<diff::>>
+    [38;5;3m<<diff header::Added regular file file3:>>[39m
+    <<diff::     >>[38;5;2m<<diff added line_number::   1>>[39m<<diff::: >>[4m[38;5;2m<<diff added token::foo>>[24m[39m
     "###);
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
@@ -89,7 +89,7 @@ fn test_diff_basic() {
     @@ -1,1 +1,0 @@
     -foo
     diff --git a/file2 b/file2
-    index 523a4a9de8...485b56a572 100644
+    index 523a4a9de8..485b56a572 100644
     --- a/file2
     +++ b/file2
     @@ -1,2 +1,3 @@
@@ -116,7 +116,7 @@ fn test_diff_basic() {
     @@ -1,1 +1,0 @@
     -foo
     diff --git a/file2 b/file2
-    index 523a4a9de8...485b56a572 100644
+    index 523a4a9de8..485b56a572 100644
     --- a/file2
     +++ b/file2
     @@ -2,1 +2,2 @@
@@ -134,29 +134,29 @@ fn test_diff_basic() {
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--git", "--color=debug"]);
     insta::assert_snapshot!(stdout, @r###"
-    [1m<<diff file_header::diff --git a/>><<diff file_header::file1>><<diff file_header:: b/>><<diff file_header::file1>><<diff file_header::>>[0m
-    [1m<<diff file_header::deleted file mode >><<diff file_header::100644>><<diff file_header::>>[0m
-    [1m<<diff file_header::index >><<diff file_header::257cc5642c>><<diff file_header::..0000000000>>[0m
-    [1m<<diff file_header::--- a/>><<diff file_header::file1>><<diff file_header::>>[0m
+    [1m<<diff file_header::diff --git a/file1 b/file1>>[0m
+    [1m<<diff file_header::deleted file mode 100644>>[0m
+    [1m<<diff file_header::index 257cc5642c..0000000000>>[0m
+    [1m<<diff file_header::--- a/file1>>[0m
     [1m<<diff file_header::+++ /dev/null>>[0m
-    [38;5;6m<<diff hunk_header::@@ ->><<diff hunk_header::1>><<diff hunk_header::,>><<diff hunk_header::1>><<diff hunk_header:: +>><<diff hunk_header::1>><<diff hunk_header::,>><<diff hunk_header::0>><<diff hunk_header:: @@>>[39m
-    [38;5;1m<<diff removed::->><<diff removed::foo>>[39m
-    [1m<<diff file_header::diff --git a/>><<diff file_header::file2>><<diff file_header:: b/>><<diff file_header::file2>><<diff file_header::>>[0m
-    [1m<<diff file_header::index >><<diff file_header::523a4a9de8>><<diff file_header::...>><<diff file_header::485b56a572>><<diff file_header:: >><<diff file_header::100644>><<diff file_header::>>[0m
-    [1m<<diff file_header::--- a/>><<diff file_header::file2>><<diff file_header::>>[0m
-    [1m<<diff file_header::+++ b/>><<diff file_header::file2>><<diff file_header::>>[0m
-    [38;5;6m<<diff hunk_header::@@ ->><<diff hunk_header::1>><<diff hunk_header::,>><<diff hunk_header::2>><<diff hunk_header:: +>><<diff hunk_header::1>><<diff hunk_header::,>><<diff hunk_header::3>><<diff hunk_header:: @@>>[39m
-    <<diff context:: >><<diff context::foo>>
-    [38;5;1m<<diff removed::->><<diff removed::baz qux>>[39m
-    [38;5;2m<<diff added::+>><<diff added::bar>>[39m
-    [38;5;2m<<diff added::+>><<diff added::baz quux>>[39m
-    [1m<<diff file_header::diff --git a/>><<diff file_header::file3>><<diff file_header:: b/>><<diff file_header::file3>><<diff file_header::>>[0m
-    [1m<<diff file_header::new file mode >><<diff file_header::100644>><<diff file_header::>>[0m
-    [1m<<diff file_header::index 0000000000..>><<diff file_header::257cc5642c>><<diff file_header::>>[0m
+    [38;5;6m<<diff hunk_header::@@ -1,1 +1,0 @@>>[39m
+    [38;5;1m<<diff removed::->>[4m<<diff removed token::foo>>[24m[39m
+    [1m<<diff file_header::diff --git a/file2 b/file2>>[0m
+    [1m<<diff file_header::index 523a4a9de8..485b56a572 100644>>[0m
+    [1m<<diff file_header::--- a/file2>>[0m
+    [1m<<diff file_header::+++ b/file2>>[0m
+    [38;5;6m<<diff hunk_header::@@ -1,2 +1,3 @@>>[39m
+    <<diff context:: foo>>
+    [38;5;1m<<diff removed::-baz >>[4m<<diff removed token::qux>>[24m<<diff removed::>>[39m
+    [38;5;2m<<diff added::+>>[4m<<diff added token::bar>>[24m[39m
+    [38;5;2m<<diff added::+baz >>[4m<<diff added token::quux>>[24m<<diff added::>>[39m
+    [1m<<diff file_header::diff --git a/file3 b/file3>>[0m
+    [1m<<diff file_header::new file mode 100644>>[0m
+    [1m<<diff file_header::index 0000000000..257cc5642c>>[0m
     [1m<<diff file_header::--- /dev/null>>[0m
-    [1m<<diff file_header::+++ b/>><<diff file_header::file3>><<diff file_header::>>[0m
-    [38;5;6m<<diff hunk_header::@@ ->><<diff hunk_header::1>><<diff hunk_header::,>><<diff hunk_header::0>><<diff hunk_header:: +>><<diff hunk_header::1>><<diff hunk_header::,>><<diff hunk_header::1>><<diff hunk_header:: @@>>[39m
-    [38;5;2m<<diff added::+>><<diff added::foo>>[39m
+    [1m<<diff file_header::+++ b/file3>>[0m
+    [38;5;6m<<diff hunk_header::@@ -1,0 +1,1 @@>>[39m
+    [38;5;2m<<diff added::+>>[4m<<diff added token::foo>>[24m[39m
     "###);
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "--git"]);
@@ -172,7 +172,7 @@ fn test_diff_basic() {
     @@ -1,1 +1,0 @@
     -foo
     diff --git a/file2 b/file2
-    index 523a4a9de8...485b56a572 100644
+    index 523a4a9de8..485b56a572 100644
     --- a/file2
     +++ b/file2
     @@ -1,2 +1,3 @@
@@ -198,15 +198,7 @@ fn test_diff_basic() {
     "###);
 
     // Filter by glob pattern
-    let stdout = test_env.jj_cmd_success(
-        &repo_path,
-        &[
-            "diff",
-            "--config-toml=ui.allow-filesets=true",
-            "-s",
-            r#"glob:"file[12]""#,
-        ],
-    );
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "glob:file[12]"]);
     insta::assert_snapshot!(stdout, @r###"
     D file1
     M file2
@@ -268,6 +260,146 @@ fn test_diff_empty() {
     "###);
 }
 
+#[test]
+fn test_diff_file_mode() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    // Test content+mode/mode-only changes of empty/non-empty files:
+    // - file1: ("",  x) -> ("2", n)  empty, content+mode
+    // - file2: ("1", x) -> ("1", n)  non-empty, mode-only
+    // - file3: ("1", n) -> ("2", x)  non-empty, content+mode
+    // - file4: ("",  n) -> ("",  x)  empty, mode-only
+
+    std::fs::write(repo_path.join("file1"), "").unwrap();
+    std::fs::write(repo_path.join("file2"), "1\n").unwrap();
+    std::fs::write(repo_path.join("file3"), "1\n").unwrap();
+    std::fs::write(repo_path.join("file4"), "").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["file", "chmod", "x", "file1", "file2"]);
+
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    std::fs::write(repo_path.join("file1"), "2\n").unwrap();
+    std::fs::write(repo_path.join("file3"), "2\n").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["file", "chmod", "n", "file1", "file2"]);
+    test_env.jj_cmd_ok(&repo_path, &["file", "chmod", "x", "file3", "file4"]);
+
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    std::fs::remove_file(repo_path.join("file1")).unwrap();
+    std::fs::remove_file(repo_path.join("file2")).unwrap();
+    std::fs::remove_file(repo_path.join("file3")).unwrap();
+    std::fs::remove_file(repo_path.join("file4")).unwrap();
+
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-r@--"]);
+    insta::assert_snapshot!(stdout, @r###"
+    Added executable file file1:
+        (empty)
+    Added executable file file2:
+            1: 1
+    Added regular file file3:
+            1: 1
+    Added regular file file4:
+        (empty)
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-r@-"]);
+    insta::assert_snapshot!(stdout, @r###"
+    Executable file became non-executable at file1:
+            1: 2
+    Executable file became non-executable at file2:
+    Non-executable file became executable at file3:
+       1    1: 12
+    Non-executable file became executable at file4:
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-r@"]);
+    insta::assert_snapshot!(stdout, @r###"
+    Removed regular file file1:
+       1     : 2
+    Removed regular file file2:
+       1     : 1
+    Removed executable file file3:
+       1     : 2
+    Removed executable file file4:
+        (empty)
+    "###);
+
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-r@--", "--git"]);
+    insta::assert_snapshot!(stdout, @r###"
+    diff --git a/file1 b/file1
+    new file mode 100755
+    index 0000000000..e69de29bb2
+    diff --git a/file2 b/file2
+    new file mode 100755
+    index 0000000000..d00491fd7e
+    --- /dev/null
+    +++ b/file2
+    @@ -1,0 +1,1 @@
+    +1
+    diff --git a/file3 b/file3
+    new file mode 100644
+    index 0000000000..d00491fd7e
+    --- /dev/null
+    +++ b/file3
+    @@ -1,0 +1,1 @@
+    +1
+    diff --git a/file4 b/file4
+    new file mode 100644
+    index 0000000000..e69de29bb2
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-r@-", "--git"]);
+    insta::assert_snapshot!(stdout, @r###"
+    diff --git a/file1 b/file1
+    old mode 100755
+    new mode 100644
+    index e69de29bb2..0cfbf08886
+    --- a/file1
+    +++ b/file1
+    @@ -1,0 +1,1 @@
+    +2
+    diff --git a/file2 b/file2
+    old mode 100755
+    new mode 100644
+    diff --git a/file3 b/file3
+    old mode 100644
+    new mode 100755
+    index d00491fd7e..0cfbf08886
+    --- a/file3
+    +++ b/file3
+    @@ -1,1 +1,1 @@
+    -1
+    +2
+    diff --git a/file4 b/file4
+    old mode 100644
+    new mode 100755
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-r@", "--git"]);
+    insta::assert_snapshot!(stdout, @r###"
+    diff --git a/file1 b/file1
+    deleted file mode 100644
+    index 0cfbf08886..0000000000
+    --- a/file1
+    +++ /dev/null
+    @@ -1,1 +1,0 @@
+    -2
+    diff --git a/file2 b/file2
+    deleted file mode 100644
+    index d00491fd7e..0000000000
+    --- a/file2
+    +++ /dev/null
+    @@ -1,1 +1,0 @@
+    -1
+    diff --git a/file3 b/file3
+    deleted file mode 100755
+    index 0cfbf08886..0000000000
+    --- a/file3
+    +++ /dev/null
+    @@ -1,1 +1,0 @@
+    -2
+    diff --git a/file4 b/file4
+    deleted file mode 100755
+    index e69de29bb2..0000000000
+    "###);
+}
+
 #[test]
 fn test_diff_types() {
     let test_env = TestEnvironment::default();
@@ -475,28 +607,28 @@ fn test_diff_relative_paths() {
     let stdout = test_env.jj_cmd_success(&repo_path.join("dir1"), &["diff", "--git"]);
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/dir1/file2 b/dir1/file2
-    index 54b060eee9...1fe912cdd8 100644
+    index 54b060eee9..1fe912cdd8 100644
     --- a/dir1/file2
     +++ b/dir1/file2
     @@ -1,1 +1,1 @@
     -foo2
     +bar2
     diff --git a/dir1/subdir1/file3 b/dir1/subdir1/file3
-    index c1ec6c6f12...f3c8b75ec6 100644
+    index c1ec6c6f12..f3c8b75ec6 100644
     --- a/dir1/subdir1/file3
     +++ b/dir1/subdir1/file3
     @@ -1,1 +1,1 @@
     -foo3
     +bar3
     diff --git a/dir2/file4 b/dir2/file4
-    index a0016dbc4c...17375f7a12 100644
+    index a0016dbc4c..17375f7a12 100644
     --- a/dir2/file4
     +++ b/dir2/file4
     @@ -1,1 +1,1 @@
     -foo4
     +bar4
     diff --git a/file1 b/file1
-    index 1715acd6a5...05c4fe6772 100644
+    index 1715acd6a5..05c4fe6772 100644
     --- a/file1
     +++ b/file1
     @@ -1,1 +1,1 @@
@@ -548,7 +680,7 @@ fn test_diff_missing_newline() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--git"]);
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/file1 b/file1
-    index 1910281566...a907ec3f43 100644
+    index 1910281566..a907ec3f43 100644
     --- a/file1
     +++ b/file1
     @@ -1,1 +1,2 @@
@@ -558,7 +690,7 @@ fn test_diff_missing_newline() {
     +bar
     \ No newline at end of file
     diff --git a/file2 b/file2
-    index a907ec3f43...1910281566 100644
+    index a907ec3f43..1910281566 100644
     --- a/file2
     +++ b/file2
     @@ -1,2 +1,1 @@
@@ -931,7 +1063,7 @@ fn test_diff_leading_trailing_context() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--git", "--context=5"]);
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/file1 b/file1
-    index 1bf57dee4a...69b3e1865c 100644
+    index 1bf57dee4a..69b3e1865c 100644
     --- a/file1
     +++ b/file1
     @@ -1,12 +1,12 @@
@@ -955,7 +1087,7 @@ fn test_diff_leading_trailing_context() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--git", "--context=3"]);
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/file1 b/file1
-    index 1bf57dee4a...69b3e1865c 100644
+    index 1bf57dee4a..69b3e1865c 100644
     --- a/file1
     +++ b/file1
     @@ -3,8 +3,8 @@
@@ -975,7 +1107,7 @@ fn test_diff_leading_trailing_context() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--git", "--context=2"]);
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/file1 b/file1
-    index 1bf57dee4a...69b3e1865c 100644
+    index 1bf57dee4a..69b3e1865c 100644
     --- a/file1
     +++ b/file1
     @@ -4,6 +4,6 @@
@@ -1035,12 +1167,12 @@ fn test_diff_external_tool() {
     │  --
     │  file2
     │  file3
-    ◉  qpvuntsm test.user@example.com 2001-02-03 08:05:08 0ad4ef22
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:08 0ad4ef22
     │  (no description set)
     │  --
     │  file1
     │  file2
-    ◉  zzzzzzzz root() 00000000
+    ◆  zzzzzzzz root() 00000000
        --
     "###);
 
@@ -1115,6 +1247,109 @@ fn test_diff_external_tool() {
     "###);
 }
 
+#[test]
+fn test_diff_external_file_by_file_tool() {
+    let mut test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
+    std::fs::write(repo_path.join("file2"), "foo\n").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    std::fs::remove_file(repo_path.join("file1")).unwrap();
+    std::fs::write(repo_path.join("file2"), "foo\nbar\n").unwrap();
+    std::fs::write(repo_path.join("file3"), "foo\n").unwrap();
+
+    let edit_script = test_env.set_up_fake_diff_editor();
+    std::fs::write(
+        edit_script,
+        "print ==\0print-files-before\0print --\0print-files-after",
+    )
+    .unwrap();
+
+    // Enabled by default, looks up the merge-tools table
+    let config = "--config-toml=ui.diff.tool='fake-diff-editor'\nmerge-tools.fake-diff-editor.\
+                  diff-invocation-mode='file-by-file'";
+
+    // diff without file patterns
+    insta::assert_snapshot!(
+        test_env.jj_cmd_success(&repo_path, &["diff", config]), @r###"
+    ==
+    file1
+    --
+    file1
+    ==
+    file2
+    --
+    file2
+    ==
+    file3
+    --
+    file3
+    "###);
+
+    // diff with file patterns
+    insta::assert_snapshot!(
+        test_env.jj_cmd_success(&repo_path, &["diff", config, "file1"]), @r###"
+    ==
+    file1
+    --
+    file1
+    "###);
+
+    insta::assert_snapshot!(
+        test_env.jj_cmd_success(&repo_path, &["log", "-p", config]), @r###"
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:09 39d9055d
+    │  (no description set)
+    │  ==
+    │  file1
+    │  --
+    │  file1
+    │  ==
+    │  file2
+    │  --
+    │  file2
+    │  ==
+    │  file3
+    │  --
+    │  file3
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:08 0ad4ef22
+    │  (no description set)
+    │  ==
+    │  file1
+    │  --
+    │  file1
+    │  ==
+    │  file2
+    │  --
+    │  file2
+    ◆  zzzzzzzz root() 00000000
+    "###);
+
+    insta::assert_snapshot!(
+        test_env.jj_cmd_success(&repo_path, &["show", config]), @r###"
+    Commit ID: 39d9055d70873099fd924b9af218289d5663eac8
+    Change ID: rlvkpnrzqnoowoytxnquwvuryrwnrmlp
+    Author: Test User <test.user@example.com> (2001-02-03 08:05:09)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:09)
+
+        (no description set)
+
+    ==
+    file1
+    --
+    file1
+    ==
+    file2
+    --
+    file2
+    ==
+    file3
+    --
+    file3
+    "###);
+}
+
 #[cfg(unix)]
 #[test]
 fn test_diff_external_tool_symlink() {
@@ -1328,6 +1563,25 @@ fn test_diff_binary() {
         (binary)
     "###);
 
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--git"]);
+    insta::assert_snapshot!(stdout, @r###"
+    diff --git a/file1.png b/file1.png
+    deleted file mode 100644
+    index 2b65b23c22..0000000000
+    Binary files a/file1.png and /dev/null differ
+    diff --git a/file2.png b/file2.png
+    index 7f036ce788..3bd1f0e297 100644
+    Binary files a/file2.png and b/file2.png differ
+    diff --git a/file3.png b/file3.png
+    new file mode 100644
+    index 0000000000..deacfbc286
+    Binary files /dev/null and b/file3.png differ
+    diff --git a/file4.png b/file4.png
+    new file mode 100644
+    index 0000000000..4227ca4e87
+    Binary files /dev/null and b/file4.png differ
+    "###);
+
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--stat"]);
     insta::assert_snapshot!(stdout, @r###"
     file1.png | 3 ---
