@@ -64,6 +64,7 @@ fn test_git_clone() {
     insta::assert_snapshot!(stderr, @r###"
     Fetching into new repo in "$TEST_ENV/clone"
     branch: main@origin [new] tracked
+    Setting the revset alias "trunk()" to "main@origin"
     Working copy now at: uuqppmxq 1f0b881a (empty) (no description set)
     Parent commit      : mzyxwzks 9f01a0e0 main | message
     Added 1 files, modified 0 files, removed 0 files
@@ -135,6 +136,21 @@ fn test_git_clone() {
     insta::assert_snapshot!(stderr, @r###"
     Error: Destination path exists and is not an empty directory
     "###);
+
+    // Clone into a nested path
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        test_env.env_root(),
+        &["git", "clone", "source", "nested/path/to/repo"],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Fetching into new repo in "$TEST_ENV/nested/path/to/repo"
+    branch: main@origin [new] tracked
+    Setting the revset alias "trunk()" to "main@origin"
+    Working copy now at: uuzqqzqu df8acbac (empty) (no description set)
+    Parent commit      : mzyxwzks 9f01a0e0 main | message
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
 }
 
 #[test]
@@ -175,6 +191,7 @@ fn test_git_clone_colocate() {
     insta::assert_snapshot!(stderr, @r###"
     Fetching into new repo in "$TEST_ENV/clone"
     branch: main@origin [new] tracked
+    Setting the revset alias "trunk()" to "main@origin"
     Working copy now at: uuqppmxq 1f0b881a (empty) (no description set)
     Parent commit      : mzyxwzks 9f01a0e0 main | message
     Added 1 files, modified 0 files, removed 0 files
@@ -306,6 +323,27 @@ fn test_git_clone_colocate() {
     insta::assert_snapshot!(stderr, @r###"
     Error: Destination path exists and is not an empty directory
     "###);
+
+    // Clone into a nested path
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        test_env.env_root(),
+        &[
+            "git",
+            "clone",
+            "source",
+            "nested/path/to/repo",
+            "--colocate",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Fetching into new repo in "$TEST_ENV/nested/path/to/repo"
+    branch: main@origin [new] tracked
+    Setting the revset alias "trunk()" to "main@origin"
+    Working copy now at: vzqnnsmr 9407107f (empty) (no description set)
+    Parent commit      : mzyxwzks 9f01a0e0 main | message
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
 }
 
 #[test]
@@ -332,6 +370,7 @@ fn test_git_clone_remote_default_branch() {
     Fetching into new repo in "$TEST_ENV/clone1"
     branch: feature1@origin [new] tracked
     branch: main@origin     [new] tracked
+    Setting the revset alias "trunk()" to "main@origin"
     Working copy now at: sqpuoqvx cad212e1 (empty) (no description set)
     Parent commit      : mzyxwzks 9f01a0e0 feature1 main | message
     Added 1 files, modified 0 files, removed 0 files
@@ -344,6 +383,15 @@ fn test_git_clone_remote_default_branch() {
       @origin: mzyxwzks 9f01a0e0 message
     "###);
 
+    // "trunk()" alias should be set to default branch "main"
+    let stdout = test_env.jj_cmd_success(
+        &test_env.env_root().join("clone1"),
+        &["config", "list", "--repo", "revset-aliases.'trunk()'"],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    revset-aliases.'trunk()' = "main@origin"
+    "###);
+
     // Only the default branch will be imported if auto-local-branch is off
     test_env.add_config("git.auto-local-branch = false");
     let (_stdout, stderr) =
@@ -352,7 +400,8 @@ fn test_git_clone_remote_default_branch() {
     Fetching into new repo in "$TEST_ENV/clone2"
     branch: feature1@origin [new] untracked
     branch: main@origin     [new] untracked
-    Working copy now at: pmmvwywv fa729b1e (empty) (no description set)
+    Setting the revset alias "trunk()" to "main@origin"
+    Working copy now at: rzvqmyuk cc8a5041 (empty) (no description set)
     Parent commit      : mzyxwzks 9f01a0e0 feature1@origin main | message
     Added 1 files, modified 0 files, removed 0 files
     "###);
@@ -362,6 +411,87 @@ fn test_git_clone_remote_default_branch() {
     main: mzyxwzks 9f01a0e0 message
       @origin: mzyxwzks 9f01a0e0 message
     "###);
+
+    // Change the default branch in remote
+    git_repo.set_head("refs/heads/feature1").unwrap();
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(test_env.env_root(), &["git", "clone", "source", "clone3"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Fetching into new repo in "$TEST_ENV/clone3"
+    branch: feature1@origin [new] untracked
+    branch: main@origin     [new] untracked
+    Setting the revset alias "trunk()" to "feature1@origin"
+    Working copy now at: nppvrztz b8a8a17b (empty) (no description set)
+    Parent commit      : mzyxwzks 9f01a0e0 feature1 main@origin | message
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+    insta::assert_snapshot!(
+        get_branch_output(&test_env, &test_env.env_root().join("clone2")), @r###"
+    feature1@origin: mzyxwzks 9f01a0e0 message
+    main: mzyxwzks 9f01a0e0 message
+      @origin: mzyxwzks 9f01a0e0 message
+    "###);
+
+    // "trunk()" alias should be set to new default branch "feature1"
+    let stdout = test_env.jj_cmd_success(
+        &test_env.env_root().join("clone3"),
+        &["config", "list", "--repo", "revset-aliases.'trunk()'"],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    revset-aliases.'trunk()' = "feature1@origin"
+    "###);
+}
+
+#[test]
+fn test_git_clone_ignore_working_copy() {
+    let test_env = TestEnvironment::default();
+    let git_repo_path = test_env.env_root().join("source");
+    let git_repo = git2::Repository::init(git_repo_path).unwrap();
+    set_up_non_empty_git_repo(&git_repo);
+
+    // Should not update working-copy files
+    let (_stdout, stderr) = test_env.jj_cmd_ok(
+        test_env.env_root(),
+        &["git", "clone", "--ignore-working-copy", "source", "clone"],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Fetching into new repo in "$TEST_ENV/clone"
+    branch: main@origin [new] untracked
+    Setting the revset alias "trunk()" to "main@origin"
+    "###);
+    let clone_path = test_env.env_root().join("clone");
+
+    let (stdout, stderr) = test_env.jj_cmd_ok(&clone_path, &["status", "--ignore-working-copy"]);
+    insta::assert_snapshot!(stdout, @r###"
+    The working copy is clean
+    Working copy : sqpuoqvx cad212e1 (empty) (no description set)
+    Parent commit: mzyxwzks 9f01a0e0 main | message
+    "###);
+    insta::assert_snapshot!(stderr, @"");
+
+    // TODO: Correct, but might be better to check out the root commit?
+    let stderr = test_env.jj_cmd_failure(&clone_path, &["status"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: The working copy is stale (not updated since operation b51416386f26).
+    Hint: Run `jj workspace update-stale` to update it.
+    See https://github.com/martinvonz/jj/blob/main/docs/working-copy.md#stale-working-copy for more information.
+    "###);
+}
+
+#[test]
+fn test_git_clone_at_operation() {
+    let test_env = TestEnvironment::default();
+    let git_repo_path = test_env.env_root().join("source");
+    let git_repo = git2::Repository::init(git_repo_path).unwrap();
+    set_up_non_empty_git_repo(&git_repo);
+
+    let stderr = test_env.jj_cmd_cli_error(
+        test_env.env_root(),
+        &["git", "clone", "--at-op=@-", "source", "clone"],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: --at-op is not respected
+    "###);
 }
 
 fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
