@@ -58,8 +58,8 @@ fn test_git_colocated() {
     test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  3e9369cd54227eb88455e1834dbc08aad6a16ac4
-    ◉  e61b6729ff4292870702f2f72b2a60165679ef37 master HEAD@git initial
-    ◉  0000000000000000000000000000000000000000
+    ○  e61b6729ff4292870702f2f72b2a60165679ef37 master HEAD@git initial
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(
         git_repo.head().unwrap().peel_to_commit().unwrap().id().to_string(),
@@ -71,8 +71,8 @@ fn test_git_colocated() {
     std::fs::write(workspace_root.join("file"), "modified").unwrap();
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  4f546c80f30abc0803fb83e5032a4d49fede4d68
-    ◉  e61b6729ff4292870702f2f72b2a60165679ef37 master HEAD@git initial
-    ◉  0000000000000000000000000000000000000000
+    ○  e61b6729ff4292870702f2f72b2a60165679ef37 master HEAD@git initial
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(
         git_repo.head().unwrap().peel_to_commit().unwrap().id().to_string(),
@@ -83,9 +83,9 @@ fn test_git_colocated() {
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  0e2301a42b288b9568344e32cfdd8c76d1e56a83
-    ◉  4f546c80f30abc0803fb83e5032a4d49fede4d68 HEAD@git
-    ◉  e61b6729ff4292870702f2f72b2a60165679ef37 master initial
-    ◉  0000000000000000000000000000000000000000
+    ○  4f546c80f30abc0803fb83e5032a4d49fede4d68 HEAD@git
+    ○  e61b6729ff4292870702f2f72b2a60165679ef37 master initial
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(
         git_repo.head().unwrap().target().unwrap().to_string(),
@@ -120,7 +120,7 @@ fn test_git_colocated_unborn_branch() {
     );
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Stage some change, and check out root. This shouldn't clobber the HEAD.
@@ -139,9 +139,9 @@ fn test_git_colocated_unborn_branch() {
     );
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  fcdbbd731496cae17161cd6be9b6cf1f759655a8
-    │ ◉  993600f1189571af5bbeb492cf657dc7d0fde48a
+    │ ○  993600f1189571af5bbeb492cf657dc7d0fde48a
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
     // Staged change shouldn't persist.
     checkout_index();
@@ -167,10 +167,10 @@ fn test_git_colocated_unborn_branch() {
     );
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  0e14610343ef50775f5c44db5aeef19aee45d9ad
-    ◉  e3e01407bd3539722ae4ffff077700d97c60cb11 HEAD@git
-    │ ◉  993600f1189571af5bbeb492cf657dc7d0fde48a
+    ○  e3e01407bd3539722ae4ffff077700d97c60cb11 HEAD@git
+    │ ○  993600f1189571af5bbeb492cf657dc7d0fde48a
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
     // Staged change shouldn't persist.
     checkout_index();
@@ -196,12 +196,12 @@ fn test_git_colocated_unborn_branch() {
     assert!(git_repo.head().is_err());
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  10dd328bb906e15890e55047740eab2812a3b2f7
-    │ ◉  ef75c0b0dcc9b080e00226908c21316acaa84dc6
-    │ ◉  e3e01407bd3539722ae4ffff077700d97c60cb11 master
+    │ ○  ef75c0b0dcc9b080e00226908c21316acaa84dc6
+    │ ○  e3e01407bd3539722ae4ffff077700d97c60cb11 master
     ├─╯
-    │ ◉  993600f1189571af5bbeb492cf657dc7d0fde48a
+    │ ○  993600f1189571af5bbeb492cf657dc7d0fde48a
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
     // Staged change shouldn't persist.
     checkout_index();
@@ -221,13 +221,13 @@ fn test_git_colocated_unborn_branch() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  101e272377a9daff75358f10dbd078df922fe68c
-    ◉  fc8af9345b0830dcb14716e04cd2af26e2d19f63 HEAD@git
-    │ ◉  ef75c0b0dcc9b080e00226908c21316acaa84dc6
-    │ ◉  e3e01407bd3539722ae4ffff077700d97c60cb11 master
+    ○  fc8af9345b0830dcb14716e04cd2af26e2d19f63 HEAD@git
+    │ ○  ef75c0b0dcc9b080e00226908c21316acaa84dc6
+    │ ○  e3e01407bd3539722ae4ffff077700d97c60cb11 master
     ├─╯
-    │ ◉  993600f1189571af5bbeb492cf657dc7d0fde48a
+    │ ○  993600f1189571af5bbeb492cf657dc7d0fde48a
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 }
 
@@ -246,7 +246,7 @@ fn test_git_colocated_export_branches_on_snapshot() {
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "foo"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  b15ef4cdd277d2c63cce6d67c1916f53a36141f7 foo
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // The branch gets updated when we modify the working copy, and it should get
@@ -254,7 +254,7 @@ fn test_git_colocated_export_branches_on_snapshot() {
     std::fs::write(workspace_root.join("file"), "modified").unwrap();
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  4d2c49a8f8e2f1ba61f48ba79e5f4a5faa6512cf foo
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(git_repo
         .find_reference("refs/heads/foo")
@@ -281,8 +281,8 @@ fn test_git_colocated_rebase_on_import() {
     // refs/heads/master we just exported
     test_env.jj_cmd_ok(&workspace_root, &["st"]);
 
-    // Move `master` and HEAD backwards, which should result in commit2 getting
-    // hidden, and a new working-copy commit at the new position.
+    // Move `master` backwards, which should result in commit2 getting hidden,
+    // and the working-copy commit rebased.
     let commit2_oid = git_repo
         .find_branch("master", git2::BranchType::Local)
         .unwrap()
@@ -292,16 +292,18 @@ fn test_git_colocated_rebase_on_import() {
     let commit2 = git_repo.find_commit(commit2_oid).unwrap();
     let commit1 = commit2.parents().next().unwrap();
     git_repo.branch("master", &commit1, true).unwrap();
-    git_repo.set_head("refs/heads/master").unwrap();
     let (stdout, stderr) = get_log_output_with_stderr(&test_env, &workspace_root);
     insta::assert_snapshot!(stdout, @r###"
-    @  5539e55eb3690b85a7ebd4a37a5e3b57f469ee94
-    ◉  47fe984daf66f7bf3ebf31b9cb3513c995afb857 master HEAD@git add a file
-    ◉  0000000000000000000000000000000000000000
+    @  15b1d70c5e33b5d2b18383292b85324d5153ffed
+    ○  47fe984daf66f7bf3ebf31b9cb3513c995afb857 master HEAD@git add a file
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(stderr, @r###"
-    Reset the working copy parent to the new Git HEAD.
     Abandoned 1 commits that are no longer reachable.
+    Rebased 1 descendant commits off of commits rewritten from git
+    Working copy now at: zsuskuln 15b1d70c (empty) (no description set)
+    Parent commit      : qpvuntsm 47fe984d master | add a file
+    Added 0 files, modified 1 files, removed 0 files
     Done importing changes from the underlying Git repo.
     "###);
 }
@@ -316,10 +318,10 @@ fn test_git_colocated_branches() {
     test_env.jj_cmd_ok(&workspace_root, &["new", "@-", "-m", "bar"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  3560559274ab431feea00b7b7e0b9250ecce951f bar
-    │ ◉  1e6f0b403ed2ff9713b5d6b1dc601e4804250cda foo
+    │ ○  1e6f0b403ed2ff9713b5d6b1dc601e4804250cda foo
     ├─╯
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
-    ◉  0000000000000000000000000000000000000000
+    ○  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Create a branch in jj. It should be exported to Git even though it points to
@@ -350,10 +352,10 @@ fn test_git_colocated_branches() {
     let (stdout, stderr) = get_log_output_with_stderr(&test_env, &workspace_root);
     insta::assert_snapshot!(stdout, @r###"
     @  096dc80da67094fbaa6683e2a205dddffa31f9a8
-    │ ◉  1e6f0b403ed2ff9713b5d6b1dc601e4804250cda master foo
+    │ ○  1e6f0b403ed2ff9713b5d6b1dc601e4804250cda master foo
     ├─╯
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
-    ◉  0000000000000000000000000000000000000000
+    ○  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(stderr, @r###"
     Abandoned 1 commits that are no longer reachable.
@@ -373,8 +375,8 @@ fn test_git_colocated_branch_forget() {
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "foo"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  65b6b74e08973b88d38404430f119c8c79465250 foo
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
-    ◉  0000000000000000000000000000000000000000
+    ○  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     foo: rlvkpnrz 65b6b74e (empty) (no description set)
@@ -383,12 +385,44 @@ fn test_git_colocated_branch_forget() {
 
     let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["branch", "forget", "foo"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Forgot 1 branches.
+    "###);
     // A forgotten branch is deleted in the git repo. For a detailed demo explaining
     // this, see `test_branch_forget_export` in `test_branch_command.rs`.
     insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @"");
 }
 
+#[test]
+fn test_git_colocated_branch_at_root() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "--colocate", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo", "-r=root()"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Created 1 branches pointing to zzzzzzzz 00000000 foo | (empty) (no description set)
+    Warning: Failed to export some branches:
+      foo: Ref cannot point to the root commit in Git
+    "###);
+
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "move", "foo"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to qpvuntsm 230dd059 foo | (empty) (no description set)
+    "###);
+
+    let (_stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &["branch", "move", "foo", "--allow-backwards", "--to=root()"],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Moved 1 branches to zzzzzzzz 00000000 foo* | (empty) (no description set)
+    Warning: Failed to export some branches:
+      foo: Ref cannot point to the root commit in Git
+    "###);
+}
+
 #[test]
 fn test_git_colocated_conflicting_git_refs() {
     let test_env = TestEnvironment::default();
@@ -400,6 +434,7 @@ fn test_git_colocated_conflicting_git_refs() {
     insta::assert_snapshot!(stdout, @"");
     insta::with_settings!({filters => vec![("Failed to set: .*", "Failed to set: ...")]}, {
         insta::assert_snapshot!(stderr, @r###"
+        Created 1 branches pointing to qpvuntsm 230dd059 main main/sub | (empty) (no description set)
         Warning: Failed to export some branches:
           main/sub: Failed to set: ...
         Hint: Git doesn't allow a branch name that looks like a parent directory of
@@ -464,10 +499,10 @@ fn test_git_colocated_checkout_non_empty_working_copy() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  149cc31cb08a1589e6c5ee2cb2061559dc758ecb new
-    │ ◉  4ec6f6506bd1903410f15b80058a7f0d8f62deea two
+    │ ○  4ec6f6506bd1903410f15b80058a7f0d8f62deea two
     ├─╯
-    ◉  e61b6729ff4292870702f2f72b2a60165679ef37 master HEAD@git initial
-    ◉  0000000000000000000000000000000000000000
+    ○  e61b6729ff4292870702f2f72b2a60165679ef37 master HEAD@git initial
+    ◆  0000000000000000000000000000000000000000
     "###);
 }
 
@@ -491,12 +526,12 @@ fn test_git_colocated_fetch_deleted_or_moved_branch() {
     test_env.jj_cmd_ok(&clone_path, &["new", "A"]);
     insta::assert_snapshot!(get_log_output(&test_env, &clone_path), @r###"
     @  9c2de797c3c299a40173c5af724329012b77cbdd
-    │ ◉  4a191a9013d3f3398ccf5e172792a61439dbcf3a C_to_move original C
+    │ ○  4a191a9013d3f3398ccf5e172792a61439dbcf3a C_to_move original C
     ├─╯
-    │ ◉  c49ec4fb50844d0e693f1609da970b11878772ee B_to_delete B_to_delete
+    │ ○  c49ec4fb50844d0e693f1609da970b11878772ee B_to_delete B_to_delete
     ├─╯
-    ◉  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 A HEAD@git A
-    ◉  0000000000000000000000000000000000000000
+    ◆  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 A HEAD@git A
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     test_env.jj_cmd_ok(&origin_path, &["branch", "delete", "B_to_delete"]);
@@ -512,11 +547,11 @@ fn test_git_colocated_fetch_deleted_or_moved_branch() {
     // "original C" and "B_to_delete" are abandoned, as the corresponding branches
     // were deleted or moved on the remote (#864)
     insta::assert_snapshot!(get_log_output(&test_env, &clone_path), @r###"
-    ◉  4f3d13296f978cbc351c46a43b4619c91b888475 C_to_move moved C
+    ○  4f3d13296f978cbc351c46a43b4619c91b888475 C_to_move moved C
     │ @  9c2de797c3c299a40173c5af724329012b77cbdd
     ├─╯
-    ◉  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 A HEAD@git A
-    ◉  0000000000000000000000000000000000000000
+    ◆  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 A HEAD@git A
+    ◆  0000000000000000000000000000000000000000
     "###);
 }
 
@@ -559,8 +594,8 @@ fn test_git_colocated_rebase_dirty_working_copy() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  6bad94b10401f5fafc8a91064661224650d10d1b feature??
-    ◉  3230d52258f6de7e9afbd10da8d64503cc7cdca5 HEAD@git
-    ◉  0000000000000000000000000000000000000000
+    ○  3230d52258f6de7e9afbd10da8d64503cc7cdca5 HEAD@git
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // The working-copy content shouldn't be lost.
@@ -588,10 +623,10 @@ fn test_git_colocated_external_checkout() {
     // Checked out anonymous branch
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  f8a23336e41840ed1757ef323402a770427dc89a
-    ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a HEAD@git B
-    │ ◉  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master A
+    ○  eccedddfa5152d99fc8ddd1081b375387a8a382a HEAD@git B
+    │ ○  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master A
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Check out another branch by external command
@@ -602,10 +637,10 @@ fn test_git_colocated_external_checkout() {
     let (stdout, stderr) = get_log_output_with_stderr(&test_env, &repo_path);
     insta::assert_snapshot!(stdout, @r###"
     @  8bb9e8d42a37c2a4e8dcfad97fce0b8f49bc7afa
-    ◉  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master HEAD@git A
-    │ ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a B
+    ○  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master HEAD@git A
+    │ ○  eccedddfa5152d99fc8ddd1081b375387a8a382a B
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(stderr, @r###"
     Reset the working copy parent to the new Git HEAD.
@@ -615,12 +650,12 @@ fn test_git_colocated_external_checkout() {
     test_env.jj_cmd_ok(&repo_path, &["new", "description(B)"]);
     test_env.jj_cmd_ok(&repo_path, &["new", "-m=C", "--no-edit"]);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    ◉  99a813753d6db988d8fc436b0d6b30a54d6b2707 C
+    ○  99a813753d6db988d8fc436b0d6b30a54d6b2707 C
     @  81e086b7f9b1dd7fde252e28bdcf4ba4abd86ce5
-    ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a HEAD@git B
-    │ ◉  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master A
+    ○  eccedddfa5152d99fc8ddd1081b375387a8a382a HEAD@git B
+    │ ○  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master A
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Check out another branch by external command
@@ -630,12 +665,12 @@ fn test_git_colocated_external_checkout() {
     let (stdout, stderr) = get_log_output_with_stderr(&test_env, &repo_path);
     insta::assert_snapshot!(stdout, @r###"
     @  ca2a4e32f08688c6fb795c4c034a0a7e09c0d804
-    ◉  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master HEAD@git A
-    │ ◉  99a813753d6db988d8fc436b0d6b30a54d6b2707 C
-    │ ◉  81e086b7f9b1dd7fde252e28bdcf4ba4abd86ce5
-    │ ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a B
+    ○  a7e4cec4256b7995129b9d1e1bda7e1df6e60678 master HEAD@git A
+    │ ○  99a813753d6db988d8fc436b0d6b30a54d6b2707 C
+    │ ○  81e086b7f9b1dd7fde252e28bdcf4ba4abd86ce5
+    │ ○  eccedddfa5152d99fc8ddd1081b375387a8a382a B
     ├─╯
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(stderr, @r###"
     Reset the working copy parent to the new Git HEAD.
@@ -652,23 +687,23 @@ fn test_git_colocated_squash_undo() {
     // Test the setup
     insta::assert_snapshot!(get_log_output_divergence(&test_env, &repo_path), @r###"
     @  rlvkpnrzqnoo 9670380ac379
-    ◉  qpvuntsmwlqt a7e4cec4256b A HEAD@git
-    ◉  zzzzzzzzzzzz 000000000000
+    ○  qpvuntsmwlqt a7e4cec4256b A HEAD@git
+    ◆  zzzzzzzzzzzz 000000000000
     "###);
 
     test_env.jj_cmd_ok(&repo_path, &["squash"]);
     insta::assert_snapshot!(get_log_output_divergence(&test_env, &repo_path), @r###"
     @  zsuskulnrvyr 6ee662324e5a
-    ◉  qpvuntsmwlqt 13ab6b96d82e A HEAD@git
-    ◉  zzzzzzzzzzzz 000000000000
+    ○  qpvuntsmwlqt 13ab6b96d82e A HEAD@git
+    ◆  zzzzzzzzzzzz 000000000000
     "###);
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
     // TODO: There should be no divergence here; 2f376ea1478c should be hidden
     // (#922)
     insta::assert_snapshot!(get_log_output_divergence(&test_env, &repo_path), @r###"
     @  rlvkpnrzqnoo 9670380ac379
-    ◉  qpvuntsmwlqt a7e4cec4256b A HEAD@git
-    ◉  zzzzzzzzzzzz 000000000000
+    ○  qpvuntsmwlqt a7e4cec4256b A HEAD@git
+    ◆  zzzzzzzzzzzz 000000000000
     "###);
 }
 
@@ -686,8 +721,8 @@ fn test_git_colocated_undo_head_move() {
         @"230dd059e1b059aefc0da06a2e5a7dbf22362f22");
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  65b6b74e08973b88d38404430f119c8c79465250
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
-    ◉  0000000000000000000000000000000000000000
+    ○  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // HEAD should be unset
@@ -695,7 +730,7 @@ fn test_git_colocated_undo_head_move() {
     assert!(git_repo.head().is_err());
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Create commit on non-root commit
@@ -703,9 +738,9 @@ fn test_git_colocated_undo_head_move() {
     test_env.jj_cmd_ok(&repo_path, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  69b19f73cf584f162f078fb0d91c55ca39d10bc7
-    ◉  eb08b363bb5ef8ee549314260488980d7bbe8f63 HEAD@git
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ○  eb08b363bb5ef8ee549314260488980d7bbe8f63 HEAD@git
+    ○  230dd059e1b059aefc0da06a2e5a7dbf22362f22
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(
         git_repo.head().unwrap().target().unwrap().to_string(),
@@ -723,8 +758,8 @@ fn test_git_colocated_undo_head_move() {
         @"230dd059e1b059aefc0da06a2e5a7dbf22362f22");
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  eb08b363bb5ef8ee549314260488980d7bbe8f63
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
-    ◉  0000000000000000000000000000000000000000
+    ○  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
+    ◆  0000000000000000000000000000000000000000
     "###);
 }
 
@@ -812,8 +847,8 @@ fn test_git_colocated_unreachable_commits() {
     test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  66ae47cee4f8c28ee8d7e4f5d9401b03c07e22f2
-    ◉  2ee37513d2b5e549f7478c671a780053614bff19 master HEAD@git initial
-    ◉  0000000000000000000000000000000000000000
+    ○  2ee37513d2b5e549f7478c671a780053614bff19 master HEAD@git initial
+    ◆  0000000000000000000000000000000000000000
     "###);
     insta::assert_snapshot!(
         git_repo.head().unwrap().peel_to_commit().unwrap().id().to_string(),
