@@ -94,6 +94,32 @@ fn test_git_init_internal() {
     assert_eq!(read_git_target(&workspace_root), "git");
 }
 
+#[test]
+fn test_git_init_internal_ignore_working_copy() {
+    let test_env = TestEnvironment::default();
+    let workspace_root = test_env.env_root().join("repo");
+    std::fs::create_dir(&workspace_root).unwrap();
+    std::fs::write(workspace_root.join("file1"), "").unwrap();
+
+    let stderr =
+        test_env.jj_cmd_cli_error(&workspace_root, &["git", "init", "--ignore-working-copy"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: --ignore-working-copy is not respected
+    "###);
+}
+
+#[test]
+fn test_git_init_internal_at_operation() {
+    let test_env = TestEnvironment::default();
+    let workspace_root = test_env.env_root().join("repo");
+    std::fs::create_dir(&workspace_root).unwrap();
+
+    let stderr = test_env.jj_cmd_cli_error(&workspace_root, &["git", "init", "--at-op=@-"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: --at-op is not respected
+    "###);
+}
+
 #[test_case(false; "full")]
 #[test_case(true; "bare")]
 fn test_git_init_external(bare: bool) {
@@ -142,12 +168,119 @@ fn test_git_init_external(bare: bool) {
     insta::allow_duplicates! {
         insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
         @  f6950fc115ae
-        ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
-        ◉  000000000000
+        ○  8d698d4a8ee1 my-branch HEAD@git My commit message
+        ◆  000000000000
         "###);
     }
 }
 
+#[test_case(false; "full")]
+#[test_case(true; "bare")]
+fn test_git_init_external_import_trunk(bare: bool) {
+    let test_env = TestEnvironment::default();
+    let git_repo_path = test_env.env_root().join("git-repo");
+    let git_repo = init_git_repo(&git_repo_path, bare);
+
+    // Add remote branch "trunk" for remote "origin", and set it as "origin/HEAD"
+    let oid = git_repo
+        .find_reference("refs/heads/my-branch")
+        .unwrap()
+        .target()
+        .unwrap();
+    git_repo
+        .reference("refs/remotes/origin/trunk", oid, false, "")
+        .unwrap();
+    git_repo
+        .reference_symbolic(
+            "refs/remotes/origin/HEAD",
+            "refs/remotes/origin/trunk",
+            false,
+            "",
+        )
+        .unwrap();
+
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        test_env.env_root(),
+        &[
+            "git",
+            "init",
+            "repo",
+            "--git-repo",
+            git_repo_path.to_str().unwrap(),
+        ],
+    );
+    insta::allow_duplicates! {
+        insta::assert_snapshot!(stdout, @"");
+        insta::assert_snapshot!(stderr, @r###"
+        Done importing changes from the underlying Git repo.
+        Setting the revset alias "trunk()" to "trunk@origin"
+        Working copy now at: sqpuoqvx f6950fc1 (empty) (no description set)
+        Parent commit      : mwrttmos 8d698d4a my-branch trunk@origin | My commit message
+        Added 1 files, modified 0 files, removed 0 files
+        Initialized repo in "repo"
+        "###);
+    }
+
+    // "trunk()" alias should be set to remote "origin"'s default branch "trunk"
+    let stdout = test_env.jj_cmd_success(
+        &test_env.env_root().join("repo"),
+        &["config", "list", "--repo", "revset-aliases.\"trunk()\""],
+    );
+    insta::allow_duplicates! {
+    insta::assert_snapshot!(stdout, @r###"
+    revset-aliases."trunk()" = "trunk@origin"
+    "###);
+    }
+}
+
+#[test]
+fn test_git_init_external_ignore_working_copy() {
+    let test_env = TestEnvironment::default();
+    let git_repo_path = test_env.env_root().join("git-repo");
+    init_git_repo(&git_repo_path, false);
+    let workspace_root = test_env.env_root().join("repo");
+    std::fs::create_dir(&workspace_root).unwrap();
+    std::fs::write(workspace_root.join("file1"), "").unwrap();
+
+    // No snapshot should be taken
+    let stderr = test_env.jj_cmd_cli_error(
+        &workspace_root,
+        &[
+            "git",
+            "init",
+            "--ignore-working-copy",
+            "--git-repo",
+            git_repo_path.to_str().unwrap(),
+        ],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: --ignore-working-copy is not respected
+    "###);
+}
+
+#[test]
+fn test_git_init_external_at_operation() {
+    let test_env = TestEnvironment::default();
+    let git_repo_path = test_env.env_root().join("git-repo");
+    init_git_repo(&git_repo_path, false);
+    let workspace_root = test_env.env_root().join("repo");
+    std::fs::create_dir(&workspace_root).unwrap();
+
+    let stderr = test_env.jj_cmd_cli_error(
+        &workspace_root,
+        &[
+            "git",
+            "init",
+            "--at-op=@-",
+            "--git-repo",
+            git_repo_path.to_str().unwrap(),
+        ],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: --at-op is not respected
+    "###);
+}
+
 #[test]
 fn test_git_init_external_non_existent_directory() {
     let test_env = TestEnvironment::default();
@@ -209,17 +342,17 @@ fn test_git_init_colocated_via_git_repo_path() {
     // Check that the Git repo's HEAD got checked out
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  f61b77cd4bb5
-    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
-    ◉  000000000000
+    ○  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◆  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f1c7aa7c62d8
-    ◉  f61b77cd4bb5 HEAD@git
-    ◉  8d698d4a8ee1 my-branch My commit message
-    ◉  000000000000
+    ○  f61b77cd4bb5 HEAD@git
+    ○  8d698d4a8ee1 my-branch My commit message
+    ◆  000000000000
     "###);
 }
 
@@ -245,17 +378,17 @@ fn test_git_init_colocated_via_git_repo_path_gitlink() {
     // Check that the Git repo's HEAD got checked out
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f61b77cd4bb5
-    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
-    ◉  000000000000
+    ○  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◆  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f1c7aa7c62d8
-    ◉  f61b77cd4bb5 HEAD@git
-    ◉  8d698d4a8ee1 my-branch My commit message
-    ◉  000000000000
+    ○  f61b77cd4bb5 HEAD@git
+    ○  8d698d4a8ee1 my-branch My commit message
+    ◆  000000000000
     "###);
 }
 
@@ -280,17 +413,17 @@ fn test_git_init_colocated_via_git_repo_path_symlink_directory() {
     // Check that the Git repo's HEAD got checked out
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f61b77cd4bb5
-    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
-    ◉  000000000000
+    ○  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◆  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f1c7aa7c62d8
-    ◉  f61b77cd4bb5 HEAD@git
-    ◉  8d698d4a8ee1 my-branch My commit message
-    ◉  000000000000
+    ○  f61b77cd4bb5 HEAD@git
+    ○  8d698d4a8ee1 my-branch My commit message
+    ◆  000000000000
     "###);
 }
 
@@ -318,17 +451,17 @@ fn test_git_init_colocated_via_git_repo_path_symlink_directory_without_bare_conf
     // Check that the Git repo's HEAD got checked out
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f61b77cd4bb5
-    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
-    ◉  000000000000
+    ○  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◆  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f1c7aa7c62d8
-    ◉  f61b77cd4bb5 HEAD@git
-    ◉  8d698d4a8ee1 my-branch My commit message
-    ◉  000000000000
+    ○  f61b77cd4bb5 HEAD@git
+    ○  8d698d4a8ee1 my-branch My commit message
+    ◆  000000000000
     "###);
 }
 
@@ -358,17 +491,17 @@ fn test_git_init_colocated_via_git_repo_path_symlink_gitlink() {
     // Check that the Git repo's HEAD got checked out
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f61b77cd4bb5
-    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
-    ◉  000000000000
+    ○  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◆  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f1c7aa7c62d8
-    ◉  f61b77cd4bb5 HEAD@git
-    ◉  8d698d4a8ee1 my-branch My commit message
-    ◉  000000000000
+    ○  f61b77cd4bb5 HEAD@git
+    ○  8d698d4a8ee1 my-branch My commit message
+    ◆  000000000000
     "###);
 }
 
@@ -485,10 +618,10 @@ fn test_git_init_colocated_dirty_working_copy() {
     │  A new-staged-file
     │  M some-file
     │  A unstaged-file
-    ◉  mwrttmos git.user@example.com 1970-01-01 11:02:03 my-branch HEAD@git 8d698d4a
+    ○  mwrttmos git.user@example.com 1970-01-01 11:02:03 my-branch HEAD@git 8d698d4a
     │  My commit message
     │  A some-file
-    ◉  zzzzzzzz root() 00000000
+    ◆  zzzzzzzz root() 00000000
     "###);
 
     // Git index should be consistent with the working copy parent. With the
@@ -505,6 +638,37 @@ fn test_git_init_colocated_dirty_working_copy() {
     "###);
 }
 
+#[test]
+fn test_git_init_colocated_ignore_working_copy() {
+    let test_env = TestEnvironment::default();
+    let workspace_root = test_env.env_root().join("repo");
+    init_git_repo(&workspace_root, false);
+    std::fs::write(workspace_root.join("file1"), "").unwrap();
+
+    let stderr = test_env.jj_cmd_cli_error(
+        &workspace_root,
+        &["git", "init", "--ignore-working-copy", "--colocate"],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: --ignore-working-copy is not respected
+    "###);
+}
+
+#[test]
+fn test_git_init_colocated_at_operation() {
+    let test_env = TestEnvironment::default();
+    let workspace_root = test_env.env_root().join("repo");
+    init_git_repo(&workspace_root, false);
+
+    let stderr = test_env.jj_cmd_cli_error(
+        &workspace_root,
+        &["git", "init", "--at-op=@-", "--colocate"],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: --at-op is not respected
+    "###);
+}
+
 #[test]
 fn test_git_init_external_but_git_dir_exists() {
     let test_env = TestEnvironment::default();
@@ -524,15 +688,15 @@ fn test_git_init_external_but_git_dir_exists() {
     // The local ".git" repository is unrelated, so no commits should be imported
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  230dd059e1b0
-    ◉  000000000000
+    ◆  000000000000
     "###);
 
     // Check that Git HEAD is not set because this isn't a colocated repo
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  4db490c88528
-    ◉  230dd059e1b0
-    ◉  000000000000
+    ○  230dd059e1b0
+    ◆  000000000000
     "###);
 }
 
@@ -553,17 +717,17 @@ fn test_git_init_colocated_via_flag_git_dir_exists() {
     // Check that the Git repo's HEAD got checked out
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f61b77cd4bb5
-    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
-    ◉  000000000000
+    ○  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◆  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  f1c7aa7c62d8
-    ◉  f61b77cd4bb5 HEAD@git
-    ◉  8d698d4a8ee1 my-branch My commit message
-    ◉  000000000000
+    ○  f61b77cd4bb5 HEAD@git
+    ○  8d698d4a8ee1 my-branch My commit message
+    ◆  000000000000
     "###);
 }
 
@@ -580,7 +744,7 @@ fn test_git_init_colocated_via_flag_git_dir_not_exists() {
     // No HEAD@git ref is available yet
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  230dd059e1b0
-    ◉  000000000000
+    ◆  000000000000
     "###);
 
     // Create the default branch (create both in case we change the default)
@@ -590,7 +754,7 @@ fn test_git_init_colocated_via_flag_git_dir_not_exists() {
     // be created on top.
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  230dd059e1b0 main master
-    ◉  000000000000
+    ◆  000000000000
     "###);
 }
 
