@@ -101,7 +101,7 @@ fn test_gitignores_ignored_file_in_target_commit() {
     );
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/ignored b/ignored
-    index 8a69467466...4d9be5127b 100644
+    index 8a69467466..4d9be5127b 100644
     --- a/ignored
     +++ b/ignored
     @@ -1,1 +1,1 @@
