@@ -125,7 +125,7 @@ fn test_ignore_working_copy() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
     @  b15ef4cdd277d2c63cce6d67c1916f53a36141f7
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Modify the file. With --ignore-working-copy, we still get the same commit
@@ -141,7 +141,7 @@ fn test_ignore_working_copy() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
     @  4d2c49a8f8e2f1ba61f48ba79e5f4a5faa6512cf
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 }
 
@@ -233,8 +233,6 @@ fn test_bad_path() {
     let subdir = repo_path.join("dir");
     std::fs::create_dir_all(&subdir).unwrap();
 
-    test_env.add_config("ui.allow-filesets = true");
-
     // cwd == workspace_root
     let stderr = test_env.jj_cmd_failure(&repo_path, &["file", "show", "../out"]);
     insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
@@ -351,30 +349,30 @@ fn test_color_config() {
     // Test that --color=always is respected.
     let stdout = test_env.jj_cmd_success(&repo_path, &["--color=always", "log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  [38;5;4m230dd059e1b059aefc0da06a2e5a7dbf22362f22[39m
-    ◉  [38;5;4m0000000000000000000000000000000000000000[39m
+    [1m[38;5;2m@[0m  [38;5;4m230dd059e1b059aefc0da06a2e5a7dbf22362f22[39m
+    [1m[38;5;14m◆[0m  [38;5;4m0000000000000000000000000000000000000000[39m
     "###);
 
     // Test that color is used if it's requested in the config file
     test_env.add_config(r#"ui.color="always""#);
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  [38;5;4m230dd059e1b059aefc0da06a2e5a7dbf22362f22[39m
-    ◉  [38;5;4m0000000000000000000000000000000000000000[39m
+    [1m[38;5;2m@[0m  [38;5;4m230dd059e1b059aefc0da06a2e5a7dbf22362f22[39m
+    [1m[38;5;14m◆[0m  [38;5;4m0000000000000000000000000000000000000000[39m
     "###);
 
     // Test that --color=never overrides the config.
     let stdout = test_env.jj_cmd_success(&repo_path, &["--color=never", "log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Test that --color=auto overrides the config.
     let stdout = test_env.jj_cmd_success(&repo_path, &["--color=auto", "log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Test that --config-toml 'ui.color="never"' overrides the config.
@@ -390,7 +388,7 @@ fn test_color_config() {
     );
     insta::assert_snapshot!(stdout, @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // --color overrides --config-toml 'ui.color=...'.
@@ -408,15 +406,15 @@ fn test_color_config() {
     );
     insta::assert_snapshot!(stdout, @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 
     // Test that NO_COLOR does NOT override the request for color in the config file
     test_env.add_env_var("NO_COLOR", "");
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  [38;5;4m230dd059e1b059aefc0da06a2e5a7dbf22362f22[39m
-    ◉  [38;5;4m0000000000000000000000000000000000000000[39m
+    [1m[38;5;2m@[0m  [38;5;4m230dd059e1b059aefc0da06a2e5a7dbf22362f22[39m
+    [1m[38;5;14m◆[0m  [38;5;4m0000000000000000000000000000000000000000[39m
     "###);
 
     // Test that per-repo config overrides the user config.
@@ -428,7 +426,7 @@ fn test_color_config() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "commit_id"]);
     insta::assert_snapshot!(stdout, @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000
     "###);
 }
 
@@ -450,8 +448,16 @@ fn test_color_ui_messages() {
     // error source
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", ".."]);
     insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
-    [1m[38;5;1mError: [39mPath ".." is not in the repo "."[0m
-    [1m[39mCaused by: [0m[39mInvalid component ".." in repo-relative path "../"[39m
+    [1m[38;5;1mError: [39mFailed to parse fileset: Invalid file pattern[0m
+    [1m[39mCaused by:[0m
+    [1m[39m1: [0m[39m --> 1:1[39m
+    [39m  |[39m
+    [39m1 | ..[39m
+    [39m  | ^^[39m
+    [39m  |[39m
+    [39m  = Invalid file pattern[39m
+    [1m[39m2: [0m[39mPath ".." is not in the repo "."[39m
+    [1m[39m3: [0m[39mInvalid component ".." in repo-relative path "../"[39m
     "###);
 
     // warning
@@ -490,9 +496,9 @@ fn test_color_ui_messages() {
     // debugging colors
     let (stdout, _stderr) = test_env.jj_cmd_ok(&repo_path, &["st", "--color", "debug"]);
     insta::assert_snapshot!(stdout, @r###"
-    <<::The working copy is clean>>
-    <<::Working copy : >>[1m[38;5;13m<<working_copy change_id shortest prefix::m>>[38;5;8m<<working_copy change_id shortest rest::zvwutvl>>[39m<<working_copy:: >>[38;5;12m<<working_copy commit_id shortest prefix::1>>[38;5;8m<<working_copy commit_id shortest rest::67f90e7>>[39m<<working_copy:: >>[38;5;10m<<working_copy empty::(empty)>>[39m<<working_copy:: >>[38;5;10m<<working_copy empty description placeholder::(no description set)>>[0m<<::>>
-    <<::Parent commit: >>[1m[38;5;5m<<change_id shortest prefix::q>>[0m[38;5;8m<<change_id shortest rest::pvuntsm>>[39m<<:: >>[1m[38;5;4m<<commit_id shortest prefix::2>>[0m[38;5;8m<<commit_id shortest rest::30dd059>>[39m<<:: >>[38;5;2m<<empty::(empty)>>[39m<<:: >>[38;5;2m<<empty description placeholder::(no description set)>>[39m<<::>>
+    The working copy is clean
+    Working copy : [1m[38;5;13m<<working_copy change_id shortest prefix::m>>[38;5;8m<<working_copy change_id shortest rest::zvwutvl>>[39m<<working_copy:: >>[38;5;12m<<working_copy commit_id shortest prefix::1>>[38;5;8m<<working_copy commit_id shortest rest::67f90e7>>[39m<<working_copy:: >>[38;5;10m<<working_copy empty::(empty)>>[39m<<working_copy:: >>[38;5;10m<<working_copy empty description placeholder::(no description set)>>[0m
+    Parent commit: [1m[38;5;5m<<change_id shortest prefix::q>>[0m[38;5;8m<<change_id shortest rest::pvuntsm>>[39m [1m[38;5;4m<<commit_id shortest prefix::2>>[0m[38;5;8m<<commit_id shortest rest::30dd059>>[39m [38;5;2m<<empty::(empty)>>[39m [38;5;2m<<empty description placeholder::(no description set)>>[39m
     "###);
 }
 
@@ -601,7 +607,7 @@ fn test_help() {
       -R, --repository <REPOSITORY>      Path to repository to operate on
           --ignore-working-copy          Don't snapshot the working copy, and don't update it
           --ignore-immutable             Allow rewriting immutable commits
-          --at-operation <AT_OPERATION>  Operation to load the repo at [default: @] [aliases: at-op]
+          --at-operation <AT_OPERATION>  Operation to load the repo at [aliases: at-op]
           --debug                        Enable debug logging
           --color <WHEN>                 When to colorize output (always, never, debug, auto)
           --quiet                        Silence non-primary command output
