@@ -67,7 +67,7 @@ fn test_interdiff_basic() {
     );
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/file2 b/file2
-    index 257cc5642c...3bd1f0e297 100644
+    index 257cc5642c..3bd1f0e297 100644
     --- a/file2
     +++ b/file2
     @@ -1,1 +1,2 @@
@@ -149,7 +149,7 @@ fn test_interdiff_conflicting() {
     );
     insta::assert_snapshot!(stdout, @r###"
     diff --git a/file b/file
-    index 0000000000...24c5735c3e 100644
+    index 0000000000..24c5735c3e 100644
     --- a/file
     +++ b/file
     @@ -1,7 +1,1 @@
