@@ -71,8 +71,8 @@ fn test_log_with_or_without_diff() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "description"]);
     insta::assert_snapshot!(stdout, @r###"
     @  a new commit
-    ◉  add a file
-    ◉
+    ○  add a file
+    ◆
     "###);
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "description", "-p"]);
@@ -81,10 +81,10 @@ fn test_log_with_or_without_diff() {
     │  Modified regular file file1:
     │     1    1: foo
     │          2: bar
-    ◉  add a file
+    ○  add a file
     │  Added regular file file1:
     │          1: foo
-    ◉
+    ◆
     "###);
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "description", "--no-graph"]);
@@ -101,11 +101,11 @@ fn test_log_with_or_without_diff() {
     │  Modified regular file file1:
     │     1    1: foo
     │          2: bar
-    ◉  add a file
+    ○  add a file
     │  A file1
     │  Added regular file file1:
     │          1: foo
-    ◉
+    ◆
     "###);
 
     // `-s` for summary, `--git` for git diff (which implies `-p`)
@@ -114,13 +114,13 @@ fn test_log_with_or_without_diff() {
     @  a new commit
     │  M file1
     │  diff --git a/file1 b/file1
-    │  index 257cc5642c...3bd1f0e297 100644
+    │  index 257cc5642c..3bd1f0e297 100644
     │  --- a/file1
     │  +++ b/file1
     │  @@ -1,1 +1,2 @@
     │   foo
     │  +bar
-    ◉  add a file
+    ○  add a file
     │  A file1
     │  diff --git a/file1 b/file1
     │  new file mode 100644
@@ -129,7 +129,7 @@ fn test_log_with_or_without_diff() {
     │  +++ b/file1
     │  @@ -1,0 +1,1 @@
     │  +foo
-    ◉
+    ◆
     "###);
 
     // `-p` enables default "summary" output, so `-s` is noop
@@ -147,9 +147,9 @@ fn test_log_with_or_without_diff() {
     insta::assert_snapshot!(stdout, @r###"
     @  a new commit
     │  M file1
-    ◉  add a file
+    ○  add a file
     │  A file1
-    ◉
+    ◆
     "###);
 
     // `-p` enables default "color-words" diff output, so `--color-words` is noop
@@ -162,10 +162,10 @@ fn test_log_with_or_without_diff() {
     │  Modified regular file file1:
     │     1    1: foo
     │          2: bar
-    ◉  add a file
+    ○  add a file
     │  Added regular file file1:
     │          1: foo
-    ◉
+    ◆
     "###);
 
     // `--git` enables git diff, so `-p` is noop
@@ -176,7 +176,7 @@ fn test_log_with_or_without_diff() {
     insta::assert_snapshot!(stdout, @r###"
     a new commit
     diff --git a/file1 b/file1
-    index 257cc5642c...3bd1f0e297 100644
+    index 257cc5642c..3bd1f0e297 100644
     --- a/file1
     +++ b/file1
     @@ -1,1 +1,2 @@
@@ -218,9 +218,9 @@ fn test_log_with_or_without_diff() {
     insta::assert_snapshot!(stdout, @r###"
     @  a new commit
     │  M file1
-    ◉  add a file
+    ○  add a file
     │  A file1
-    ◉
+    ◆
     "###);
     let stdout = test_env.jj_cmd_success(
         &repo_path,
@@ -241,7 +241,7 @@ fn test_log_with_or_without_diff() {
     insta::assert_snapshot!(stdout, @r###"
     @  a new commit
     │  diff --git a/file1 b/file1
-    ~  index 257cc5642c...3bd1f0e297 100644
+    ~  index 257cc5642c..3bd1f0e297 100644
        --- a/file1
        +++ b/file1
        @@ -1,1 +1,2 @@
@@ -255,7 +255,7 @@ fn test_log_with_or_without_diff() {
     insta::assert_snapshot!(stdout, @r###"
     a new commit
     diff --git a/file1 b/file1
-    index 257cc5642c...3bd1f0e297 100644
+    index 257cc5642c..3bd1f0e297 100644
     --- a/file1
     +++ b/file1
     @@ -1,1 +1,2 @@
@@ -505,7 +505,7 @@ fn test_log_prefix_highlight_styled() {
     insta::assert_snapshot!(
         test_env.jj_cmd_success(&repo_path, &["log", "-r", "original", "-T", &prefix_format(Some(12))]),
         @r###"
-    ◉  Change qpvuntsmwlqt initial e0e22b9fae75 original
+    ○  Change qpvuntsmwlqt initial e0e22b9fae75 original
     │
     ~
     "###
@@ -523,17 +523,17 @@ fn test_log_prefix_highlight_styled() {
     );
     insta::assert_snapshot!(stdout,
         @r###"
-    @  Change [1m[38;5;5mwq[0m[38;5;8mnwkozpkust[39m commit9 [1m[38;5;4med[0m[38;5;8me204633421[39m
-    ◉  Change [1m[38;5;5mkm[0m[38;5;8mkuslswpqwq[39m commit8 [1m[38;5;4mef3[0m[38;5;8md013266cd[39m
-    ◉  Change [1m[38;5;5mkp[0m[38;5;8mqxywonksrl[39m commit7 [1m[38;5;4maf[0m[38;5;8m95b841712d[39m
-    ◉  Change [1m[38;5;5mzn[0m[38;5;8mkkpsqqskkl[39m commit6 [1m[38;5;4m23[0m[38;5;8mc1103d3427[39m
-    ◉  Change [1m[38;5;5myo[0m[38;5;8mstqsxwqrlt[39m commit5 [1m[38;5;4mb87[0m[38;5;8maa9b24921[39m
-    ◉  Change [1m[38;5;5mvr[0m[38;5;8muxwmqvtpmx[39m commit4 [1m[38;5;4m1e[0m[38;5;8ma31a205ce9[39m
-    ◉  Change [1m[38;5;5myq[0m[38;5;8mosqzytrlsw[39m commit3 [1m[38;5;4m34[0m[38;5;8mbefb94f4eb[39m
-    ◉  Change [1m[38;5;5mro[0m[38;5;8myxmykxtrkr[39m commit2 [1m[38;5;4mcc[0m[38;5;8m0c127948ef[39m
-    ◉  Change [1m[38;5;5mmz[0m[38;5;8mvwutvlkqwt[39m commit1 [1m[38;5;4m1b[0m[38;5;8m7b715afc3f[39m
-    ◉  Change [1m[38;5;5mqpv[0m[38;5;8muntsmwlqt[39m initial [1m[38;5;4me0[0m[38;5;8me22b9fae75[39m [38;5;5moriginal[39m
-    ◉  Change [1m[38;5;5mzzz[0m[38;5;8mzzzzzzzzz[39m [1m[38;5;4m00[0m[38;5;8m0000000000[39m
+    [1m[38;5;2m@[0m  Change [1m[38;5;5mwq[0m[38;5;8mnwkozpkust[39m commit9 [1m[38;5;4med[0m[38;5;8me204633421[39m
+    ○  Change [1m[38;5;5mkm[0m[38;5;8mkuslswpqwq[39m commit8 [1m[38;5;4mef3[0m[38;5;8md013266cd[39m
+    ○  Change [1m[38;5;5mkp[0m[38;5;8mqxywonksrl[39m commit7 [1m[38;5;4maf[0m[38;5;8m95b841712d[39m
+    ○  Change [1m[38;5;5mzn[0m[38;5;8mkkpsqqskkl[39m commit6 [1m[38;5;4m23[0m[38;5;8mc1103d3427[39m
+    ○  Change [1m[38;5;5myo[0m[38;5;8mstqsxwqrlt[39m commit5 [1m[38;5;4mb87[0m[38;5;8maa9b24921[39m
+    ○  Change [1m[38;5;5mvr[0m[38;5;8muxwmqvtpmx[39m commit4 [1m[38;5;4m1e[0m[38;5;8ma31a205ce9[39m
+    ○  Change [1m[38;5;5myq[0m[38;5;8mosqzytrlsw[39m commit3 [1m[38;5;4m34[0m[38;5;8mbefb94f4eb[39m
+    ○  Change [1m[38;5;5mro[0m[38;5;8myxmykxtrkr[39m commit2 [1m[38;5;4mcc[0m[38;5;8m0c127948ef[39m
+    ○  Change [1m[38;5;5mmz[0m[38;5;8mvwutvlkqwt[39m commit1 [1m[38;5;4m1b[0m[38;5;8m7b715afc3f[39m
+    ○  Change [1m[38;5;5mqpv[0m[38;5;8muntsmwlqt[39m initial [1m[38;5;4me0[0m[38;5;8me22b9fae75[39m [38;5;5moriginal[39m
+    [1m[38;5;14m◆[0m  Change [1m[38;5;5mzzz[0m[38;5;8mzzzzzzzzz[39m [1m[38;5;4m00[0m[38;5;8m0000000000[39m
     "###
     );
     let stdout = test_env.jj_cmd_success(
@@ -549,17 +549,17 @@ fn test_log_prefix_highlight_styled() {
     );
     insta::assert_snapshot!(stdout,
         @r###"
-    @  Change [1m[38;5;5mwq[0m[38;5;8mn[39m commit9 [1m[38;5;4med[0m[38;5;8me[39m
-    ◉  Change [1m[38;5;5mkm[0m[38;5;8mk[39m commit8 [1m[38;5;4mef3[0m
-    ◉  Change [1m[38;5;5mkp[0m[38;5;8mq[39m commit7 [1m[38;5;4maf[0m[38;5;8m9[39m
-    ◉  Change [1m[38;5;5mzn[0m[38;5;8mk[39m commit6 [1m[38;5;4m23[0m[38;5;8mc[39m
-    ◉  Change [1m[38;5;5myo[0m[38;5;8ms[39m commit5 [1m[38;5;4mb87[0m
-    ◉  Change [1m[38;5;5mvr[0m[38;5;8mu[39m commit4 [1m[38;5;4m1e[0m[38;5;8ma[39m
-    ◉  Change [1m[38;5;5myq[0m[38;5;8mo[39m commit3 [1m[38;5;4m34[0m[38;5;8mb[39m
-    ◉  Change [1m[38;5;5mro[0m[38;5;8my[39m commit2 [1m[38;5;4mcc[0m[38;5;8m0[39m
-    ◉  Change [1m[38;5;5mmz[0m[38;5;8mv[39m commit1 [1m[38;5;4m1b[0m[38;5;8m7[39m
-    ◉  Change [1m[38;5;5mqpv[0m initial [1m[38;5;4me0[0m[38;5;8me[39m [38;5;5moriginal[39m
-    ◉  Change [1m[38;5;5mzzz[0m [1m[38;5;4m00[0m[38;5;8m0[39m
+    [1m[38;5;2m@[0m  Change [1m[38;5;5mwq[0m[38;5;8mn[39m commit9 [1m[38;5;4med[0m[38;5;8me[39m
+    ○  Change [1m[38;5;5mkm[0m[38;5;8mk[39m commit8 [1m[38;5;4mef3[0m
+    ○  Change [1m[38;5;5mkp[0m[38;5;8mq[39m commit7 [1m[38;5;4maf[0m[38;5;8m9[39m
+    ○  Change [1m[38;5;5mzn[0m[38;5;8mk[39m commit6 [1m[38;5;4m23[0m[38;5;8mc[39m
+    ○  Change [1m[38;5;5myo[0m[38;5;8ms[39m commit5 [1m[38;5;4mb87[0m
+    ○  Change [1m[38;5;5mvr[0m[38;5;8mu[39m commit4 [1m[38;5;4m1e[0m[38;5;8ma[39m
+    ○  Change [1m[38;5;5myq[0m[38;5;8mo[39m commit3 [1m[38;5;4m34[0m[38;5;8mb[39m
+    ○  Change [1m[38;5;5mro[0m[38;5;8my[39m commit2 [1m[38;5;4mcc[0m[38;5;8m0[39m
+    ○  Change [1m[38;5;5mmz[0m[38;5;8mv[39m commit1 [1m[38;5;4m1b[0m[38;5;8m7[39m
+    ○  Change [1m[38;5;5mqpv[0m initial [1m[38;5;4me0[0m[38;5;8me[39m [38;5;5moriginal[39m
+    [1m[38;5;14m◆[0m  Change [1m[38;5;5mzzz[0m [1m[38;5;4m00[0m[38;5;8m0[39m
     "###
     );
     let stdout = test_env.jj_cmd_success(
@@ -575,17 +575,17 @@ fn test_log_prefix_highlight_styled() {
     );
     insta::assert_snapshot!(stdout,
         @r###"
-    @  Change [1m[38;5;5mwq[0m commit9 [1m[38;5;4med[0m
-    ◉  Change [1m[38;5;5mkm[0m commit8 [1m[38;5;4mef3[0m
-    ◉  Change [1m[38;5;5mkp[0m commit7 [1m[38;5;4maf[0m
-    ◉  Change [1m[38;5;5mzn[0m commit6 [1m[38;5;4m23[0m
-    ◉  Change [1m[38;5;5myo[0m commit5 [1m[38;5;4mb87[0m
-    ◉  Change [1m[38;5;5mvr[0m commit4 [1m[38;5;4m1e[0m
-    ◉  Change [1m[38;5;5myq[0m commit3 [1m[38;5;4m34[0m
-    ◉  Change [1m[38;5;5mro[0m commit2 [1m[38;5;4mcc[0m
-    ◉  Change [1m[38;5;5mmz[0m commit1 [1m[38;5;4m1b[0m
-    ◉  Change [1m[38;5;5mqpv[0m initial [1m[38;5;4me0[0m [38;5;5moriginal[39m
-    ◉  Change [1m[38;5;5mzzz[0m [1m[38;5;4m00[0m
+    [1m[38;5;2m@[0m  Change [1m[38;5;5mwq[0m commit9 [1m[38;5;4med[0m
+    ○  Change [1m[38;5;5mkm[0m commit8 [1m[38;5;4mef3[0m
+    ○  Change [1m[38;5;5mkp[0m commit7 [1m[38;5;4maf[0m
+    ○  Change [1m[38;5;5mzn[0m commit6 [1m[38;5;4m23[0m
+    ○  Change [1m[38;5;5myo[0m commit5 [1m[38;5;4mb87[0m
+    ○  Change [1m[38;5;5mvr[0m commit4 [1m[38;5;4m1e[0m
+    ○  Change [1m[38;5;5myq[0m commit3 [1m[38;5;4m34[0m
+    ○  Change [1m[38;5;5mro[0m commit2 [1m[38;5;4mcc[0m
+    ○  Change [1m[38;5;5mmz[0m commit1 [1m[38;5;4m1b[0m
+    ○  Change [1m[38;5;5mqpv[0m initial [1m[38;5;4me0[0m [38;5;5moriginal[39m
+    [1m[38;5;14m◆[0m  Change [1m[38;5;5mzzz[0m [1m[38;5;4m00[0m
     "###
     );
 }
@@ -621,7 +621,7 @@ fn test_log_prefix_highlight_counts_hidden_commits() {
         test_env.jj_cmd_success(&repo_path, &["log", "-r", "all()", "-T", prefix_format]),
         @r###"
     @  Change q[pvuntsmwlqt] initial e0[e22b9fae75] original
-    ◉  Change z[zzzzzzzzzzz] 0[00000000000]
+    ◆  Change z[zzzzzzzzzzz] 0[00000000000]
     "###
     );
 
@@ -637,9 +637,9 @@ fn test_log_prefix_highlight_counts_hidden_commits() {
         test_env.jj_cmd_success(&repo_path, &["log", "-T", prefix_format]),
         @r###"
     @  Change wq[nwkozpkust] 44[4c3c5066d3]
-    │ ◉  Change qpv[untsmwlqt] initial e0e[22b9fae75] original
+    │ ○  Change qpv[untsmwlqt] initial e0e[22b9fae75] original
     ├─╯
-    ◉  Change zzz[zzzzzzzzz] 00[0000000000]
+    ◆  Change zzz[zzzzzzzzz] 00[0000000000]
     "###
     );
     insta::assert_snapshot!(
@@ -668,22 +668,22 @@ fn test_log_short_shortest_length_parameter() {
     insta::assert_snapshot!(
         render(r#"commit_id.short(0) ++ "|" ++ commit_id.shortest(0)"#), @r###"
     @  |2
-    ◉  |0
+    ◆  |0
     "###);
     insta::assert_snapshot!(
         render(r#"commit_id.short(-0) ++ "|" ++ commit_id.shortest(-0)"#), @r###"
     @  |2
-    ◉  |0
+    ◆  |0
     "###);
     insta::assert_snapshot!(
         render(r#"commit_id.short(-100) ++ "|" ++ commit_id.shortest(-100)"#), @r###"
     @  <Error: out of range integral type conversion attempted>|<Error: out of range integral type conversion attempted>
-    ◉  <Error: out of range integral type conversion attempted>|<Error: out of range integral type conversion attempted>
+    ◆  <Error: out of range integral type conversion attempted>|<Error: out of range integral type conversion attempted>
     "###);
     insta::assert_snapshot!(
         render(r#"commit_id.short(100) ++ "|" ++ commit_id.shortest(100)"#), @r###"
     @  230dd059e1b059aefc0da06a2e5a7dbf22362f22|230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    ◉  0000000000000000000000000000000000000000|0000000000000000000000000000000000000000
+    ◆  0000000000000000000000000000000000000000|0000000000000000000000000000000000000000
     "###);
 }
 
@@ -734,7 +734,7 @@ fn test_log_divergence() {
     // No divergence
     insta::assert_snapshot!(stdout, @r###"
     @  description 1
-    ◉
+    ◆
     "###);
 
     // Create divergence
@@ -744,10 +744,10 @@ fn test_log_divergence() {
     );
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["log", "-T", template]);
     insta::assert_snapshot!(stdout, @r###"
-    ◉  description 2 !divergence!
+    ○  description 2 !divergence!
     │ @  description 1 !divergence!
     ├─╯
-    ◉
+    ◆
     "###);
     insta::assert_snapshot!(stderr, @r###"
     Concurrent modification detected, resolving automatically.
@@ -765,8 +765,8 @@ fn test_log_reversed() {
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "description", "--reversed"]);
     insta::assert_snapshot!(stdout, @r###"
-    ◉
-    ◉  first
+    ◆
+    ○  first
     @  second
     "###);
 
@@ -795,7 +795,7 @@ fn test_log_filtered_by_path() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "description", "file1"]);
     insta::assert_snapshot!(stdout, @r###"
     @  second
-    ◉  first
+    ○  first
     │
     ~
     "###);
@@ -811,7 +811,7 @@ fn test_log_filtered_by_path() {
     insta::assert_snapshot!(stdout, @r###"
     @  second
     │  M file1
-    ◉  first
+    ○  first
     │  A file1
     ~
     "###);
@@ -825,25 +825,29 @@ fn test_log_filtered_by_path() {
     A file2
     "###);
 
-    // Fileset/pattern syntax is disabled by default.
+    // Fileset/pattern syntax can be disabled.
     let stderr = test_env.jj_cmd_failure(
         test_env.env_root(),
-        &["log", "-R", repo_path.to_str().unwrap(), "all()"],
+        &[
+            "log",
+            "--config-toml=ui.allow-filesets=false",
+            "-R",
+            repo_path.to_str().unwrap(),
+            "all()",
+        ],
     );
     insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
     Error: Path "all()" is not in the repo "repo"
     Caused by: Invalid component ".." in repo-relative path "../all()"
     "###);
 
-    test_env.add_config("ui.allow-filesets = true");
-
     // empty revisions are filtered out by "all()" fileset.
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-Tdescription", "-s", "all()"]);
     insta::assert_snapshot!(stdout, @r###"
     @  second
     │  M file1
     │  A file2
-    ◉  first
+    ○  first
     │  A file1
     ~
     "###);
@@ -863,7 +867,7 @@ fn test_log_filtered_by_path() {
     insta::assert_snapshot!(stdout.replace('\\', "/"), @r###"
     @  second
     │  M repo/file1
-    ◉  first
+    ○  first
     │  A repo/file1
     ~
     "###);
@@ -908,8 +912,8 @@ fn test_log_limit() {
     insta::assert_snapshot!(stdout, @r###"
     @    d
     ├─╮
-    │ ◉  b
-    ◉ │  c
+    │ ○  b
+    ○ │  c
     ├─╯
     "###);
 
@@ -918,7 +922,7 @@ fn test_log_limit() {
     insta::assert_snapshot!(stdout, @r###"
     @    d
     ├─╮
-    │ ◉  b
+    │ ○  b
     "###);
 
     let stdout = test_env.jj_cmd_success(
@@ -936,10 +940,10 @@ fn test_log_limit() {
         &["log", "-T", "description", "--limit=3", "--reversed"],
     );
     insta::assert_snapshot!(stdout, @r###"
-    ◉
-    ◉    a
+    ◆
+    ○    a
     ├─╮
-    │ ◉  c
+    │ ○  c
     "###);
     let stdout = test_env.jj_cmd_success(
         &repo_path,
@@ -963,7 +967,7 @@ fn test_log_limit() {
         &["log", "-T", "description", "--limit=1", "b", "c"],
     );
     insta::assert_snapshot!(stdout, @r###"
-    ◉  c
+    ○  c
     │
     ~
     "###);
@@ -1101,7 +1105,7 @@ fn test_multiple_revsets() {
     insta::assert_snapshot!(
         test_env.jj_cmd_success(&repo_path, &["log", "-T", "branches", "-rfoo"]),
         @r###"
-    ◉  foo
+    ○  foo
     │
     ~
     "###);
@@ -1109,15 +1113,15 @@ fn test_multiple_revsets() {
         test_env.jj_cmd_success(&repo_path, &["log", "-T", "branches", "-rfoo", "-rbar", "-rbaz"]),
         @r###"
     @  baz
-    ◉  bar
-    ◉  foo
+    ○  bar
+    ○  foo
     │
     ~
     "###);
     insta::assert_snapshot!(
         test_env.jj_cmd_success(&repo_path, &["log", "-T", "branches", "-rfoo", "-rfoo"]),
         @r###"
-    ◉  foo
+    ○  foo
     │
     ~
     "###);
@@ -1148,26 +1152,26 @@ fn test_graph_template_color() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", template]);
     insta::assert_snapshot!(stdout, @r###"
     @  single line
-    ◉  first line
+    ○  first line
     │  second line
     │  third line
-    ◉
+    ◆
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["--color=always", "log", "-T", template]);
     insta::assert_snapshot!(stdout, @r###"
-    @  [1m[38;5;2msingle line[0m
-    ◉  [38;5;1mfirst line[39m
+    [1m[38;5;2m@[0m  [1m[38;5;2msingle line[0m
+    ○  [38;5;1mfirst line[39m
     │  [38;5;1msecond line[39m
     │  [38;5;1mthird line[39m
-    ◉
+    [1m[38;5;14m◆[0m
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["--color=debug", "log", "-T", template]);
     insta::assert_snapshot!(stdout, @r###"
-    <<node::@>>  [1m[38;5;2m<<log working_copy description::single line>>[0m
-    <<node::◉>>  [38;5;1m<<log description::first line>>[39m
+    [1m[38;5;2m<<node working_copy::@>>[0m  [1m[38;5;2m<<log working_copy description::single line>>[0m
+    <<node::○>>  [38;5;1m<<log description::first line>>[39m
     │  [38;5;1m<<log description::second line>>[39m
     │  [38;5;1m<<log description::third line>>[39m
-    <<node::◉>>
+    [1m[38;5;14m<<node immutable::◆>>[0m
     "###);
 }
 
@@ -1195,15 +1199,15 @@ fn test_graph_styles() {
     insta::assert_snapshot!(stdout, @r###"
     @    merge
     ├─╮
-    │ ◉  side branch
+    │ ○  side branch
     │ │  with
     │ │  long
     │ │  description
-    │ ◉  main branch 2
+    │ ○  main branch 2
     ├─╯
-    ◉  main branch 1
-    ◉  initial
-    ◉
+    ○  main branch 1
+    ○  initial
+    ◆
     "###);
 
     // ASCII style
@@ -1220,7 +1224,7 @@ fn test_graph_styles() {
     |/
     o  main branch 1
     o  initial
-    o
+    +
     "###);
 
     // Large ASCII style
@@ -1239,7 +1243,7 @@ fn test_graph_styles() {
     |/
     o  main branch 1
     o  initial
-    o
+    +
     "###);
 
     // Curved style
@@ -1248,15 +1252,15 @@ fn test_graph_styles() {
     insta::assert_snapshot!(stdout, @r###"
     @    merge
     ├─╮
-    │ ◉  side branch
+    │ ○  side branch
     │ │  with
     │ │  long
     │ │  description
-    │ ◉  main branch 2
+    │ ○  main branch 2
     ├─╯
-    ◉  main branch 1
-    ◉  initial
-    ◉
+    ○  main branch 1
+    ○  initial
+    ◆
     "###);
 
     // Square style
@@ -1265,15 +1269,15 @@ fn test_graph_styles() {
     insta::assert_snapshot!(stdout, @r###"
     @    merge
     ├─┐
-    │ ◉  side branch
+    │ ○  side branch
     │ │  with
     │ │  long
     │ │  description
-    │ ◉  main branch 2
+    │ ○  main branch 2
     ├─┘
-    ◉  main branch 1
-    ◉  initial
-    ◉
+    ○  main branch 1
+    ○  initial
+    ◆
     "###);
 }
 
@@ -1324,7 +1328,7 @@ fn test_log_word_wrap() {
 
     // Color labels should be preserved
     insta::assert_snapshot!(render(&["log", "-r@", "--color=always"], 40, true), @r###"
-    @  [1m[38;5;13mm[38;5;8mzvwutvl[39m [38;5;3mtest.user@example.com[39m[0m
+    [1m[38;5;2m@[0m  [1m[38;5;13mm[38;5;8mzvwutvl[39m [38;5;3mtest.user@example.com[39m[0m
     │  [1m[38;5;14m2001-02-03 08:05:11[39m [38;5;12m04[38;5;8m4c0400[39m[0m
     ~  [1m[38;5;10m(empty)[39m merge[0m
     "###);
@@ -1336,18 +1340,18 @@ fn test_log_word_wrap() {
     ├─╮  3 4 5
     │ │  6 7 8
     │ │  9
-    │ ◉  0 1 2
+    │ ○  0 1 2
     │ │  3 4 5
     │ │  6 7 8
     │ │  9
-    │ ◉  0 1 2
+    │ ○  0 1 2
     ├─╯  3 4 5
     │    6 7 8
     │    9
-    ◉  0 1 2 3
+    ○  0 1 2 3
     │  4 5 6 7
     │  8 9
-    ◉  0 1 2 3
+    ◆  0 1 2 3
        4 5 6 7
        8 9
     "###);
@@ -1373,6 +1377,53 @@ fn test_log_word_wrap() {
     "###);
 }
 
+#[test]
+fn test_log_diff_stat_width() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+    let render = |args: &[&str], columns: u32| {
+        let assert = test_env
+            .jj_cmd(&repo_path, args)
+            .env("COLUMNS", columns.to_string())
+            .assert()
+            .success()
+            .stderr("");
+        get_stdout_string(&assert)
+    };
+
+    std::fs::write(repo_path.join("file1"), "foo\n".repeat(100)).unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "root()"]);
+    std::fs::write(repo_path.join("file2"), "foo\n".repeat(100)).unwrap();
+
+    insta::assert_snapshot!(render(&["log", "--stat", "--no-graph"], 30), @r###"
+    rlvkpnrz test.user@example.com 2001-02-03 08:05:09 287520bf
+    (no description set)
+    file2 | 100 +++++++++++++++
+    1 file changed, 100 insertions(+), 0 deletions(-)
+    qpvuntsm test.user@example.com 2001-02-03 08:05:08 e292def1
+    (no description set)
+    file1 | 100 +++++++++++++++
+    1 file changed, 100 insertions(+), 0 deletions(-)
+    zzzzzzzz root() 00000000
+    0 files changed, 0 insertions(+), 0 deletions(-)
+    "###);
+
+    // Graph width should be subtracted
+    insta::assert_snapshot!(render(&["log", "--stat"], 30), @r###"
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:09 287520bf
+    │  (no description set)
+    │  file2 | 100 ++++++++++++
+    │  1 file changed, 100 insertions(+), 0 deletions(-)
+    │ ○  qpvuntsm test.user@example.com 2001-02-03 08:05:08 e292def1
+    ├─╯  (no description set)
+    │    file1 | 100 ++++++++++
+    │    1 file changed, 100 insertions(+), 0 deletions(-)
+    ◆  zzzzzzzz root() 00000000
+       0 files changed, 0 insertions(+), 0 deletions(-)
+    "###);
+}
+
 #[test]
 fn test_elided() {
     // Test that elided commits are shown as synthetic nodes.
@@ -1401,17 +1452,17 @@ fn test_elided() {
     insta::assert_snapshot!(get_log("::"), @r###"
     @    merge
     ├─╮
-    │ ◉  side branch 2
+    │ ○  side branch 2
     │ │
-    │ ◉  side branch 1
+    │ ○  side branch 1
     │ │
-    ◉ │  main branch 2
+    ○ │  main branch 2
     │ │
-    ◉ │  main branch 1
+    ○ │  main branch 1
     ├─╯
-    ◉  initial
+    ○  initial
     │
-    ◉
+    ◆
     "###);
 
     // Elide some commits from each side of the merge. It's unclear that a revision
@@ -1420,11 +1471,11 @@ fn test_elided() {
     insta::assert_snapshot!(get_log("@ | @- | description(initial)"), @r###"
     @    merge
     ├─╮
-    │ ◉  side branch 2
+    │ ○  side branch 2
     │ ╷
-    ◉ ╷  main branch 2
+    ○ ╷  main branch 2
     ├─╯
-    ◉  initial
+    ○  initial
     │
     ~
     "###);
@@ -1432,11 +1483,11 @@ fn test_elided() {
     // Elide shared commits. It's unclear that a revision was skipped on the right
     // side (#1252).
     insta::assert_snapshot!(get_log("@-- | root()"), @r###"
-    ◉  side branch 1
+    ○  side branch 1
     ╷
-    ╷ ◉  main branch 1
+    ╷ ○  main branch 1
     ╭─╯
-    ◉
+    ◆
     "###);
 
     // Now test the same thing with synthetic nodes for elided commits
@@ -1446,14 +1497,14 @@ fn test_elided() {
     insta::assert_snapshot!(get_log("@ | @- | description(initial)"), @r###"
     @    merge
     ├─╮
-    │ ◉  side branch 2
+    │ ○  side branch 2
     │ │
-    │ ◌  (elided revisions)
-    ◉ │  main branch 2
+    │ ~  (elided revisions)
+    ○ │  main branch 2
     │ │
-    ◌ │  (elided revisions)
+    ~ │  (elided revisions)
     ├─╯
-    ◉  initial
+    ○  initial
     │
     ~
     "###);
@@ -1461,14 +1512,14 @@ fn test_elided() {
     // Elide shared commits. To keep the implementation simple, it still gets
     // rendered as two synthetic nodes.
     insta::assert_snapshot!(get_log("@-- | root()"), @r###"
-    ◉  side branch 1
+    ○  side branch 1
     │
-    ◌  (elided revisions)
-    │ ◉  main branch 1
+    ~  (elided revisions)
+    │ ○  main branch 1
     │ │
-    │ ◌  (elided revisions)
+    │ ~  (elided revisions)
     ├─╯
-    ◉
+    ◆
     "###);
 }
 
