@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::Path;
+use std::path::{Path, PathBuf};
 
 use crate::common::TestEnvironment;
 
@@ -38,7 +38,7 @@ fn test_split_by_paths() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  qpvuntsmwlqt false
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
     insta::assert_snapshot!(get_recorded_dates(&test_env, &repo_path,"@"), @r###"
     Author date:  2001-02-03 04:05:08.000 +07:00
@@ -72,8 +72,8 @@ fn test_split_by_paths() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  zsuskulnrvyr false
-    ◉  qpvuntsmwlqt false
-    ◉  zzzzzzzzzzzz true
+    ○  qpvuntsmwlqt false
+    ◆  zzzzzzzzzzzz true
     "###);
 
     // The author dates of the new commits should be inherited from the commit being
@@ -111,9 +111,9 @@ fn test_split_by_paths() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  zsuskulnrvyr false
-    ◉  znkkpsqqskkl true
-    ◉  qpvuntsmwlqt false
-    ◉  zzzzzzzzzzzz true
+    ○  znkkpsqqskkl true
+    ○  qpvuntsmwlqt false
+    ◆  zzzzzzzzzzzz true
     "###);
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "-r", "@--"]);
@@ -139,9 +139,9 @@ fn test_split_by_paths() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     @  zsuskulnrvyr false
-    ◉  lylxulplsnyw false
-    ◉  qpvuntsmwlqt true
-    ◉  zzzzzzzzzzzz true
+    ○  lylxulplsnyw false
+    ○  qpvuntsmwlqt true
+    ◆  zzzzzzzzzzzz true
     "###);
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "-r", "@-"]);
@@ -182,32 +182,30 @@ fn test_split_with_non_empty_description() {
     Parent commit      : qpvuntsm 231a3c00 part 1
     "###);
 
-    assert_eq!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(),
-        r#"JJ: Enter a description for the first commit.
-test
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+    JJ: Enter a description for the first commit.
+    test
 
-JJ: This commit contains the following changes:
-JJ:     A file1
+    JJ: This commit contains the following changes:
+    JJ:     A file1
 
-JJ: Lines starting with "JJ: " (like this one) will be removed.
-"#
-    );
-    assert_eq!(
-        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(),
-        r#"JJ: Enter a description for the second commit.
-test
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r###"
+    JJ: Enter a description for the second commit.
+    test
 
-JJ: This commit contains the following changes:
-JJ:     A file2
+    JJ: This commit contains the following changes:
+    JJ:     A file2
 
-JJ: Lines starting with "JJ: " (like this one) will be removed.
-"#
-    );
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @  kkmpptxzrspx false part 2
-    ◉  qpvuntsmwlqt false part 1
-    ◉  zzzzzzzzzzzz true
+    ○  qpvuntsmwlqt false part 1
+    ◆  zzzzzzzzzzzz true
     "###);
 }
 
@@ -244,23 +242,23 @@ fn test_split_with_default_description() {
     // prompted to add a description to the first commit, which will use the
     // default value we set. The second commit will inherit the empty
     // description from the commit being split.
-    assert_eq!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(),
-        r#"JJ: Enter a description for the first commit.
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+    JJ: Enter a description for the first commit.
 
 
-TESTED=TODO
-JJ: This commit contains the following changes:
-JJ:     A file1
+    TESTED=TODO
 
-JJ: Lines starting with "JJ: " (like this one) will be removed.
-"#
-    );
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
     assert!(!test_env.env_root().join("editor2").exists());
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @  kkmpptxzrspx false test_branch
-    ◉  qpvuntsmwlqt false TESTED=TODO
-    ◉  zzzzzzzzzzzz true
+    ○  qpvuntsmwlqt false TESTED=TODO
+    ◆  zzzzzzzzzzzz true
     "###);
 }
 
@@ -282,10 +280,10 @@ fn test_split_with_merge_child() {
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @    zsuskulnrvyr true 2
     ├─╮
-    │ ◉  kkmpptxzrspx false a
-    ◉ │  qpvuntsmwlqt true 1
+    │ ○  kkmpptxzrspx false a
+    ○ │  qpvuntsmwlqt true 1
     ├─╯
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 
     // Set up the editor and do the split.
@@ -309,11 +307,11 @@ fn test_split_with_merge_child() {
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @    zsuskulnrvyr true 2
     ├─╮
-    │ ◉  royxmykxtrkr false Add file2
-    │ ◉  kkmpptxzrspx false Add file1
-    ◉ │  qpvuntsmwlqt true 1
+    │ ○  royxmykxtrkr false Add file2
+    │ ○  kkmpptxzrspx false Add file1
+    ○ │  qpvuntsmwlqt true 1
     ├─╯
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 }
 
@@ -334,7 +332,7 @@ fn test_split_siblings_no_descendants() {
     test_env.jj_cmd_ok(&workspace_path, &["branch", "create", "test_branch"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @  qpvuntsmwlqt false test_branch
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 
     let edit_script = test_env.set_up_fake_editor();
@@ -354,27 +352,27 @@ fn test_split_siblings_no_descendants() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @  zsuskulnrvyr false test_branch
-    │ ◉  qpvuntsmwlqt false TESTED=TODO
+    │ ○  qpvuntsmwlqt false TESTED=TODO
     ├─╯
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 
     // Since the commit being split has no description, the user will only be
     // prompted to add a description to the first commit, which will use the
     // default value we set. The second commit will inherit the empty
     // description from the commit being split.
-    assert_eq!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(),
-        r#"JJ: Enter a description for the first commit.
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+    JJ: Enter a description for the first commit.
 
 
-TESTED=TODO
-JJ: This commit contains the following changes:
-JJ:     A file1
+    TESTED=TODO
 
-JJ: Lines starting with "JJ: " (like this one) will be removed.
-"#
-    );
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
     assert!(!test_env.env_root().join("editor2").exists());
 }
 
@@ -401,10 +399,10 @@ fn test_split_siblings_with_descendants() {
     test_env.jj_cmd_ok(&workspace_path, &["prev", "--edit"]);
     test_env.jj_cmd_ok(&workspace_path, &["prev", "--edit"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
-    ◉  kkmpptxzrspx false Add file4
-    ◉  rlvkpnrzqnoo false Add file3
+    ○  kkmpptxzrspx false Add file4
+    ○  rlvkpnrzqnoo false Add file3
     @  qpvuntsmwlqt false Add file1 & file2
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 
     // Set up the editor and do the split.
@@ -432,39 +430,37 @@ fn test_split_siblings_with_descendants() {
     Added 0 files, modified 0 files, removed 1 files
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
-    ◉  kkmpptxzrspx false Add file4
-    ◉    rlvkpnrzqnoo false Add file3
+    ○  kkmpptxzrspx false Add file4
+    ○    rlvkpnrzqnoo false Add file3
     ├─╮
     │ @  vruxwmqvtpmx false Add file2
-    ◉ │  qpvuntsmwlqt false Add file1
+    ○ │  qpvuntsmwlqt false Add file1
     ├─╯
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 
     // The commit we're splitting has a description, so the user will be
     // prompted to enter a description for each of the sibling commits.
-    assert_eq!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(),
-        r#"JJ: Enter a description for the first commit.
-Add file1 & file2
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+    JJ: Enter a description for the first commit.
+    Add file1 & file2
 
-JJ: This commit contains the following changes:
-JJ:     A file1
+    JJ: This commit contains the following changes:
+    JJ:     A file1
 
-JJ: Lines starting with "JJ: " (like this one) will be removed.
-"#
-    );
-    assert_eq!(
-        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(),
-        r#"JJ: Enter a description for the second commit.
-Add file1 & file2
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r###"
+    JJ: Enter a description for the second commit.
+    Add file1 & file2
 
-JJ: This commit contains the following changes:
-JJ:     A file2
+    JJ: This commit contains the following changes:
+    JJ:     A file2
 
-JJ: Lines starting with "JJ: " (like this one) will be removed.
-"#
-    );
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
 }
 
 // This test makes sure that the children of the commit being split retain any
@@ -485,10 +481,10 @@ fn test_split_siblings_with_merge_child() {
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @    zsuskulnrvyr true 2
     ├─╮
-    │ ◉  kkmpptxzrspx false a
-    ◉ │  qpvuntsmwlqt true 1
+    │ ○  kkmpptxzrspx false a
+    ○ │  qpvuntsmwlqt true 1
     ├─╯
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 
     // Set up the editor and do the split.
@@ -515,12 +511,12 @@ fn test_split_siblings_with_merge_child() {
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
     @      zsuskulnrvyr true 2
     ├─┬─╮
-    │ │ ◉  royxmykxtrkr false Add file2
-    │ ◉ │  kkmpptxzrspx false Add file1
+    │ │ ○  royxmykxtrkr false Add file2
+    │ ○ │  kkmpptxzrspx false Add file1
     │ ├─╯
-    ◉ │  qpvuntsmwlqt true 1
+    ○ │  qpvuntsmwlqt true 1
     ├─╯
-    ◉  zzzzzzzzzzzz true
+    ◆  zzzzzzzzzzzz true
     "###);
 }
 
@@ -538,3 +534,72 @@ fn test_split_empty() {
     Hint: Use `jj new` if you want to create another empty commit.
     "###);
 }
+
+#[test]
+fn test_split_message_editor_avoids_unc() {
+    let mut test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    std::fs::write(repo_path.join("file1"), "foo").unwrap();
+    std::fs::write(repo_path.join("file2"), "foo").unwrap();
+
+    let edit_script = test_env.set_up_fake_editor();
+    std::fs::write(edit_script, "dump-path path").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["split", "file2"]);
+
+    let edited_path =
+        PathBuf::from(std::fs::read_to_string(test_env.env_root().join("path")).unwrap());
+    // While `assert!(!edited_path.starts_with("//?/"))` could work here in most
+    // cases, it fails when it is not safe to strip the prefix, such as paths
+    // over 260 chars.
+    assert_eq!(edited_path, dunce::simplified(&edited_path));
+}
+
+#[test]
+fn test_split_interactive() {
+    let mut test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let workspace_path = test_env.env_root().join("repo");
+
+    std::fs::write(workspace_path.join("file1"), "foo\n").unwrap();
+    std::fs::write(workspace_path.join("file2"), "bar\n").unwrap();
+    let edit_script = test_env.set_up_fake_editor();
+    std::fs::write(edit_script, ["dump editor"].join("\0")).unwrap();
+
+    let diff_editor = test_env.set_up_fake_diff_editor();
+    let diff_script = ["rm file2", "dump JJ-INSTRUCTIONS instrs"].join("\0");
+    std::fs::write(diff_editor, diff_script).unwrap();
+
+    // Split the working commit interactively and select only file1
+    let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_path, &["split"]);
+
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("instrs")).unwrap(), @r###"
+    You are splitting a commit into two: qpvuntsm 44af2155 (no description set)
+
+    The diff initially shows the changes in the commit you're splitting.
+
+    Adjust the right side until it shows the contents you want for the first commit.
+    The remainder will be in the second commit. If you don't make any changes, then
+    the operation will be aborted.
+    "###);
+
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r###"
+    JJ: Enter a description for the first commit.
+
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
+
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    First part: qpvuntsm 0e15949e (no description set)
+    Second part: rlvkpnrz 9ed12e4c (no description set)
+    Working copy now at: rlvkpnrz 9ed12e4c (no description set)
+    Parent commit      : qpvuntsm 0e15949e (no description set)
+    "###);
+}
