@@ -47,18 +47,18 @@ fn test_workspaces_add_second_workspace() {
     // Can see the working-copy commit in each workspace in the log output. The "@"
     // node in the graph indicates the current workspace's working-copy commit.
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉  5ed2222c28e2 second@
+    ○  5ed2222c28e2 second@
     │ @  8183d0fcaa4c default@
     ├─╯
-    ◉  751b12b7b981
-    ◉  000000000000
+    ○  751b12b7b981
+    ◆  000000000000
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &secondary_path), @r###"
     @  5ed2222c28e2 second@
-    │ ◉  8183d0fcaa4c default@
+    │ ○  8183d0fcaa4c default@
     ├─╯
-    ◉  751b12b7b981
-    ◉  000000000000
+    ○  751b12b7b981
+    ◆  000000000000
     "###);
 
     // Both workspaces show up when we list them
@@ -117,14 +117,100 @@ fn test_workspaces_add_second_workspace_on_merge() {
 
     // The new workspace's working-copy commit shares all parents with the old one.
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉    7013a493bd09 second@
+    ○    7013a493bd09 second@
     ├─╮
     │ │ @  35e47bff781e default@
     ╭─┬─╯
-    │ ◉  444b77e99d43
-    ◉ │  1694f2ddf8ec
+    │ ○  444b77e99d43
+    ○ │  1694f2ddf8ec
     ├─╯
-    ◉  000000000000
+    ◆  000000000000
+    "###);
+}
+
+/// Test that --ignore-working-copy is respected
+#[test]
+fn test_workspaces_add_ignore_working_copy() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "main"]);
+    let main_path = test_env.env_root().join("main");
+
+    // TODO: maybe better to error out early?
+    let stderr = test_env.jj_cmd_failure(
+        &main_path,
+        &["workspace", "add", "--ignore-working-copy", "../secondary"],
+    );
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Created workspace in "../secondary"
+    Error: This command must be able to update the working copy.
+    Hint: Don't use --ignore-working-copy.
+    "###);
+}
+
+/// Test that --at-op is respected
+#[test]
+fn test_workspaces_add_at_operation() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "main"]);
+    let main_path = test_env.env_root().join("main");
+
+    std::fs::write(main_path.join("file1"), "").unwrap();
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&main_path, &["commit", "-m1"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: rlvkpnrz 18d8b994 (empty) (no description set)
+    Parent commit      : qpvuntsm 3364a7ed 1
+    "###);
+
+    std::fs::write(main_path.join("file2"), "").unwrap();
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&main_path, &["commit", "-m2"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: kkmpptxz 2e7dc5ab (empty) (no description set)
+    Parent commit      : rlvkpnrz 0dbaa19a 2
+    "###);
+
+    // --at-op should disable snapshot in the main workspace, but the newly
+    // created workspace should still be writable.
+    std::fs::write(main_path.join("file3"), "").unwrap();
+    let (_stdout, stderr) = test_env.jj_cmd_ok(
+        &main_path,
+        &["workspace", "add", "--at-op=@-", "../secondary"],
+    );
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Created workspace in "../secondary"
+    Working copy now at: rzvqmyuk a4d1cbc9 (empty) (no description set)
+    Parent commit      : qpvuntsm 3364a7ed 1
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+    let secondary_path = test_env.env_root().join("secondary");
+
+    // New snapshot can be taken in the secondary workspace.
+    std::fs::write(secondary_path.join("file4"), "").unwrap();
+    let (stdout, stderr) = test_env.jj_cmd_ok(&secondary_path, &["status"]);
+    insta::assert_snapshot!(stdout, @r###"
+    Working copy changes:
+    A file4
+    Working copy : rzvqmyuk 2ba74f85 (no description set)
+    Parent commit: qpvuntsm 3364a7ed 1
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Concurrent modification detected, resolving automatically.
+    "###);
+
+    let stdout = test_env.jj_cmd_success(&secondary_path, &["op", "log", "-Tdescription"]);
+    insta::assert_snapshot!(stdout, @r###"
+    @  snapshot working copy
+    ○    resolve concurrent operations
+    ├─╮
+    ○ │  commit cd06097124e3e5860867e35c2bb105902c28ea38
+    │ ○  create initial working-copy commit in workspace secondary
+    │ ○  add workspace 'secondary'
+    ├─╯
+    ○  snapshot working copy
+    ○  commit 1c867a0762e30de4591890ea208849f793742c1b
+    ○  snapshot working copy
+    ○  add workspace 'default'
+    ○  initialize repo
+    ○
     "###);
 }
 
@@ -169,20 +255,20 @@ fn test_workspaces_add_workspace_at_revision() {
     // Can see the working-copy commit in each workspace in the log output. The "@"
     // node in the graph indicates the current workspace's working-copy commit.
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉  e374e74aa0c8 second@
+    ○  e374e74aa0c8 second@
     │ @  dadeedb493e8 default@
-    │ ◉  c420244c6398
+    │ ○  c420244c6398
     ├─╯
-    ◉  f6097c2f7cac
-    ◉  000000000000
+    ○  f6097c2f7cac
+    ◆  000000000000
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &secondary_path), @r###"
     @  e374e74aa0c8 second@
-    │ ◉  dadeedb493e8 default@
-    │ ◉  c420244c6398
+    │ ○  dadeedb493e8 default@
+    │ ○  c420244c6398
     ├─╯
-    ◉  f6097c2f7cac
-    ◉  000000000000
+    ○  f6097c2f7cac
+    ◆  000000000000
     "###);
 }
 
@@ -208,13 +294,13 @@ fn test_workspaces_add_workspace_multiple_revisions() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
     @  5b36783cd11c
-    │ ◉  6c843d62ca29
+    │ ○  6c843d62ca29
     ├─╯
-    │ ◉  544cd61f2d26
+    │ ○  544cd61f2d26
     ├─╯
-    │ ◉  f6097c2f7cac
+    │ ○  f6097c2f7cac
     ├─╯
-    ◉  000000000000
+    ◆  000000000000
     "###);
 
     let (_, stderr) = test_env.jj_cmd_ok(
@@ -239,16 +325,107 @@ fn test_workspaces_add_workspace_multiple_revisions() {
     "###);
 
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉      f4fa64f40944 merge@
+    ○      f4fa64f40944 merge@
     ├─┬─╮
-    │ │ ◉  f6097c2f7cac
-    │ ◉ │  544cd61f2d26
+    │ │ ○  f6097c2f7cac
+    │ ○ │  544cd61f2d26
     │ ├─╯
-    ◉ │  6c843d62ca29
+    ○ │  6c843d62ca29
     ├─╯
     │ @  5b36783cd11c default@
     ├─╯
-    ◉  000000000000
+    ◆  000000000000
+    "###);
+}
+
+#[test]
+fn test_workspaces_add_workspace_from_subdir() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "main"]);
+    let main_path = test_env.env_root().join("main");
+    let subdir_path = main_path.join("subdir");
+    let secondary_path = test_env.env_root().join("secondary");
+
+    std::fs::create_dir(&subdir_path).unwrap();
+    std::fs::write(subdir_path.join("file"), "contents").unwrap();
+    test_env.jj_cmd_ok(&main_path, &["commit", "-m", "initial"]);
+
+    let stdout = test_env.jj_cmd_success(&main_path, &["workspace", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
+    default: rlvkpnrz e1038e77 (empty) (no description set)
+    "###);
+
+    // Create workspace while in sub-directory of current workspace
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(&subdir_path, &["workspace", "add", "../../secondary"]);
+    insta::assert_snapshot!(stdout.replace('\\', "/"), @"");
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Created workspace in "../../secondary"
+    Working copy now at: rzvqmyuk 7ad84461 (empty) (no description set)
+    Parent commit      : qpvuntsm a3a43d9e initial
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+
+    // Both workspaces show up when we list them
+    let stdout = test_env.jj_cmd_success(&secondary_path, &["workspace", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
+    default: rlvkpnrz e1038e77 (empty) (no description set)
+    secondary: rzvqmyuk 7ad84461 (empty) (no description set)
+    "###);
+}
+
+#[test]
+fn test_workspaces_add_workspace_in_current_workspace() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "main"]);
+    let main_path = test_env.env_root().join("main");
+
+    std::fs::write(main_path.join("file"), "contents").unwrap();
+    test_env.jj_cmd_ok(&main_path, &["commit", "-m", "initial"]);
+
+    // Try to create workspace using name instead of path
+    let (stdout, stderr) = test_env.jj_cmd_ok(&main_path, &["workspace", "add", "secondary"]);
+    insta::assert_snapshot!(stdout.replace('\\', "/"), @"");
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Created workspace in "secondary"
+    Warning: Workspace created inside current directory. If this was unintentional, delete the "secondary" directory and run `jj workspace forget secondary` to remove it.
+    Working copy now at: pmmvwywv 0a77a39d (empty) (no description set)
+    Parent commit      : qpvuntsm 751b12b7 initial
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+
+    // Workspace created despite warning
+    let stdout = test_env.jj_cmd_success(&main_path, &["workspace", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
+    default: rlvkpnrz 46d9ba8b (no description set)
+    secondary: pmmvwywv 0a77a39d (empty) (no description set)
+    "###);
+
+    // Use explicit path instead (no warning)
+    let (stdout, stderr) = test_env.jj_cmd_ok(&main_path, &["workspace", "add", "./third"]);
+    insta::assert_snapshot!(stdout.replace('\\', "/"), @"");
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Created workspace in "third"
+    Working copy now at: zxsnswpr 64746d4b (empty) (no description set)
+    Parent commit      : qpvuntsm 751b12b7 initial
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+
+    // Both workspaces created
+    let stdout = test_env.jj_cmd_success(&main_path, &["workspace", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
+    default: rlvkpnrz 477c647f (no description set)
+    secondary: pmmvwywv 0a77a39d (empty) (no description set)
+    third: zxsnswpr 64746d4b (empty) (no description set)
+    "###);
+
+    // Can see files from the other workspaces in main workspace, since they are
+    // child directories and will therefore be snapshotted
+    let stdout = test_env.jj_cmd_success(&main_path, &["file", "list"]);
+    insta::assert_snapshot!(stdout.replace('\\', "/"), @r###"
+    file
+    secondary/file
+    third/file
     "###);
 }
 
@@ -267,11 +444,11 @@ fn test_workspaces_conflicting_edits() {
     test_env.jj_cmd_ok(&main_path, &["workspace", "add", "../secondary"]);
 
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉  3224de8ae048 secondary@
+    ○  3224de8ae048 secondary@
     │ @  06b57f44a3ca default@
     ├─╯
-    ◉  506f4ec3c2c6
-    ◉  000000000000
+    ○  506f4ec3c2c6
+    ◆  000000000000
     "###);
 
     // Make changes in both working copies
@@ -290,21 +467,21 @@ fn test_workspaces_conflicting_edits() {
     // The secondary workspace's working-copy commit was updated
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
     @  a58c9a9b19ce default@
-    │ ◉  e82cd4ee8faa secondary@
+    │ ○  e82cd4ee8faa secondary@
     ├─╯
-    ◉  d41244767d45
-    ◉  000000000000
+    ○  d41244767d45
+    ◆  000000000000
     "###);
     let stderr = test_env.jj_cmd_failure(&secondary_path, &["st"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: The working copy is stale (not updated since operation f46ea702e886).
+    Error: The working copy is stale (not updated since operation 0da24da631e3).
     Hint: Run `jj workspace update-stale` to update it.
     See https://github.com/martinvonz/jj/blob/main/docs/working-copy.md#stale-working-copy for more information.
     "###);
     // Same error on second run, and from another command
     let stderr = test_env.jj_cmd_failure(&secondary_path, &["log"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: The working copy is stale (not updated since operation f46ea702e886).
+    Error: The working copy is stale (not updated since operation 0da24da631e3).
     Hint: Run `jj workspace update-stale` to update it.
     See https://github.com/martinvonz/jj/blob/main/docs/working-copy.md#stale-working-copy for more information.
     "###);
@@ -321,25 +498,25 @@ fn test_workspaces_conflicting_edits() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &secondary_path),
     @r###"
-    ◉  a28c85ce128b (divergent)
-    │ ◉  a58c9a9b19ce default@
+    ×  a28c85ce128b (divergent)
+    │ ○  a58c9a9b19ce default@
     ├─╯
     │ @  e82cd4ee8faa secondary@ (divergent)
     ├─╯
-    ◉  d41244767d45
-    ◉  000000000000
+    ○  d41244767d45
+    ◆  000000000000
     "###);
     // The stale working copy should have been resolved by the previous command
     let stdout = get_log_output(&test_env, &secondary_path);
     assert!(!stdout.starts_with("The working copy is stale"));
     insta::assert_snapshot!(stdout, @r###"
-    ◉  a28c85ce128b (divergent)
-    │ ◉  a58c9a9b19ce default@
+    ×  a28c85ce128b (divergent)
+    │ ○  a58c9a9b19ce default@
     ├─╯
     │ @  e82cd4ee8faa secondary@ (divergent)
     ├─╯
-    ◉  d41244767d45
-    ◉  000000000000
+    ○  d41244767d45
+    ◆  000000000000
     "###);
 }
 
@@ -357,11 +534,11 @@ fn test_workspaces_updated_by_other() {
     test_env.jj_cmd_ok(&main_path, &["workspace", "add", "../secondary"]);
 
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉  3224de8ae048 secondary@
+    ○  3224de8ae048 secondary@
     │ @  06b57f44a3ca default@
     ├─╯
-    ◉  506f4ec3c2c6
-    ◉  000000000000
+    ○  506f4ec3c2c6
+    ◆  000000000000
     "###);
 
     // Rewrite the check-out commit in one workspace.
@@ -377,14 +554,14 @@ fn test_workspaces_updated_by_other() {
     // The secondary workspace's working-copy commit was updated.
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
     @  a58c9a9b19ce default@
-    │ ◉  e82cd4ee8faa secondary@
+    │ ○  e82cd4ee8faa secondary@
     ├─╯
-    ◉  d41244767d45
-    ◉  000000000000
+    ○  d41244767d45
+    ◆  000000000000
     "###);
     let stderr = test_env.jj_cmd_failure(&secondary_path, &["st"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: The working copy is stale (not updated since operation f46ea702e886).
+    Error: The working copy is stale (not updated since operation 0da24da631e3).
     Hint: Run `jj workspace update-stale` to update it.
     See https://github.com/martinvonz/jj/blob/main/docs/working-copy.md#stale-working-copy for more information.
     "###);
@@ -398,11 +575,11 @@ fn test_workspaces_updated_by_other() {
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &secondary_path),
     @r###"
-    ◉  a58c9a9b19ce default@
+    ○  a58c9a9b19ce default@
     │ @  e82cd4ee8faa secondary@
     ├─╯
-    ◉  d41244767d45
-    ◉  000000000000
+    ○  d41244767d45
+    ◆  000000000000
     "###);
 }
 
@@ -455,16 +632,16 @@ fn test_workspaces_current_op_discarded_by_other() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    @  8e5ea0fbda abandon commit 3540d386892997a2a927078635a2d933e37499fb8691938a2f540c25bccffd9e8a60b2d5a8cb94bb3eeab17e1c56f96aafa2bcb66fa1e4eb96911d093d7a579e
-    ◉  f336f5b6e8 Create initial working-copy commit in workspace secondary
-    ◉  aacb3bda7d add workspace 'secondary'
-    ◉  46bcf7d75e new empty commit
-    ◉  4d2f5d7cbf snapshot working copy
-    ◉  2f863a1573 new empty commit
-    ◉  f01631d976 snapshot working copy
-    ◉  17dbb2fe40 add workspace 'default'
-    ◉  cecfee9647 initialize repo
-    ◉  0000000000
+    @  7337338f0b abandon commit 20dd439c4bd12c6ad56c187ac490bd0141804618f638dc5c4dc92ff9aecba20f152b23160db9dcf61beb31a5cb14091d9def5a36d11c9599cc4d2e5689236af1
+    ○  f4bd4d046b create initial working-copy commit in workspace secondary
+    ○  0f99641958 add workspace 'secondary'
+    ○  5641361f60 new empty commit
+    ○  3a6c319c59 snapshot working copy
+    ○  42c6005842 new empty commit
+    ○  6a45045541 snapshot working copy
+    ○  a9e6630bf0 add workspace 'default'
+    ○  cecfee9647 initialize repo
+    ○  0000000000
     "###);
 
     // Abandon ops, including the one the secondary workspace is currently on.
@@ -472,11 +649,11 @@ fn test_workspaces_current_op_discarded_by_other() {
     test_env.jj_cmd_ok(&main_path, &["util", "gc", "--expire=now"]);
 
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    @  cc0b087cb874 default@
-    │ ◉  376eee1462a7 secondary@
+    ○  96b31dafdc41 secondary@
+    │ @  6c051bd1ccd5 default@
     ├─╯
-    ◉  7788883a847c
-    ◉  000000000000
+    ○  7c5b25a4fc8f
+    ◆  000000000000
     "###);
 
     let stderr = test_env.jj_cmd_failure(&secondary_path, &["st"]);
@@ -488,18 +665,18 @@ fn test_workspaces_current_op_discarded_by_other() {
 
     let (stdout, stderr) = test_env.jj_cmd_ok(&secondary_path, &["workspace", "update-stale"]);
     insta::assert_snapshot!(stderr, @r###"
-    Failed to read working copy's current operation; attempting recovery. Error message from read attempt: Object f336f5b6e83bb901dce6d05d83193f7d0cad2b6375a9910d586c844a479feb130c30d417bdf3030f980d9bacca117584a654e9bdf74b41b30021651e28fbfc8c of type operation not found
-    Created and checked out recovery commit 6803354995e6
+    Failed to read working copy's current operation; attempting recovery. Error message from read attempt: Object f4bd4d046b3cdf61b0fda7738a0b1414c0aedc6c8229d39a35ee26facc358cad8b588b04d7eba1302a82409c529f69dbb1ff9ea28789d935b74f123f377aa30b of type operation not found
+    Created and checked out recovery commit 62f70695e3b0
     "###);
     insta::assert_snapshot!(stdout, @"");
 
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉  a8f7db7868c1 secondary@
-    ◉  376eee1462a7
-    │ @  cc0b087cb874 default@
+    ○  b0b400439a82 secondary@
+    ○  96b31dafdc41
+    │ @  6c051bd1ccd5 default@
     ├─╯
-    ◉  7788883a847c
-    ◉  000000000000
+    ○  7c5b25a4fc8f
+    ◆  000000000000
     "###);
 
     // The sparse patterns should remain
@@ -516,8 +693,8 @@ fn test_workspaces_current_op_discarded_by_other() {
     A added
     D deleted
     M modified
-    Working copy : kmkuslsw a8f7db78 (no description set)
-    Parent commit: rzvqmyuk 376eee14 (empty) (no description set)
+    Working copy : kmkuslsw b0b40043 (no description set)
+    Parent commit: rzvqmyuk 96b31daf (empty) (no description set)
     "###);
     // The modified file should have the same contents it had before (not reset to
     // the base contents)
@@ -528,9 +705,9 @@ fn test_workspaces_current_op_discarded_by_other() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&secondary_path, &["obslog"]);
     insta::assert_snapshot!(stderr, @"");
     insta::assert_snapshot!(stdout, @r###"
-    @  kmkuslsw test.user@example.com 2001-02-03 08:05:18 secondary@ a8f7db78
+    @  kmkuslsw test.user@example.com 2001-02-03 08:05:18 secondary@ b0b40043
     │  (no description set)
-    ◉  kmkuslsw hidden test.user@example.com 2001-02-03 08:05:18 68033549
+    ○  kmkuslsw hidden test.user@example.com 2001-02-03 08:05:18 62f70695
        (empty) (no description set)
     "###);
 }
@@ -559,8 +736,8 @@ fn test_workspaces_update_stale_noop() {
     let stdout = test_env.jj_cmd_success(&main_path, &["op", "log", "-Tdescription"]);
     insta::assert_snapshot!(stdout, @r###"
     @  add workspace 'default'
-    ◉  initialize repo
-    ◉
+    ○  initialize repo
+    ○
     "###);
 }
 
@@ -591,11 +768,11 @@ fn test_workspaces_update_stale_snapshot() {
 
     insta::assert_snapshot!(get_log_output(&test_env, &secondary_path), @r###"
     @  e672fd8fefac secondary@
-    │ ◉  ea37b073f5ab default@
-    │ ◉  b13c81dedc64
+    │ ○  ea37b073f5ab default@
+    │ ○  b13c81dedc64
     ├─╯
-    ◉  e6e9989f1179
-    ◉  000000000000
+    ○  e6e9989f1179
+    ◆  000000000000
     "###);
 }
 
@@ -628,16 +805,13 @@ fn test_workspaces_forget() {
     insta::assert_snapshot!(stderr, @"");
 
     // The old working copy doesn't get an "@" in the log output
-    // TODO: We should abandon the empty working copy commit
     // TODO: It seems useful to still have the "secondary@" marker here even though
     // there's only one workspace. We should show it when the command is not run
     // from that workspace.
     insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
-    ◉  18463f438cc9
-    │ ◉  909d51b17292
-    ├─╯
-    ◉  4e8f9d2be039
-    ◉  000000000000
+    ○  18463f438cc9
+    ○  4e8f9d2be039
+    ◆  000000000000
     "###);
 
     // Revision "@" cannot be used
@@ -695,7 +869,7 @@ fn test_workspaces_forget_multi_transaction() {
     // the op log should have multiple workspaces forgotten in a single tx
     let stdout = test_env.jj_cmd_success(&main_path, &["op", "log", "--limit", "1"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  6c88cdee70e6 test-username@host.example.com 2001-02-03 04:05:12.000 +07:00 - 2001-02-03 04:05:12.000 +07:00
+    @  b7ab9f1c16cc test-username@host.example.com 2001-02-03 04:05:12.000 +07:00 - 2001-02-03 04:05:12.000 +07:00
     │  forget workspaces second, third
     │  args: jj workspace forget second third
     "###);
@@ -712,6 +886,65 @@ fn test_workspaces_forget_multi_transaction() {
     "###);
 }
 
+#[test]
+fn test_workspaces_forget_abandon_commits() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "main"]);
+    let main_path = test_env.env_root().join("main");
+
+    std::fs::write(main_path.join("file"), "contents").unwrap();
+
+    test_env.jj_cmd_ok(&main_path, &["workspace", "add", "../second"]);
+    test_env.jj_cmd_ok(&main_path, &["workspace", "add", "../third"]);
+    test_env.jj_cmd_ok(&main_path, &["workspace", "add", "../fourth"]);
+    let third_path = test_env.env_root().join("third");
+    test_env.jj_cmd_ok(&third_path, &["edit", "second@"]);
+    let fourth_path = test_env.env_root().join("fourth");
+    test_env.jj_cmd_ok(&fourth_path, &["edit", "second@"]);
+
+    // there should be four workspaces, three of which are at the same empty commit
+    let stdout = test_env.jj_cmd_success(&main_path, &["workspace", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
+    default: qpvuntsm 4e8f9d2b (no description set)
+    fourth: uuqppmxq 57d63245 (empty) (no description set)
+    second: uuqppmxq 57d63245 (empty) (no description set)
+    third: uuqppmxq 57d63245 (empty) (no description set)
+    "###);
+    insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
+    ○  57d63245a308 fourth@ second@ third@
+    │ @  4e8f9d2be039 default@
+    ├─╯
+    ◆  000000000000
+    "###);
+
+    // delete the default workspace (should not abandon commit since not empty)
+    test_env.jj_cmd_success(&main_path, &["workspace", "forget", "default"]);
+    insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
+    ○  57d63245a308 fourth@ second@ third@
+    │ ○  4e8f9d2be039
+    ├─╯
+    ◆  000000000000
+    "###);
+
+    // delete the second workspace (should not abandon commit since other workspaces
+    // still have commit checked out)
+    test_env.jj_cmd_success(&main_path, &["workspace", "forget", "second"]);
+    insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
+    ○  57d63245a308 fourth@ third@
+    │ ○  4e8f9d2be039
+    ├─╯
+    ◆  000000000000
+    "###);
+
+    // delete the last 2 workspaces (commit should be abandoned now even though
+    // forgotten in same tx)
+    test_env.jj_cmd_success(&main_path, &["workspace", "forget", "third", "fourth"]);
+    insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r###"
+    ○  4e8f9d2be039
+    ◆  000000000000
+    "###);
+}
+
 /// Test context of commit summary template
 #[test]
 fn test_list_workspaces_template() {
@@ -795,11 +1028,11 @@ fn test_debug_snapshot() {
     @  e1e762d39b39 test-username@host.example.com 2001-02-03 04:05:08.000 +07:00 - 2001-02-03 04:05:08.000 +07:00
     │  snapshot working copy
     │  args: jj debug snapshot
-    ◉  b51416386f26 test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
+    ○  b51416386f26 test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
     │  add workspace 'default'
-    ◉  9a7d829846af test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
+    ○  9a7d829846af test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
     │  initialize repo
-    ◉  000000000000 root()
+    ○  000000000000 root()
     "###);
     test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "initial"]);
     let stdout = test_env.jj_cmd_success(&repo_path, &["op", "log"]);
@@ -807,14 +1040,14 @@ fn test_debug_snapshot() {
     @  9ac6e7144e8a test-username@host.example.com 2001-02-03 04:05:10.000 +07:00 - 2001-02-03 04:05:10.000 +07:00
     │  describe commit 4e8f9d2be039994f589b4e57ac5e9488703e604d
     │  args: jj describe -m initial
-    ◉  e1e762d39b39 test-username@host.example.com 2001-02-03 04:05:08.000 +07:00 - 2001-02-03 04:05:08.000 +07:00
+    ○  e1e762d39b39 test-username@host.example.com 2001-02-03 04:05:08.000 +07:00 - 2001-02-03 04:05:08.000 +07:00
     │  snapshot working copy
     │  args: jj debug snapshot
-    ◉  b51416386f26 test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
+    ○  b51416386f26 test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
     │  add workspace 'default'
-    ◉  9a7d829846af test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
+    ○  9a7d829846af test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
     │  initialize repo
-    ◉  000000000000 root()
+    ○  000000000000 root()
     "###);
 }
 
