@@ -17,16 +17,17 @@
 use std::io::{Read, Write};
 use std::iter::zip;
 
-use futures::{StreamExt, TryStreamExt};
+use futures::{try_join, Stream, StreamExt, TryStreamExt};
 use itertools::Itertools;
-use regex::bytes::Regex;
+use regex::bytes::{Regex, RegexBuilder};
 
 use crate::backend::{BackendError, BackendResult, CommitId, FileId, SymlinkId, TreeId, TreeValue};
-use crate::diff::{find_line_ranges, Diff, DiffHunk};
+use crate::diff::{Diff, DiffHunk};
 use crate::files;
 use crate::files::{ContentHunk, MergeResult};
 use crate::merge::{Merge, MergeBuilder, MergedTreeValue};
-use crate::repo_path::RepoPath;
+use crate::merged_tree::TreeDiffStream;
+use crate::repo_path::{RepoPath, RepoPathBuf};
 use crate::store::Store;
 
 const CONFLICT_START_LINE: &[u8] = b"<<<<<<<";
@@ -46,11 +47,10 @@ const CONFLICT_PLUS_LINE_CHAR: u8 = CONFLICT_PLUS_LINE[0];
 // separators. This could be useful to make it possible to allow conflict
 // markers inside the text of the conflicts.
 static CONFLICT_MARKER_REGEX: once_cell::sync::Lazy<Regex> = once_cell::sync::Lazy::new(|| {
-    Regex::new(
-        r"(<{7}|>{7}|%{7}|\-{7}|\+{7})( .*)?
-",
-    )
-    .unwrap()
+    RegexBuilder::new(r"^(<{7}|>{7}|%{7}|\-{7}|\+{7})( .*)?$")
+        .multi_line(true)
+        .build()
+        .unwrap()
 });
 
 fn write_diff_hunks(hunks: &[DiffHunk], file: &mut dyn Write) -> std::io::Result<()> {
@@ -218,8 +218,7 @@ pub fn materialize_merge_result(
     single_hunk: &Merge<ContentHunk>,
     output: &mut dyn Write,
 ) -> std::io::Result<()> {
-    let slices = single_hunk.map(|content| content.0.as_slice());
-    let merge_result = files::merge(&slices);
+    let merge_result = files::merge(single_hunk);
     match merge_result {
         MergeResult::Resolved(content) => {
             output.write_all(&content.0)?;
@@ -259,17 +258,12 @@ pub fn materialize_merge_result(
                             output.write_all(&left.0)?;
                             continue;
                         };
-                        let diff1 = Diff::for_tokenizer(&[&left.0, &right1.0], find_line_ranges)
-                            .hunks()
-                            .collect_vec();
+                        let diff1 = Diff::by_line([&left.0, &right1.0]).hunks().collect_vec();
                         // Check if the diff against the next positive term is better. Since
                         // we want to preserve the order of the terms, we don't match against
                         // any later positive terms.
                         if let Some(right2) = hunk.get_add(add_index + 1) {
-                            let diff2 =
-                                Diff::for_tokenizer(&[&left.0, &right2.0], find_line_ranges)
-                                    .hunks()
-                                    .collect_vec();
+                            let diff2 = Diff::by_line([&left.0, &right2.0]).hunks().collect_vec();
                             if diff_size(&diff2) < diff_size(&diff1) {
                                 // If the next positive term is a better match, emit
                                 // the current positive term as a snapshot and the next
@@ -331,6 +325,30 @@ fn diff_size(hunks: &[DiffHunk]) -> usize {
         .sum()
 }
 
+pub fn materialized_diff_stream<'a>(
+    store: &'a Store,
+    tree_diff: TreeDiffStream<'a>,
+) -> impl Stream<
+    Item = (
+        RepoPathBuf,
+        BackendResult<(MaterializedTreeValue, MaterializedTreeValue)>,
+    ),
+> + 'a {
+    tree_diff
+        .map(|(path, diff)| async {
+            match diff {
+                Err(err) => (path, Err(err)),
+                Ok((before, after)) => {
+                    let before_future = materialize_tree_value(store, &path, before);
+                    let after_future = materialize_tree_value(store, &path, after);
+                    let values = try_join!(before_future, after_future);
+                    (path, values)
+                }
+            }
+        })
+        .buffered((store.concurrency() / 2).max(1))
+}
+
 /// Parses conflict markers from a slice. Returns None if there were no valid
 /// conflict markers. The caller has to provide the expected number of merge
 /// sides (adds). Conflict markers that are otherwise valid will be considered
