@@ -20,14 +20,18 @@ use std::collections::{BTreeSet, BinaryHeap, HashSet};
 use std::ops::Range;
 use std::rc::Rc;
 use std::sync::Arc;
-use std::{fmt, iter};
+use std::{fmt, iter, str};
 
+use futures::StreamExt as _;
 use itertools::Itertools;
+use pollster::FutureExt as _;
 
 use super::rev_walk::{EagerRevWalk, PeekableRevWalk, RevWalk, RevWalkBuilder};
 use super::revset_graph_iterator::RevsetGraphWalk;
-use crate::backend::{ChangeId, CommitId, MillisSinceEpoch};
-use crate::default_index::{AsCompositeIndex, CompositeIndex, IndexEntry, IndexPosition};
+use crate::backend::{BackendError, BackendResult, ChangeId, CommitId, MillisSinceEpoch};
+use crate::commit::Commit;
+use crate::conflicts::{materialized_diff_stream, MaterializedTreeValue};
+use crate::default_index::{AsCompositeIndex, CompositeIndex, IndexPosition};
 use crate::graph::GraphEdge;
 use crate::matchers::{Matcher, Visit};
 use crate::repo_path::RepoPath;
@@ -36,6 +40,7 @@ use crate::revset::{
     RevsetFilterPredicate, GENERATION_RANGE_FULL,
 };
 use crate::store::Store;
+use crate::str_util::StringPattern;
 use crate::{rewrite, union_find};
 
 type BoxedPredicateFn<'a> = Box<dyn FnMut(&CompositeIndex, IndexPosition) -> bool + 'a>;
@@ -1033,6 +1038,7 @@ fn build_predicate_fn(
     store: Arc<Store>,
     predicate: &RevsetFilterPredicate,
 ) -> Box<dyn ToPredicateFn> {
+    // TODO: propagate BackendError
     match predicate {
         RevsetFilterPredicate::ParentCount(parent_count_range) => {
             let parent_count_range = parent_count_range.clone();
@@ -1052,8 +1058,7 @@ fn build_predicate_fn(
         RevsetFilterPredicate::Author(pattern) => {
             let pattern = pattern.clone();
             // TODO: Make these functions that take a needle to search for accept some
-            // syntax for specifying whether it's a regex and whether it's
-            // case-sensitive.
+            // syntax for specifying whether it's a regex.
             box_pure_predicate_fn(move |index, pos| {
                 let entry = index.entry_by_pos(pos);
                 let commit = store.get_commit(&entry.commit_id()).unwrap();
@@ -1069,11 +1074,40 @@ fn build_predicate_fn(
                     || pattern.matches(&commit.committer().email)
             })
         }
+        RevsetFilterPredicate::AuthorDate(expression) => {
+            let expression = *expression;
+            box_pure_predicate_fn(move |index, pos| {
+                let entry = index.entry_by_pos(pos);
+                let commit = store.get_commit(&entry.commit_id()).unwrap();
+                let author_date = &commit.author().timestamp;
+                expression.matches(author_date)
+            })
+        }
+        RevsetFilterPredicate::CommitterDate(expression) => {
+            let expression = *expression;
+            box_pure_predicate_fn(move |index, pos| {
+                let entry = index.entry_by_pos(pos);
+                let commit = store.get_commit(&entry.commit_id()).unwrap();
+                let committer_date = &commit.committer().timestamp;
+                expression.matches(committer_date)
+            })
+        }
         RevsetFilterPredicate::File(expr) => {
             let matcher: Rc<dyn Matcher> = expr.to_matcher().into();
             box_pure_predicate_fn(move |index, pos| {
                 let entry = index.entry_by_pos(pos);
-                has_diff_from_parent(&store, index, &entry, matcher.as_ref())
+                let commit = store.get_commit(&entry.commit_id()).unwrap();
+                has_diff_from_parent(&store, index, &commit, matcher.as_ref()).unwrap()
+            })
+        }
+        RevsetFilterPredicate::DiffContains { text, files } => {
+            let text_pattern = text.clone();
+            let files_matcher: Rc<dyn Matcher> = files.to_matcher().into();
+            box_pure_predicate_fn(move |index, pos| {
+                let entry = index.entry_by_pos(pos);
+                let commit = store.get_commit(&entry.commit_id()).unwrap();
+                matches_diff_from_parent(&store, index, &commit, &text_pattern, &*files_matcher)
+                    .unwrap()
             })
         }
         RevsetFilterPredicate::HasConflict => box_pure_predicate_fn(move |index, pos| {
@@ -1095,23 +1129,91 @@ fn build_predicate_fn(
 fn has_diff_from_parent(
     store: &Arc<Store>,
     index: &CompositeIndex,
-    entry: &IndexEntry<'_>,
+    commit: &Commit,
     matcher: &dyn Matcher,
-) -> bool {
-    let commit = store.get_commit(&entry.commit_id()).unwrap();
-    let parents: Vec<_> = commit.parents().try_collect().unwrap();
+) -> BackendResult<bool> {
+    let parents: Vec<_> = commit.parents().try_collect()?;
     if let [parent] = parents.as_slice() {
         // Fast path: no need to load the root tree
         let unchanged = commit.tree_id() == parent.tree_id();
         if matcher.visit(RepoPath::root()) == Visit::AllRecursively {
-            return !unchanged;
+            return Ok(!unchanged);
         } else if unchanged {
-            return false;
+            return Ok(false);
+        }
+    }
+    let from_tree = rewrite::merge_commit_trees_without_repo(store, &index, &parents)?;
+    let to_tree = commit.tree()?;
+    Ok(from_tree.diff(&to_tree, matcher).next().is_some())
+}
+
+fn matches_diff_from_parent(
+    store: &Arc<Store>,
+    index: &CompositeIndex,
+    commit: &Commit,
+    text_pattern: &StringPattern,
+    files_matcher: &dyn Matcher,
+) -> BackendResult<bool> {
+    let parents: Vec<_> = commit.parents().try_collect()?;
+    let from_tree = rewrite::merge_commit_trees_without_repo(store, &index, &parents)?;
+    let to_tree = commit.tree()?;
+    let tree_diff = from_tree.diff_stream(&to_tree, files_matcher);
+    // Conflicts are compared in materialized form. Alternatively, conflict
+    // pairs can be compared one by one. #4062
+    let mut diff_stream = materialized_diff_stream(store, tree_diff);
+    async {
+        while let Some((path, diff)) = diff_stream.next().await {
+            let (left_value, right_value) = diff?;
+            let left_content = to_file_content(&path, left_value)?;
+            let right_content = to_file_content(&path, right_value)?;
+            // Filter lines prior to comparison. This might produce inferior
+            // hunks due to lack of contexts, but is way faster than full diff.
+            let left_lines = match_lines(&left_content, text_pattern);
+            let right_lines = match_lines(&right_content, text_pattern);
+            if left_lines.ne(right_lines) {
+                return Ok(true);
+            }
+        }
+        Ok(false)
+    }
+    .block_on()
+}
+
+fn match_lines<'a: 'b, 'b>(
+    text: &'a [u8],
+    pattern: &'b StringPattern,
+) -> impl Iterator<Item = &'a [u8]> + 'b {
+    // The pattern is matched line by line so that it can be anchored to line
+    // start/end. For example, exact:"" will match blank lines.
+    text.split_inclusive(|b| *b == b'\n').filter(|line| {
+        let line = line.strip_suffix(b"\n").unwrap_or(line);
+        // TODO: add .matches_bytes() or .to_bytes_matcher()
+        str::from_utf8(line).map_or(false, |line| pattern.matches(line))
+    })
+}
+
+fn to_file_content(path: &RepoPath, value: MaterializedTreeValue) -> BackendResult<Vec<u8>> {
+    match value {
+        MaterializedTreeValue::Absent => Ok(vec![]),
+        MaterializedTreeValue::AccessDenied(_) => Ok(vec![]),
+        MaterializedTreeValue::File { id, mut reader, .. } => {
+            let mut content = vec![];
+            reader
+                .read_to_end(&mut content)
+                .map_err(|err| BackendError::ReadFile {
+                    path: path.to_owned(),
+                    id: id.clone(),
+                    source: err.into(),
+                })?;
+            Ok(content)
+        }
+        MaterializedTreeValue::Symlink { id: _, target } => Ok(target.into_bytes()),
+        MaterializedTreeValue::GitSubmodule(_) => Ok(vec![]),
+        MaterializedTreeValue::Conflict { contents, .. } => Ok(contents),
+        MaterializedTreeValue::Tree(id) => {
+            panic!("Unexpected tree with id {id:?} in diff at path {path:?}");
         }
     }
-    let from_tree = rewrite::merge_commit_trees_without_repo(store, &index, &parents).unwrap();
-    let to_tree = commit.tree().unwrap();
-    from_tree.diff(&to_tree, matcher).next().is_some()
 }
 
 #[cfg(test)]
