@@ -16,10 +16,10 @@
 
 use std::cmp::{max, min, Ordering};
 use std::collections::{BTreeMap, HashMap};
-use std::fmt::{Debug, Formatter};
 use std::ops::Range;
-use std::slice;
+use std::{iter, slice};
 
+use bstr::BStr;
 use itertools::Itertools;
 
 pub fn find_line_ranges(text: &[u8]) -> Vec<Range<usize>> {
@@ -168,56 +168,84 @@ pub(crate) fn unchanged_ranges(
         return vec![];
     }
 
+    // Prioritize LCS-based algorithm than leading/trailing matches
+    let result = unchanged_ranges_lcs(left, right, left_ranges, right_ranges);
+    if !result.is_empty() {
+        return result;
+    }
+
+    // Trim leading common ranges (i.e. grow previous unchanged region)
+    let common_leading_len = iter::zip(left_ranges, right_ranges)
+        .take_while(|&(l, r)| left[l.clone()] == right[r.clone()])
+        .count();
+    let (left_leading_ranges, left_ranges) = left_ranges.split_at(common_leading_len);
+    let (right_leading_ranges, right_ranges) = right_ranges.split_at(common_leading_len);
+
+    // Trim trailing common ranges (i.e. grow next unchanged region)
+    let common_trailing_len = iter::zip(left_ranges.iter().rev(), right_ranges.iter().rev())
+        .take_while(|&(l, r)| left[l.clone()] == right[r.clone()])
+        .count();
+    let left_trailing_ranges = &left_ranges[(left_ranges.len() - common_trailing_len)..];
+    let right_trailing_ranges = &right_ranges[(right_ranges.len() - common_trailing_len)..];
+
+    itertools::chain(
+        iter::zip(
+            left_leading_ranges.iter().cloned(),
+            right_leading_ranges.iter().cloned(),
+        ),
+        iter::zip(
+            left_trailing_ranges.iter().cloned(),
+            right_trailing_ranges.iter().cloned(),
+        ),
+    )
+    .collect()
+}
+
+fn unchanged_ranges_lcs(
+    left: &[u8],
+    right: &[u8],
+    left_ranges: &[Range<usize>],
+    right_ranges: &[Range<usize>],
+) -> Vec<(Range<usize>, Range<usize>)> {
     let max_occurrences = 100;
-    let mut left_histogram = Histogram::calculate(left, left_ranges, max_occurrences);
+    let left_histogram = Histogram::calculate(left, left_ranges, max_occurrences);
     if *left_histogram.count_to_words.keys().next().unwrap() > max_occurrences {
         // If there are very many occurrences of all words, then we just give up.
         return vec![];
     }
-    let mut right_histogram = Histogram::calculate(right, right_ranges, max_occurrences);
+    let right_histogram = Histogram::calculate(right, right_ranges, max_occurrences);
     // Look for words with few occurrences in `left` (could equally well have picked
     // `right`?). If any of them also occur in `right`, then we add the words to
     // the LCS.
-    let mut uncommon_shared_words = vec![];
-    while !left_histogram.count_to_words.is_empty() && uncommon_shared_words.is_empty() {
-        let left_words = left_histogram
-            .count_to_words
-            .first_entry()
-            .map(|x| x.remove())
-            .unwrap();
-        for left_word in left_words {
-            if right_histogram.word_to_positions.contains_key(left_word) {
-                uncommon_shared_words.push(left_word);
-            }
-        }
-    }
-    if uncommon_shared_words.is_empty() {
+    let Some(uncommon_shared_words) = left_histogram
+        .count_to_words
+        .iter()
+        .map(|(left_count, left_words)| -> Vec<&[u8]> {
+            left_words
+                .iter()
+                .copied()
+                .filter(|left_word| {
+                    let right_count = right_histogram
+                        .word_to_positions
+                        .get(left_word)
+                        .map_or(0, |right_positions| right_positions.len());
+                    *left_count == right_count
+                })
+                .collect()
+        })
+        .find(|words| !words.is_empty())
+    else {
         return vec![];
-    }
-
-    // Let's say our inputs are "a b a b" and "a b c c b a b". We will have found
-    // the least common words to be "a" and "b". We now assume that each
-    // occurrence of each word lines up in the left and right input. We do that
-    // by numbering the shared occurrences, effectively instead comparing "a1 b1
-    // a2 b2" and "a1 b1 c c b2 a2 b". We then walk the common words in the
-    // right input in order (["a1", "b1", "b2", "a2"]), and record the index of
-    // that word in the left input ([0,1,3,2]). We then find the LCS and split
-    // points based on that ([0,1,3] or [0,1,2] are both valid).
+    };
 
     // [(index into left_ranges, word, occurrence #)]
     let mut left_positions = vec![];
     let mut right_positions = vec![];
     for uncommon_shared_word in uncommon_shared_words {
-        let left_occurrences = left_histogram
-            .word_to_positions
-            .get_mut(uncommon_shared_word)
-            .unwrap();
-        let right_occurrences = right_histogram
-            .word_to_positions
-            .get_mut(uncommon_shared_word)
-            .unwrap();
-        let shared_count = min(left_occurrences.len(), right_occurrences.len());
-        for occurrence in 0..shared_count {
+        let left_occurrences = &left_histogram.word_to_positions[uncommon_shared_word];
+        let right_occurrences = &right_histogram.word_to_positions[uncommon_shared_word];
+        assert_eq!(left_occurrences.len(), right_occurrences.len());
+        for occurrence in 0..left_occurrences.len() {
             left_positions.push((
                 left_occurrences[occurrence],
                 uncommon_shared_word,
@@ -326,8 +354,8 @@ impl Ord for UnchangedRange {
 /// of them.
 #[derive(Clone, Debug)]
 pub struct Diff<'input> {
-    base_input: &'input [u8],
-    other_inputs: Vec<&'input [u8]>,
+    base_input: &'input BStr,
+    other_inputs: Vec<&'input BStr>,
     // The key is a range in the base input. The value is the start of each non-base region
     // relative to the base region's start. By making them relative, they don't need to change
     // when the base range changes.
@@ -380,20 +408,33 @@ fn intersect_regions(
 }
 
 impl<'input> Diff<'input> {
-    pub fn for_tokenizer(
-        inputs: &[&'input [u8]],
+    pub fn for_tokenizer<T: AsRef<[u8]> + ?Sized + 'input>(
+        inputs: impl IntoIterator<Item = &'input T>,
         tokenizer: impl Fn(&[u8]) -> Vec<Range<usize>>,
     ) -> Self {
-        assert!(!inputs.is_empty());
-        let base_input = inputs[0];
-        let other_inputs = inputs.iter().skip(1).copied().collect_vec();
+        let mut inputs = inputs.into_iter().map(BStr::new);
+        let base_input = inputs.next().expect("inputs must not be empty");
+        let other_inputs = inputs.collect_vec();
         // First tokenize each input
-        let base_token_ranges: Vec<Range<usize>> = tokenizer(base_input);
-        let other_token_ranges: Vec<Vec<Range<usize>>> = other_inputs
+        let base_token_ranges = tokenizer(base_input);
+        let other_token_ranges = other_inputs
             .iter()
             .map(|other_input| tokenizer(other_input))
             .collect_vec();
+        Self::with_inputs_and_token_ranges(
+            base_input,
+            other_inputs,
+            &base_token_ranges,
+            &other_token_ranges,
+        )
+    }
 
+    fn with_inputs_and_token_ranges(
+        base_input: &'input BStr,
+        other_inputs: Vec<&'input BStr>,
+        base_token_ranges: &[Range<usize>],
+        other_token_ranges: &[Vec<Range<usize>>],
+    ) -> Self {
         // Look for unchanged regions. Initially consider the whole range of the base
         // input as unchanged (compared to itself). Then diff each other input
         // against the base. Intersect the previously found ranges with the
@@ -406,7 +447,7 @@ impl<'input> Diff<'input> {
             let unchanged_diff_ranges = unchanged_ranges(
                 base_input,
                 other_inputs[i],
-                &base_token_ranges,
+                base_token_ranges,
                 other_token_ranges,
             );
             unchanged_regions = intersect_regions(unchanged_regions, &unchanged_diff_ranges);
@@ -430,17 +471,28 @@ impl<'input> Diff<'input> {
         diff
     }
 
-    pub fn unrefined(inputs: &[&'input [u8]]) -> Self {
+    pub fn unrefined<T: AsRef<[u8]> + ?Sized + 'input>(
+        inputs: impl IntoIterator<Item = &'input T>,
+    ) -> Self {
         Diff::for_tokenizer(inputs, |_| vec![])
     }
 
+    /// Compares `inputs` line by line.
+    pub fn by_line<T: AsRef<[u8]> + ?Sized + 'input>(
+        inputs: impl IntoIterator<Item = &'input T>,
+    ) -> Self {
+        Diff::for_tokenizer(inputs, find_line_ranges)
+    }
+
     // TODO: At least when merging, it's wasteful to refine the diff if e.g. if 2
     // out of 3 inputs match in the differing regions. Perhaps the refine()
     // method should be on the hunk instead (probably returning a new Diff)?
     // That would let each user decide which hunks to refine. However, it would
     // probably mean that many callers repeat the same code. Perhaps it
     // should be possible to refine a whole diff *or* individual hunks.
-    pub fn default_refinement(inputs: &[&'input [u8]]) -> Self {
+    pub fn default_refinement<T: AsRef<[u8]> + ?Sized + 'input>(
+        inputs: impl IntoIterator<Item = &'input T>,
+    ) -> Self {
         let mut diff = Diff::for_tokenizer(inputs, find_line_ranges);
         diff.refine_changed_regions(find_word_ranges);
         diff.refine_changed_regions(find_nonword_ranges);
@@ -480,7 +532,7 @@ impl<'input> Diff<'input> {
                 slices.push(&self.other_inputs[i][changed_range]);
             }
 
-            let refined_diff = Diff::for_tokenizer(&slices, &tokenizer);
+            let refined_diff = Diff::for_tokenizer(slices, &tokenizer);
 
             for UnchangedRange {
                 base_range,
@@ -535,29 +587,21 @@ impl<'input> Diff<'input> {
     }
 }
 
-#[derive(PartialEq, Eq, Clone)]
+#[derive(PartialEq, Eq, Clone, Debug)]
 pub enum DiffHunk<'input> {
-    Matching(&'input [u8]),
-    Different(Vec<&'input [u8]>),
+    Matching(&'input BStr),
+    Different(Vec<&'input BStr>),
 }
 
-impl Debug for DiffHunk<'_> {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), std::fmt::Error> {
-        match self {
-            DiffHunk::Matching(slice) => f
-                .debug_tuple("DiffHunk::Matching")
-                .field(&String::from_utf8_lossy(slice))
-                .finish(),
-            DiffHunk::Different(slices) => f
-                .debug_tuple("DiffHunk::Different")
-                .field(
-                    &slices
-                        .iter()
-                        .map(|slice| String::from_utf8_lossy(slice))
-                        .collect_vec(),
-                )
-                .finish(),
-        }
+impl<'input> DiffHunk<'input> {
+    pub fn matching<T: AsRef<[u8]> + ?Sized>(content: &'input T) -> Self {
+        DiffHunk::Matching(BStr::new(content))
+    }
+
+    pub fn different<T: AsRef<[u8]> + ?Sized + 'input>(
+        contents: impl IntoIterator<Item = &'input T>,
+    ) -> Self {
+        DiffHunk::Different(contents.into_iter().map(BStr::new).collect())
     }
 }
 
@@ -608,16 +652,16 @@ impl<'diff, 'input> Iterator for DiffHunkIterator<'diff, 'input> {
 /// the changed ranges at the word level.
 pub fn diff<'a>(left: &'a [u8], right: &'a [u8]) -> Vec<DiffHunk<'a>> {
     if left == right {
-        return vec![DiffHunk::Matching(left)];
+        return vec![DiffHunk::matching(left)];
     }
     if left.is_empty() {
-        return vec![DiffHunk::Different(vec![b"", right])];
+        return vec![DiffHunk::different([b"", right])];
     }
     if right.is_empty() {
-        return vec![DiffHunk::Different(vec![left, b""])];
+        return vec![DiffHunk::different([left, b""])];
     }
 
-    Diff::default_refinement(&[left, right])
+    Diff::default_refinement([left, right])
         .hunks()
         .collect_vec()
 }
@@ -758,6 +802,8 @@ mod tests {
 
     #[test]
     fn test_unchanged_ranges_non_unique_removed() {
+        // We used to consider the first two "a" in the first input to match the two
+        // "a"s in the second input. We no longer do.
         assert_eq!(
             unchanged_ranges(
                 b"a a a a",
@@ -765,12 +811,41 @@ mod tests {
                 &[0..1, 2..3, 4..5, 6..7],
                 &[0..1, 2..3, 4..5, 6..7],
             ),
-            vec![(0..1, 0..1), (2..3, 4..5)]
+            vec![(0..1, 0..1)]
+        );
+        assert_eq!(
+            unchanged_ranges(
+                b"a a a a",
+                b"b a c a",
+                &[0..1, 2..3, 4..5, 6..7],
+                &[0..1, 2..3, 4..5, 6..7],
+            ),
+            vec![(6..7, 6..7)]
+        );
+        assert_eq!(
+            unchanged_ranges(
+                b"a a a a",
+                b"b a a c",
+                &[0..1, 2..3, 4..5, 6..7],
+                &[0..1, 2..3, 4..5, 6..7],
+            ),
+            vec![]
+        );
+        assert_eq!(
+            unchanged_ranges(
+                b"a a a a",
+                b"a b c a",
+                &[0..1, 2..3, 4..5, 6..7],
+                &[0..1, 2..3, 4..5, 6..7],
+            ),
+            vec![(0..1, 0..1), (6..7, 6..7)]
         );
     }
 
     #[test]
     fn test_unchanged_ranges_non_unique_added() {
+        // We used to consider the first two "a" in the first input to match the two
+        // "a"s in the second input. We no longer do.
         assert_eq!(
             unchanged_ranges(
                 b"a b a c",
@@ -778,7 +853,34 @@ mod tests {
                 &[0..1, 2..3, 4..5, 6..7],
                 &[0..1, 2..3, 4..5, 6..7],
             ),
-            vec![(0..1, 0..1), (4..5, 2..3)]
+            vec![(0..1, 0..1)]
+        );
+        assert_eq!(
+            unchanged_ranges(
+                b"b a c a",
+                b"a a a a",
+                &[0..1, 2..3, 4..5, 6..7],
+                &[0..1, 2..3, 4..5, 6..7],
+            ),
+            vec![(6..7, 6..7)]
+        );
+        assert_eq!(
+            unchanged_ranges(
+                b"b a a c",
+                b"a a a a",
+                &[0..1, 2..3, 4..5, 6..7],
+                &[0..1, 2..3, 4..5, 6..7],
+            ),
+            vec![]
+        );
+        assert_eq!(
+            unchanged_ranges(
+                b"a b c a",
+                b"a a a a",
+                &[0..1, 2..3, 4..5, 6..7],
+                &[0..1, 2..3, 4..5, 6..7],
+            ),
+            vec![(0..1, 0..1), (6..7, 6..7)]
         );
     }
 
@@ -863,52 +965,52 @@ mod tests {
 
     #[test]
     fn test_diff_single_input() {
-        let diff = Diff::default_refinement(&[b"abc"]);
-        assert_eq!(diff.hunks().collect_vec(), vec![DiffHunk::Matching(b"abc")]);
+        let diff = Diff::default_refinement(["abc"]);
+        assert_eq!(diff.hunks().collect_vec(), vec![DiffHunk::matching("abc")]);
     }
 
     #[test]
     fn test_diff_single_empty_input() {
-        let diff = Diff::default_refinement(&[b""]);
+        let diff = Diff::default_refinement([""]);
         assert_eq!(diff.hunks().collect_vec(), vec![]);
     }
 
     #[test]
     fn test_diff_two_inputs_one_different() {
-        let diff = Diff::default_refinement(&[b"a b c", b"a X c"]);
+        let diff = Diff::default_refinement(["a b c", "a X c"]);
         assert_eq!(
             diff.hunks().collect_vec(),
             vec![
-                DiffHunk::Matching(b"a "),
-                DiffHunk::Different(vec![b"b", b"X"]),
-                DiffHunk::Matching(b" c"),
+                DiffHunk::matching("a "),
+                DiffHunk::different(["b", "X"]),
+                DiffHunk::matching(" c"),
             ]
         );
     }
 
     #[test]
     fn test_diff_multiple_inputs_one_different() {
-        let diff = Diff::default_refinement(&[b"a b c", b"a X c", b"a b c"]);
+        let diff = Diff::default_refinement(["a b c", "a X c", "a b c"]);
         assert_eq!(
             diff.hunks().collect_vec(),
             vec![
-                DiffHunk::Matching(b"a "),
-                DiffHunk::Different(vec![b"b", b"X", b"b"]),
-                DiffHunk::Matching(b" c"),
+                DiffHunk::matching("a "),
+                DiffHunk::different(["b", "X", "b"]),
+                DiffHunk::matching(" c"),
             ]
         );
     }
 
     #[test]
     fn test_diff_multiple_inputs_all_different() {
-        let diff = Diff::default_refinement(&[b"a b c", b"a X c", b"a c X"]);
+        let diff = Diff::default_refinement(["a b c", "a X c", "a c X"]);
         assert_eq!(
             diff.hunks().collect_vec(),
             vec![
-                DiffHunk::Matching(b"a "),
-                DiffHunk::Different(vec![b"b ", b"X ", b""]),
-                DiffHunk::Matching(b"c"),
-                DiffHunk::Different(vec![b"", b"", b" X"]),
+                DiffHunk::matching("a "),
+                DiffHunk::different(["b ", "X ", ""]),
+                DiffHunk::matching("c"),
+                DiffHunk::different(["", "", " X"]),
             ]
         );
     }
@@ -917,15 +1019,15 @@ mod tests {
     fn test_diff_for_tokenizer_compacted() {
         // Tests that unchanged regions are compacted when using for_tokenizer()
         let diff = Diff::for_tokenizer(
-            &[b"a\nb\nc\nd\ne\nf\ng", b"a\nb\nc\nX\ne\nf\ng"],
+            ["a\nb\nc\nd\ne\nf\ng", "a\nb\nc\nX\ne\nf\ng"],
             find_line_ranges,
         );
         assert_eq!(
             diff.hunks().collect_vec(),
             vec![
-                DiffHunk::Matching(b"a\nb\nc\n"),
-                DiffHunk::Different(vec![b"d\n", b"X\n"]),
-                DiffHunk::Matching(b"e\nf\ng"),
+                DiffHunk::matching("a\nb\nc\n"),
+                DiffHunk::different(["d\n", "X\n"]),
+                DiffHunk::matching("e\nf\ng"),
             ]
         );
     }
@@ -934,7 +1036,7 @@ mod tests {
     fn test_diff_nothing_in_common() {
         assert_eq!(
             diff(b"aaa", b"bb"),
-            vec![DiffHunk::Different(vec![b"aaa", b"bb"])]
+            vec![DiffHunk::different(["aaa", "bb"])]
         );
     }
 
@@ -943,9 +1045,9 @@ mod tests {
         assert_eq!(
             diff(b"a z", b"a S z"),
             vec![
-                DiffHunk::Matching(b"a "),
-                DiffHunk::Different(vec![b"", b"S "]),
-                DiffHunk::Matching(b"z"),
+                DiffHunk::matching("a "),
+                DiffHunk::different(["", "S "]),
+                DiffHunk::matching("z"),
             ]
         );
     }
@@ -955,11 +1057,11 @@ mod tests {
         assert_eq!(
             diff(b"a R R S S z", b"a S S R R z"),
             vec![
-                DiffHunk::Matching(b"a "),
-                DiffHunk::Different(vec![b"R R ", b""]),
-                DiffHunk::Matching(b"S S "),
-                DiffHunk::Different(vec![b"", b"R R "]),
-                DiffHunk::Matching(b"z")
+                DiffHunk::matching("a "),
+                DiffHunk::different(["R R ", ""]),
+                DiffHunk::matching("S S "),
+                DiffHunk::different(["", "R R "]),
+                DiffHunk::matching("z")
             ],
         );
     }
@@ -972,19 +1074,19 @@ mod tests {
                 b"a r r x q y z q b y q x r r c",
             ),
             vec![
-                DiffHunk::Matching(b"a "),
-                DiffHunk::Different(vec![b"q", b"r"]),
-                DiffHunk::Matching(b" "),
-                DiffHunk::Different(vec![b"", b"r "]),
-                DiffHunk::Matching(b"x q y "),
-                DiffHunk::Different(vec![b"q ", b""]),
-                DiffHunk::Matching(b"z q b "),
-                DiffHunk::Different(vec![b"q ", b""]),
-                DiffHunk::Matching(b"y q x "),
-                DiffHunk::Different(vec![b"q", b"r"]),
-                DiffHunk::Matching(b" "),
-                DiffHunk::Different(vec![b"", b"r "]),
-                DiffHunk::Matching(b"c"),
+                DiffHunk::matching("a "),
+                DiffHunk::different(["q", "r"]),
+                DiffHunk::matching(" "),
+                DiffHunk::different(["", "r "]),
+                DiffHunk::matching("x q y "),
+                DiffHunk::different(["q ", ""]),
+                DiffHunk::matching("z q b "),
+                DiffHunk::different(["q ", ""]),
+                DiffHunk::matching("y q x "),
+                DiffHunk::different(["q", "r"]),
+                DiffHunk::matching(" "),
+                DiffHunk::different(["", "r "]),
+                DiffHunk::matching("c"),
             ]
         );
     }
@@ -1001,11 +1103,11 @@ mod tests {
                 b"    pub fn write_fmt(&mut self, fmt: fmt::Arguments<\'_>) -> io::Result<()> {\n        self.styler().write_fmt(fmt)\n"
             ),
             vec![
-                DiffHunk::Matching(b"    pub fn write_fmt(&mut self, fmt: fmt::Arguments<\'_>) "),
-                DiffHunk::Different(vec![b"", b"-> io::Result<()> "]),
-                DiffHunk::Matching(b"{\n        self.styler().write_fmt(fmt)"),
-                DiffHunk::Different(vec![b".unwrap()", b""]),
-                DiffHunk::Matching(b"\n")
+                DiffHunk::matching("    pub fn write_fmt(&mut self, fmt: fmt::Arguments<\'_>) "),
+                DiffHunk::different(["", "-> io::Result<()> "]),
+                DiffHunk::matching("{\n        self.styler().write_fmt(fmt)"),
+                DiffHunk::different([".unwrap()", ""]),
+                DiffHunk::matching("\n")
             ]
         );
     }
@@ -1013,6 +1115,7 @@ mod tests {
     #[test]
     fn test_diff_real_case_gitgit_read_tree_c() {
         // This is the diff from commit e497ea2a9b in the git.git repo
+        #[rustfmt::skip]
         assert_eq!(
             diff(
                 br##"/*
@@ -1155,23 +1258,19 @@ int main(int argc, char **argv)
 "##,
             ),
             vec![
-               DiffHunk::Matching(b"/*\n * GIT - The information manager from hell\n *\n * Copyright (C) Linus Torvalds, 2005\n */\n#include \"#cache.h\"\n\n"),
-               DiffHunk::Different(vec![b"", b"static void create_directories(const char *path)\n{\n\tint len = strlen(path);\n\tchar *buf = malloc(len + 1);\n\tconst char *slash = path;\n\n\twhile ((slash = strchr(slash+1, \'/\')) != NULL) {\n\t\tlen = slash - path;\n\t\tmemcpy(buf, path, len);\n\t\tbuf[len] = 0;\n\t\tmkdir(buf, 0700);\n\t}\n}\n\nstatic int create_file(const char *path)\n{\n\tint fd = open(path, O_WRONLY | O_TRUNC | O_CREAT, 0600);\n\tif (fd < 0) {\n\t\tif (errno == ENOENT) {\n\t\t\tcreate_directories(path);\n\t\t\tfd = open(path, O_WRONLY | O_TRUNC | O_CREAT, 0600);\n\t\t}\n\t}\n\treturn fd;\n}\n\n"]),
-               DiffHunk::Matching(b"static int unpack(unsigned char *sha1)\n{\n\tvoid *buffer;\n\tunsigned long size;\n\tchar type[20];\n\n\tbuffer = read_sha1_file(sha1, type, &size);\n\tif (!buffer)\n\t\tusage(\"unable to read sha1 file\");\n\tif (strcmp(type, \"tree\"))\n\t\tusage(\"expected a \'tree\' node\");\n\twhile (size) {\n\t\tint len = strlen(buffer)+1;\n\t\tunsigned char *sha1 = buffer + len;\n\t\tchar *path = strchr(buffer, \' \')+1;\n"),
-               DiffHunk::Different(vec![b"", b"\t\tchar *data;\n\t\tunsigned long filesize;\n"]),
-               DiffHunk::Matching(b"\t\tunsigned int mode;\n"),
-               DiffHunk::Different(vec![b"", b"\t\tint fd;\n\n"]),
-               DiffHunk::Matching(b"\t\tif (size < len + 20 || sscanf(buffer, \"%o\", &mode) != 1)\n\t\t\tusage(\"corrupt \'tree\' file\");\n\t\tbuffer = sha1 + 20;\n\t\tsize -= len + 20;\n\t\t"),
-               DiffHunk::Different(vec![b"printf", b"data = read_sha1_file"]),
-               DiffHunk::Matching(b"("),
-               DiffHunk::Different(vec![b"\"%o %s (%s)\\n\", mode, path, sha1_to_hex(", b""]),
-               DiffHunk::Matching(b"sha1"),
-               DiffHunk::Different(vec![b"", b", type, &filesize"]),
-               DiffHunk::Matching(b")"),
-               DiffHunk::Different(vec![b")", b""]),
-               DiffHunk::Matching(b";\n"),
-               DiffHunk::Different(vec![b"", b"\t\tif (!data || strcmp(type, \"blob\"))\n\t\t\tusage(\"tree file refers to bad file data\");\n\t\tfd = create_file(path);\n\t\tif (fd < 0)\n\t\t\tusage(\"unable to create file\");\n\t\tif (write(fd, data, filesize) != filesize)\n\t\t\tusage(\"unable to write file\");\n\t\tfchmod(fd, mode);\n\t\tclose(fd);\n\t\tfree(data);\n"]),
-               DiffHunk::Matching(b"\t}\n\treturn 0;\n}\n\nint main(int argc, char **argv)\n{\n\tint fd;\n\tunsigned char sha1[20];\n\n\tif (argc != 2)\n\t\tusage(\"read-tree <key>\");\n\tif (get_sha1_hex(argv[1], sha1) < 0)\n\t\tusage(\"read-tree <key>\");\n\tsha1_file_directory = getenv(DB_ENVIRONMENT);\n\tif (!sha1_file_directory)\n\t\tsha1_file_directory = DEFAULT_DB_ENVIRONMENT;\n\tif (unpack(sha1) < 0)\n\t\tusage(\"unpack failed\");\n\treturn 0;\n}\n")
+               DiffHunk::matching("/*\n * GIT - The information manager from hell\n *\n * Copyright (C) Linus Torvalds, 2005\n */\n#include \"#cache.h\"\n\n"),
+               DiffHunk::different(["", "static void create_directories(const char *path)\n{\n\tint len = strlen(path);\n\tchar *buf = malloc(len + 1);\n\tconst char *slash = path;\n\n\twhile ((slash = strchr(slash+1, \'/\')) != NULL) {\n\t\tlen = slash - path;\n\t\tmemcpy(buf, path, len);\n\t\tbuf[len] = 0;\n\t\tmkdir(buf, 0700);\n\t}\n}\n\nstatic int create_file(const char *path)\n{\n\tint fd = open(path, O_WRONLY | O_TRUNC | O_CREAT, 0600);\n\tif (fd < 0) {\n\t\tif (errno == ENOENT) {\n\t\t\tcreate_directories(path);\n\t\t\tfd = open(path, O_WRONLY | O_TRUNC | O_CREAT, 0600);\n\t\t}\n\t}\n\treturn fd;\n}\n\n"]),
+               DiffHunk::matching("static int unpack(unsigned char *sha1)\n{\n\tvoid *buffer;\n\tunsigned long size;\n\tchar type[20];\n\n\tbuffer = read_sha1_file(sha1, type, &size);\n\tif (!buffer)\n\t\tusage(\"unable to read sha1 file\");\n\tif (strcmp(type, \"tree\"))\n\t\tusage(\"expected a \'tree\' node\");\n\twhile (size) {\n\t\tint len = strlen(buffer)+1;\n\t\tunsigned char *sha1 = buffer + len;\n\t\tchar *path = strchr(buffer, \' \')+1;\n"),
+               DiffHunk::different(["", "\t\tchar *data;\n\t\tunsigned long filesize;\n"]),
+               DiffHunk::matching("\t\tunsigned int mode;\n"),
+               DiffHunk::different(["", "\t\tint fd;\n\n"]),
+               DiffHunk::matching("\t\tif (size < len + 20 || sscanf(buffer, \"%o\", &mode) != 1)\n\t\t\tusage(\"corrupt \'tree\' file\");\n\t\tbuffer = sha1 + 20;\n\t\tsize -= len + 20;\n\t\t"),
+               DiffHunk::different(["printf(\"%o %s (%s)\\n\", mode, path,", "data ="]),
+               DiffHunk::matching(" "),
+               DiffHunk::different(["sha1_to_hex", "read_sha1_file"]),
+               DiffHunk::matching("(sha1"),
+               DiffHunk::different([")", ", type, &filesize);\n\t\tif (!data || strcmp(type, \"blob\"))\n\t\t\tusage(\"tree file refers to bad file data\");\n\t\tfd = create_file(path);\n\t\tif (fd < 0)\n\t\t\tusage(\"unable to create file\");\n\t\tif (write(fd, data, filesize) != filesize)\n\t\t\tusage(\"unable to write file\");\n\t\tfchmod(fd, mode);\n\t\tclose(fd);\n\t\tfree(data"]),
+               DiffHunk::matching(");\n\t}\n\treturn 0;\n}\n\nint main(int argc, char **argv)\n{\n\tint fd;\n\tunsigned char sha1[20];\n\n\tif (argc != 2)\n\t\tusage(\"read-tree <key>\");\n\tif (get_sha1_hex(argv[1], sha1) < 0)\n\t\tusage(\"read-tree <key>\");\n\tsha1_file_directory = getenv(DB_ENVIRONMENT);\n\tif (!sha1_file_directory)\n\t\tsha1_file_directory = DEFAULT_DB_ENVIRONMENT;\n\tif (unpack(sha1) < 0)\n\t\tusage(\"unpack failed\");\n\treturn 0;\n}\n"),
             ]
         );
     }
