@@ -164,9 +164,14 @@ fn rename_rules_in_pest_error(err: pest::error::Error<Rule>) -> pest::error::Err
 pub enum ExpressionKind<'i> {
     Identifier(&'i str),
     String(String),
-    StringPattern { kind: &'i str, value: String },
+    StringPattern {
+        kind: &'i str,
+        value: String,
+    },
     Unary(UnaryOp, Box<ExpressionNode<'i>>),
     Binary(BinaryOp, Box<ExpressionNode<'i>>, Box<ExpressionNode<'i>>),
+    /// `x | y | ..`
+    UnionAll(Vec<ExpressionNode<'i>>),
     FunctionCall(Box<FunctionCallNode<'i>>),
 }
 
@@ -178,8 +183,6 @@ pub enum UnaryOp {
 
 #[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 pub enum BinaryOp {
-    /// `|`
-    Union,
     /// `&`
     Intersection,
     /// `~`
@@ -189,6 +192,20 @@ pub enum BinaryOp {
 pub type ExpressionNode<'i> = dsl_util::ExpressionNode<'i, ExpressionKind<'i>>;
 pub type FunctionCallNode<'i> = dsl_util::FunctionCallNode<'i, ExpressionKind<'i>>;
 
+fn union_nodes<'i>(lhs: ExpressionNode<'i>, rhs: ExpressionNode<'i>) -> ExpressionNode<'i> {
+    let span = lhs.span.start_pos().span(&rhs.span.end_pos());
+    let expr = match lhs.kind {
+        // Flatten "x | y | z" to save recursion stack. Machine-generated query
+        // might have long chain of unions.
+        ExpressionKind::UnionAll(mut nodes) => {
+            nodes.push(rhs);
+            ExpressionKind::UnionAll(nodes)
+        }
+        _ => ExpressionKind::UnionAll(vec![lhs, rhs]),
+    };
+    ExpressionNode::new(expr, span)
+}
+
 fn parse_function_call_node(pair: Pair<Rule>) -> FilesetParseResult<FunctionCallNode> {
     assert_eq!(pair.as_rule(), Rule::function);
     let (name_pair, args_pair) = pair.into_inner().collect_tuple().unwrap();
@@ -273,7 +290,7 @@ fn parse_expression_node(pair: Pair<Rule>) -> FilesetParseResult<ExpressionNode>
         })
         .map_infix(|lhs, op, rhs| {
             let op_kind = match op.as_rule() {
-                Rule::union_op => BinaryOp::Union,
+                Rule::union_op => return Ok(union_nodes(lhs?, rhs?)),
                 Rule::intersection_op => BinaryOp::Intersection,
                 Rule::difference_op => BinaryOp::Difference,
                 r => panic!("unexpected infix operator rule {r:?}"),
@@ -288,7 +305,6 @@ fn parse_expression_node(pair: Pair<Rule>) -> FilesetParseResult<ExpressionNode>
 }
 
 /// Parses text into expression tree. No name resolution is made at this stage.
-#[cfg(test)] // TODO: alias will be parsed with no bare_string fallback
 pub fn parse_program(text: &str) -> FilesetParseResult<ExpressionNode> {
     let mut pairs = FilesetParser::parse(Rule::program, text)?;
     let first = pairs.next().unwrap();
@@ -389,6 +405,10 @@ mod tests {
                 let rhs = Box::new(normalize_tree(*rhs));
                 ExpressionKind::Binary(op, lhs, rhs)
             }
+            ExpressionKind::UnionAll(nodes) => {
+                let nodes = normalize_list(nodes);
+                ExpressionKind::UnionAll(nodes)
+            }
             ExpressionKind::FunctionCall(function) => {
                 let function = Box::new(normalize_function_call(*function));
                 ExpressionKind::FunctionCall(function)
@@ -438,6 +458,10 @@ mod tests {
             parse_into_kind(r#"Windows\Path"#),
             Ok(ExpressionKind::Identifier(r#"Windows\Path"#))
         );
+        assert_eq!(
+            parse_into_kind("glob*[chars]?"),
+            Ok(ExpressionKind::Identifier("glob*[chars]?"))
+        );
     }
 
     #[test]
@@ -482,6 +506,13 @@ mod tests {
                 value: "bar".to_owned()
             })
         );
+        assert_eq!(
+            parse_into_kind(" foo:glob*[chars]? "),
+            Ok(ExpressionKind::StringPattern {
+                kind: "foo",
+                value: "glob*[chars]?".to_owned()
+            })
+        );
         assert_eq!(
             parse_into_kind(r#" foo:"bar" "#),
             Ok(ExpressionKind::StringPattern {
@@ -525,7 +556,11 @@ mod tests {
         );
         assert_matches!(
             parse_into_kind("x|y"),
-            Ok(ExpressionKind::Binary(BinaryOp::Union, _, _))
+            Ok(ExpressionKind::UnionAll(nodes)) if nodes.len() == 2
+        );
+        assert_matches!(
+            parse_into_kind("x|y|z"),
+            Ok(ExpressionKind::UnionAll(nodes)) if nodes.len() == 3
         );
         assert_matches!(
             parse_into_kind("x&y"),
@@ -621,6 +656,13 @@ mod tests {
                 value: " bar baz".to_owned()
             })
         );
+        assert_eq!(
+            parse_maybe_bare_into_kind("foo:glob * [chars]?"),
+            Ok(ExpressionKind::StringPattern {
+                kind: "foo",
+                value: "glob * [chars]?".to_owned()
+            })
+        );
         assert_eq!(
             parse_maybe_bare_into_kind("foo:bar:baz"),
             Err(FilesetParseErrorKind::SyntaxError)
