@@ -116,8 +116,7 @@ impl Rule {
 #[derive(Debug, Error)]
 #[error("{pest_error}")]
 pub struct RevsetParseError {
-    // TODO: move parsing tests to this module and drop pub(super)
-    pub(super) kind: RevsetParseErrorKind,
+    kind: RevsetParseErrorKind,
     pest_error: Box<pest::error::Error<Rule>>,
     source: Option<Box<dyn error::Error + Send + Sync>>,
 }
@@ -312,6 +311,8 @@ pub enum ExpressionKind<'i> {
     RangeAll,
     Unary(UnaryOp, Box<ExpressionNode<'i>>),
     Binary(BinaryOp, Box<ExpressionNode<'i>>, Box<ExpressionNode<'i>>),
+    /// `x | y | ..`
+    UnionAll(Vec<ExpressionNode<'i>>),
     FunctionCall(Box<FunctionCallNode<'i>>),
     /// `name: body`
     Modifier(Box<ModifierNode<'i>>),
@@ -342,6 +343,10 @@ impl<'i> FoldableExpression<'i> for ExpressionKind<'i> {
                 let rhs = Box::new(folder.fold_expression(*rhs)?);
                 Ok(ExpressionKind::Binary(op, lhs, rhs))
             }
+            ExpressionKind::UnionAll(nodes) => {
+                let nodes = dsl_util::fold_expression_nodes(folder, nodes)?;
+                Ok(ExpressionKind::UnionAll(nodes))
+            }
             ExpressionKind::FunctionCall(function) => folder.fold_function_call(function, span),
             ExpressionKind::Modifier(modifier) => {
                 let modifier = Box::new(ModifierNode {
@@ -393,8 +398,6 @@ pub enum UnaryOp {
 
 #[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 pub enum BinaryOp {
-    /// `|`
-    Union,
     /// `&`
     Intersection,
     /// `~`
@@ -419,6 +422,20 @@ pub struct ModifierNode<'i> {
     pub body: ExpressionNode<'i>,
 }
 
+fn union_nodes<'i>(lhs: ExpressionNode<'i>, rhs: ExpressionNode<'i>) -> ExpressionNode<'i> {
+    let span = lhs.span.start_pos().span(&rhs.span.end_pos());
+    let expr = match lhs.kind {
+        // Flatten "x | y | z" to save recursion stack. Machine-generated query
+        // might have long chain of unions.
+        ExpressionKind::UnionAll(mut nodes) => {
+            nodes.push(rhs);
+            ExpressionKind::UnionAll(nodes)
+        }
+        _ => ExpressionKind::UnionAll(vec![lhs, rhs]),
+    };
+    ExpressionNode::new(expr, span)
+}
+
 pub(super) fn parse_program(revset_str: &str) -> Result<ExpressionNode, RevsetParseError> {
     let mut pairs = RevsetParser::parse(Rule::program, revset_str)?;
     let first = pairs.next().unwrap();
@@ -552,7 +569,7 @@ fn parse_expression_node(pairs: Pairs<Rule>) -> Result<ExpressionNode, RevsetPar
         })
         .map_infix(|lhs, op, rhs| {
             let op_kind = match op.as_rule() {
-                Rule::union_op => BinaryOp::Union,
+                Rule::union_op => return Ok(union_nodes(lhs?, rhs?)),
                 Rule::compat_add_op => Err(not_infix_op(&op, "|", "union"))?,
                 Rule::intersection_op => BinaryOp::Intersection,
                 Rule::difference_op => BinaryOp::Difference,
@@ -738,7 +755,7 @@ pub(super) fn expect_program_with<B, M>(
     parse_body: impl FnOnce(&ExpressionNode) -> Result<B, RevsetParseError>,
     parse_modifier: impl FnOnce(&str, pest::Span<'_>) -> Result<M, RevsetParseError>,
 ) -> Result<(B, Option<M>), RevsetParseError> {
-    expect_literal_with(node, |node| match &node.kind {
+    expect_expression_with(node, |node| match &node.kind {
         ExpressionKind::Modifier(modifier) => {
             let parsed_modifier = parse_modifier(modifier.name, modifier.name_span)?;
             Ok((parse_body(&modifier.body)?, Some(parsed_modifier)))
@@ -755,7 +772,7 @@ pub(super) fn expect_pattern_with<T, E: Into<Box<dyn error::Error + Send + Sync>
     let wrap_error = |err: E| {
         RevsetParseError::expression(format!("Invalid {type_name}"), node.span).with_source(err)
     };
-    expect_literal_with(node, |node| match &node.kind {
+    expect_expression_with(node, |node| match &node.kind {
         ExpressionKind::Identifier(name) => parse_pattern(name, None).map_err(wrap_error),
         ExpressionKind::String(name) => parse_pattern(name, None).map_err(wrap_error),
         ExpressionKind::StringPattern { kind, value } => {
@@ -778,7 +795,7 @@ pub fn expect_literal<T: FromStr>(
             node.span,
         )
     };
-    expect_literal_with(node, |node| match &node.kind {
+    expect_expression_with(node, |node| match &node.kind {
         ExpressionKind::Identifier(name) => name.parse().map_err(|_| make_error()),
         ExpressionKind::String(name) => name.parse().map_err(|_| make_error()),
         _ => Err(make_error()),
@@ -787,12 +804,12 @@ pub fn expect_literal<T: FromStr>(
 
 /// Applies the give function to the innermost `node` by unwrapping alias
 /// expansion nodes.
-fn expect_literal_with<T>(
+pub(super) fn expect_expression_with<T>(
     node: &ExpressionNode,
     f: impl FnOnce(&ExpressionNode) -> Result<T, RevsetParseError>,
 ) -> Result<T, RevsetParseError> {
     if let ExpressionKind::AliasExpanded(id, subst) = &node.kind {
-        expect_literal_with(subst, f).map_err(|e| e.within_alias_expansion(*id, node.span))
+        expect_expression_with(subst, f).map_err(|e| e.within_alias_expansion(*id, node.span))
     } else {
         f(node)
     }
@@ -884,6 +901,10 @@ mod tests {
                 let rhs = Box::new(normalize_tree(*rhs));
                 ExpressionKind::Binary(op, lhs, rhs)
             }
+            ExpressionKind::UnionAll(nodes) => {
+                let nodes = normalize_list(nodes);
+                ExpressionKind::UnionAll(nodes)
+            }
             ExpressionKind::FunctionCall(function) => {
                 let function = Box::new(normalize_function_call(*function));
                 ExpressionKind::FunctionCall(function)
@@ -1068,7 +1089,11 @@ mod tests {
         // Parse the "union" operator
         assert_matches!(
             parse_into_kind("foo | bar"),
-            Ok(ExpressionKind::Binary(BinaryOp::Union, _, _))
+            Ok(ExpressionKind::UnionAll(nodes)) if nodes.len() == 2
+        );
+        assert_matches!(
+            parse_into_kind("foo | bar | baz"),
+            Ok(ExpressionKind::UnionAll(nodes)) if nodes.len() == 3
         );
         // Parse the "difference" operator
         assert_matches!(
@@ -1480,8 +1505,8 @@ mod tests {
     #[test]
     fn test_expand_symbol_alias() {
         assert_eq!(
-            with_aliases([("AB", "a|b")]).parse_normalized("AB|c"),
-            parse_normalized("(a|b)|c")
+            with_aliases([("AB", "a&b")]).parse_normalized("AB|c"),
+            parse_normalized("(a&b)|c")
         );
         assert_eq!(
             with_aliases([("AB", "a|b")]).parse_normalized("AB::heads(AB)"),
