@@ -28,7 +28,6 @@ use crate::commit_builder::CommitBuilder;
 use crate::index::Index;
 use crate::matchers::{Matcher, Visit};
 use crate::merged_tree::{MergedTree, MergedTreeBuilder};
-use crate::object_id::ObjectId;
 use crate::repo::{MutableRepo, Repo};
 use crate::repo_path::RepoPath;
 use crate::settings::UserSettings;
@@ -304,41 +303,25 @@ pub fn rebase_commit_with_options(
     }
 }
 
+/// Moves changes from `sources` to the `destination` parent, returns new tree.
 pub fn rebase_to_dest_parent(
     repo: &dyn Repo,
-    source: &Commit,
+    sources: &[Commit],
     destination: &Commit,
 ) -> BackendResult<MergedTree> {
-    if source.parent_ids() == destination.parent_ids() {
-        Ok(source.tree()?)
-    } else {
-        let destination_parent_tree = destination.parent_tree(repo)?;
-        let source_parent_tree = source.parent_tree(repo)?;
-        let source_tree = source.tree()?;
-        let rebased_tree = destination_parent_tree.merge(&source_parent_tree, &source_tree)?;
-        Ok(rebased_tree)
+    if let [source] = sources {
+        if source.parent_ids() == destination.parent_ids() {
+            return source.tree();
+        }
     }
-}
-
-pub fn back_out_commit(
-    settings: &UserSettings,
-    mut_repo: &mut MutableRepo,
-    old_commit: &Commit,
-    new_parents: &[Commit],
-) -> BackendResult<Commit> {
-    let old_base_tree = old_commit.parent_tree(mut_repo)?;
-    let new_base_tree = merge_commit_trees(mut_repo, new_parents)?;
-    let old_tree = old_commit.tree()?;
-    let new_tree = new_base_tree.merge(&old_tree, &old_base_tree)?;
-    let new_parent_ids = new_parents
-        .iter()
-        .map(|commit| commit.id().clone())
-        .collect();
-    // TODO: i18n the description based on repo language
-    mut_repo
-        .new_commit(settings, new_parent_ids, new_tree.id())
-        .set_description(format!("backout of commit {}", &old_commit.id().hex()))
-        .write()
+    sources.iter().try_fold(
+        destination.parent_tree(repo)?,
+        |destination_tree, source| {
+            let source_parent_tree = source.parent_tree(repo)?;
+            let source_tree = source.tree()?;
+            destination_tree.merge(&source_parent_tree, &source_tree)
+        },
+    )
 }
 
 #[derive(Clone, Copy, Default, PartialEq, Eq, Debug)]
