@@ -172,6 +172,10 @@ impl UserSettings {
     // address
     pub const USER_EMAIL_PLACEHOLDER: &'static str = "(no email configured)";
 
+    pub fn commit_timestamp(&self) -> Option<Timestamp> {
+        self.timestamp.to_owned()
+    }
+
     pub fn operation_timestamp(&self) -> Option<Timestamp> {
         get_timestamp_config(&self.config, "debug.operation-timestamp")
     }
@@ -236,16 +240,16 @@ impl UserSettings {
     pub fn commit_node_template(&self) -> String {
         self.node_template_for_key(
             "templates.log_node",
-            r#"if(self, if(current_working_copy, "@", "◉"), "◌")"#,
-            r#"if(self, if(current_working_copy, "@", "o"), ".")"#,
+            "builtin_log_node",
+            "builtin_log_node_ascii",
         )
     }
 
     pub fn op_node_template(&self) -> String {
         self.node_template_for_key(
             "templates.op_log_node",
-            r#"if(current_operation, "@", "◉")"#,
-            r#"if(current_operation, "@", "o")"#,
+            "builtin_op_log_node",
+            "builtin_op_log_node_ascii",
         )
     }
 
