@@ -36,7 +36,7 @@ use crate::op_store::{
     OpStore, OpStoreError, OpStoreResult, Operation, OperationId, OperationMetadata, RefTarget,
     RemoteRef, RemoteRefState, RemoteView, View, ViewId, WorkspaceId,
 };
-use crate::{dag_walk, git, op_store};
+use crate::{dag_walk, op_store};
 
 // BLAKE2b-512 hash length in bytes
 const OPERATION_ID_LENGTH: usize = 64;
@@ -356,6 +356,7 @@ fn operation_metadata_to_proto(
         description: metadata.description.clone(),
         hostname: metadata.hostname.clone(),
         username: metadata.username.clone(),
+        is_snapshot: metadata.is_snapshot,
         tags: metadata.tags.clone(),
     }
 }
@@ -371,6 +372,7 @@ fn operation_metadata_from_proto(
         description: proto.description,
         hostname: proto.hostname,
         username: proto.username,
+        is_snapshot: proto.is_snapshot,
         tags: proto.tags,
     }
 }
@@ -526,8 +528,11 @@ fn branch_views_from_proto_legacy(
                 // than deleted but yet-to-be-pushed local branch. Alternatively, we could read
                 // git.auto-local-branch setting here, but that wouldn't always work since the
                 // setting could be toggled after the branch got merged.
+                #[cfg(feature = "git")]
                 let is_git_tracking =
-                    remote_branch.remote_name == git::REMOTE_NAME_FOR_LOCAL_GIT_REPO;
+                    remote_branch.remote_name == crate::git::REMOTE_NAME_FOR_LOCAL_GIT_REPO;
+                #[cfg(not(feature = "git"))]
+                let is_git_tracking = false;
                 let default_state = if is_git_tracking || local_target.is_present() {
                     RemoteRefState::Tracking
                 } else {
@@ -576,13 +581,21 @@ fn migrate_git_refs_to_remote(view: &mut View) {
             git_view.branches.insert(name.to_owned(), remote_ref);
         }
     }
-    view.remote_views
-        .insert(git::REMOTE_NAME_FOR_LOCAL_GIT_REPO.to_owned(), git_view);
+    #[cfg(feature = "git")]
+    {
+        view.remote_views.insert(
+            crate::git::REMOTE_NAME_FOR_LOCAL_GIT_REPO.to_owned(),
+            git_view,
+        );
 
-    // jj < 0.9 might have imported refs from remote named "git"
-    let reserved_git_ref_prefix = format!("refs/remotes/{}/", git::REMOTE_NAME_FOR_LOCAL_GIT_REPO);
-    view.git_refs
-        .retain(|name, _| !name.starts_with(&reserved_git_ref_prefix));
+        // jj < 0.9 might have imported refs from remote named "git"
+        let reserved_git_ref_prefix = format!(
+            "refs/remotes/{}/",
+            crate::git::REMOTE_NAME_FOR_LOCAL_GIT_REPO
+        );
+        view.git_refs
+            .retain(|name, _| !name.starts_with(&reserved_git_ref_prefix));
+    }
 }
 
 fn ref_target_to_proto(value: &RefTarget) -> Option<crate::protos::op_store::RefTarget> {
@@ -668,7 +681,7 @@ fn remote_ref_state_to_proto(state: RemoteRefState) -> Option<i32> {
 }
 
 fn remote_ref_state_from_proto(proto_value: Option<i32>) -> Option<RemoteRefState> {
-    let proto_state = proto_value.and_then(crate::protos::op_store::RemoteRefState::from_i32)?;
+    let proto_state = proto_value?.try_into().ok()?;
     let state = match proto_state {
         crate::protos::op_store::RemoteRefState::New => RemoteRefState::New,
         crate::protos::op_store::RemoteRefState::Tracking => RemoteRefState::Tracking,
@@ -683,10 +696,6 @@ mod tests {
     use maplit::{btreemap, hashmap, hashset};
 
     use super::*;
-    use crate::backend::{CommitId, MillisSinceEpoch, Timestamp};
-    use crate::content_hash::blake2b_hash;
-    use crate::object_id::ObjectId;
-    use crate::op_store::{OperationMetadata, RefTarget, WorkspaceId};
 
     fn create_view() -> View {
         let new_remote_ref = |target: &RefTarget| RemoteRef {
@@ -757,6 +766,7 @@ mod tests {
                 description: "check out foo".to_string(),
                 hostname: "some.host.example.com".to_string(),
                 username: "someone".to_string(),
+                is_snapshot: false,
                 tags: hashmap! {
                     "key1".to_string() => "value1".to_string(),
                     "key2".to_string() => "value2".to_string(),
@@ -770,7 +780,7 @@ mod tests {
         // Test exact output so we detect regressions in compatibility
         assert_snapshot!(
             ViewId::new(blake2b_hash(&create_view()).to_vec()).hex(),
-            @"19af495b5a828c38e933c518a53eb9f7fb7129348a0950d7971b86adf7eb25a13bf45c7f358da1776b27fe6f4bc9134787389d976b768e46ac6cef56bc20c083"
+            @"f426676b3a2f7c6b9ec8677cb05ed249d0d244ab7e86a7c51117e2d8a4829db65e55970c761231e2107d303bf3d33a1f2afdd4ed2181f223e99753674b20a35e"
         );
     }
 
@@ -779,7 +789,7 @@ mod tests {
         // Test exact output so we detect regressions in compatibility
         assert_snapshot!(
             OperationId::new(blake2b_hash(&create_operation()).to_vec()).hex(),
-            @"3ec986c29ff8eb808ea8f6325d6307cea75ef02987536c8e4645406aba51afc8e229957a6e855170d77a66098c58912309323f5e0b32760caa2b59dc84d45fcf"
+            @"20b495d54aa3be3a672a2ed6dbbf7a711dabce4cc0161d657e5177070491c1e780eec3fd35c2aa9dcc22371462aeb412a502a847f29419e65718f56a0ad1b2d0"
         );
     }
 
