@@ -20,6 +20,7 @@ use itertools::Itertools as _;
 
 use crate::backend::Timestamp;
 use crate::index::ReadonlyIndex;
+use crate::op_heads_store::OpHeadsStore;
 use crate::op_store::OperationMetadata;
 use crate::operation::Operation;
 use crate::repo::{MutableRepo, ReadonlyRepo, Repo, RepoLoader, RepoLoaderError};
@@ -27,6 +28,17 @@ use crate::settings::UserSettings;
 use crate::view::View;
 use crate::{dag_walk, op_store};
 
+/// An in-memory representation of a repo and any changes being made to it.
+///
+/// Within the scope of a transaction, changes to the repository are made
+/// in-memory to `mut_repo` and published to the repo backend when
+/// [`Transaction::commit`] is called. When a transaction is committed, it
+/// becomes atomically visible as an Operation in the op log that represents the
+/// transaction itself, and as a View that represents the state of the repo
+/// after the transaction. This is similar to how a Commit represents a change
+/// to the contents of the repository and a Tree represents the repository's
+/// contents after the change. See the documentation for [`op_store::Operation`]
+/// and [`op_store::View`] for more information.
 pub struct Transaction {
     mut_repo: MutableRepo,
     parent_ops: Vec<Operation>,
@@ -37,7 +49,7 @@ pub struct Transaction {
 impl Transaction {
     pub fn new(mut_repo: MutableRepo, user_settings: &UserSettings) -> Transaction {
         let parent_ops = vec![mut_repo.base_repo().operation().clone()];
-        let op_metadata = create_op_metadata(user_settings, "".to_string());
+        let op_metadata = create_op_metadata(user_settings, "".to_string(), false);
         let end_time = user_settings.operation_timestamp();
         Transaction {
             mut_repo,
@@ -80,6 +92,10 @@ impl Transaction {
         Ok(())
     }
 
+    pub fn set_is_snapshot(&mut self, is_snapshot: bool) {
+        self.op_metadata.is_snapshot = is_snapshot;
+    }
+
     /// Writes the transaction to the operation store and publishes it.
     pub fn commit(self, description: impl Into<String>) -> Arc<ReadonlyRepo> {
         self.write(description).publish()
@@ -115,13 +131,17 @@ impl Transaction {
 
         let index = base_repo
             .index_store()
-            .write_index(mut_index, operation.id())
+            .write_index(mut_index, &operation)
             .unwrap();
-        UnpublishedOperation::new(base_repo.loader(), operation, view, index)
+        UnpublishedOperation::new(&base_repo.loader(), operation, view, index)
     }
 }
 
-pub fn create_op_metadata(user_settings: &UserSettings, description: String) -> OperationMetadata {
+pub fn create_op_metadata(
+    user_settings: &UserSettings,
+    description: String,
+    is_snapshot: bool,
+) -> OperationMetadata {
     let start_time = user_settings
         .operation_timestamp()
         .unwrap_or_else(Timestamp::now);
@@ -134,74 +154,48 @@ pub fn create_op_metadata(user_settings: &UserSettings, description: String) ->
         description,
         hostname,
         username,
+        is_snapshot,
         tags: Default::default(),
     }
 }
 
-struct NewRepoData {
-    operation: Operation,
-    view: View,
-    index: Box<dyn ReadonlyIndex>,
-}
-
+/// An Operation which has been written to the operation store but not
+/// published. The repo can be loaded at an unpublished Operation, but the
+/// Operation will not be visible in the op log if the repo is loaded at head.
+///
+/// Either [`Self::publish`] or [`Self::leave_unpublished`] must be called to
+/// finish the operation.
+#[must_use = "Either publish() or leave_unpublished() must be called to finish the operation."]
 pub struct UnpublishedOperation {
-    repo_loader: RepoLoader,
-    data: Option<NewRepoData>,
-    closed: bool,
+    op_heads_store: Arc<dyn OpHeadsStore>,
+    repo: Arc<ReadonlyRepo>,
 }
 
 impl UnpublishedOperation {
     fn new(
-        repo_loader: RepoLoader,
+        repo_loader: &RepoLoader,
         operation: Operation,
         view: View,
         index: Box<dyn ReadonlyIndex>,
     ) -> Self {
-        let data = Some(NewRepoData {
-            operation,
-            view,
-            index,
-        });
         UnpublishedOperation {
-            repo_loader,
-            data,
-            closed: false,
+            op_heads_store: repo_loader.op_heads_store().clone(),
+            repo: repo_loader.create_from(operation, view, index),
         }
     }
 
     pub fn operation(&self) -> &Operation {
-        &self.data.as_ref().unwrap().operation
+        self.repo.operation()
     }
 
-    pub fn publish(mut self) -> Arc<ReadonlyRepo> {
-        let data = self.data.take().unwrap();
-        {
-            let _lock = self.repo_loader.op_heads_store().lock();
-            self.repo_loader
-                .op_heads_store()
-                .update_op_heads(data.operation.parent_ids(), data.operation.id());
-        }
-        let repo = self
-            .repo_loader
-            .create_from(data.operation, data.view, data.index);
-        self.closed = true;
-        repo
+    pub fn publish(self) -> Arc<ReadonlyRepo> {
+        let _lock = self.op_heads_store.lock();
+        self.op_heads_store
+            .update_op_heads(self.operation().parent_ids(), self.operation().id());
+        self.repo
     }
 
-    pub fn leave_unpublished(mut self) -> Arc<ReadonlyRepo> {
-        let data = self.data.take().unwrap();
-        let repo = self
-            .repo_loader
-            .create_from(data.operation, data.view, data.index);
-        self.closed = true;
-        repo
-    }
-}
-
-impl Drop for UnpublishedOperation {
-    fn drop(&mut self) {
-        if !self.closed && !std::thread::panicking() {
-            eprintln!("BUG: UnpublishedOperation was dropped without being closed.");
-        }
+    pub fn leave_unpublished(self) -> Arc<ReadonlyRepo> {
+        self.repo
     }
 }
