@@ -20,10 +20,11 @@ use itertools::Itertools;
 
 use crate::backend::CommitId;
 use crate::op_store::{BranchTarget, RefTarget, RefTargetOptionExt as _, RemoteRef, WorkspaceId};
-use crate::refs::TrackingRefPair;
+use crate::refs::LocalAndRemoteRef;
 use crate::str_util::StringPattern;
 use crate::{op_store, refs};
 
+/// A wrapper around [`op_store::View`] that defines additional methods.
 #[derive(PartialEq, Eq, Debug, Clone)]
 pub struct View {
     data: op_store::View,
@@ -95,25 +96,6 @@ impl View {
         self.data.head_ids.remove(head_id);
     }
 
-    /// Returns true if any local or remote branch of the given `name` exists.
-    #[must_use]
-    pub fn has_branch(&self, name: &str) -> bool {
-        self.data.local_branches.contains_key(name)
-            || self
-                .data
-                .remote_views
-                .values()
-                .any(|remote_view| remote_view.branches.contains_key(name))
-    }
-
-    // TODO: maybe rename to forget_branch() because this seems unusual operation?
-    pub fn remove_branch(&mut self, name: &str) {
-        self.data.local_branches.remove(name);
-        for remote_view in self.data.remote_views.values_mut() {
-            remote_view.branches.remove(name);
-        }
-    }
-
     /// Iterates local branch `(name, target)`s in lexicographical order.
     pub fn local_branches(&self) -> impl Iterator<Item = (&str, &RefTarget)> {
         self.data
@@ -122,6 +104,16 @@ impl View {
             .map(|(name, target)| (name.as_ref(), target))
     }
 
+    /// Iterates local branches `(name, target)` in lexicographical order where
+    /// the target adds `commit_id`.
+    pub fn local_branches_for_commit<'a: 'b, 'b>(
+        &'a self,
+        commit_id: &'b CommitId,
+    ) -> impl Iterator<Item = (&'a str, &'a RefTarget)> + 'b {
+        self.local_branches()
+            .filter(|(_, target)| target.added_ids().contains(commit_id))
+    }
+
     /// Iterates local branch `(name, target)`s matching the given pattern.
     /// Entries are sorted by `name`.
     pub fn local_branches_matching<'a: 'b, 'b>(
@@ -224,10 +216,10 @@ impl View {
     pub fn local_remote_branches<'a>(
         &'a self,
         remote_name: &str,
-    ) -> impl Iterator<Item = (&'a str, TrackingRefPair<'a>)> + 'a {
+    ) -> impl Iterator<Item = (&'a str, LocalAndRemoteRef<'a>)> + 'a {
         refs::iter_named_local_remote_refs(self.local_branches(), self.remote_branches(remote_name))
             .map(|(name, (local_target, remote_ref))| {
-                let targets = TrackingRefPair {
+                let targets = LocalAndRemoteRef {
                     local_target,
                     remote_ref,
                 };
@@ -248,7 +240,7 @@ impl View {
         &'a self,
         branch_pattern: &'b StringPattern,
         remote_name: &str,
-    ) -> impl Iterator<Item = (&'a str, TrackingRefPair<'a>)> + 'b {
+    ) -> impl Iterator<Item = (&'a str, LocalAndRemoteRef<'a>)> + 'b {
         // Change remote_name to StringPattern if needed, but merge-join adapter won't
         // be usable.
         let maybe_remote_view = self.data.remote_views.get(remote_name);
@@ -260,7 +252,7 @@ impl View {
                 .flatten(),
         )
         .map(|(name, (local_target, remote_ref))| {
-            let targets = TrackingRefPair {
+            let targets = LocalAndRemoteRef {
                 local_target,
                 remote_ref,
             };
