@@ -12,16 +12,23 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
+use std::path::Path;
 use std::process::Command;
 use std::sync::Arc;
-use std::time::SystemTime;
+use std::time::{Duration, SystemTime};
 
-use jj_lib::backend::CommitId;
+use futures::executor::block_on_stream;
+use jj_lib::backend::{CommitId, CopySource, CopySources};
+use jj_lib::commit::Commit;
 use jj_lib::git_backend::GitBackend;
 use jj_lib::repo::{ReadonlyRepo, Repo};
+use jj_lib::repo_path::{RepoPath, RepoPathBuf};
+use jj_lib::settings::UserSettings;
+use jj_lib::store::Store;
+use jj_lib::transaction::Transaction;
 use maplit::hashset;
-use testutils::{create_random_commit, CommitGraphBuilder, TestRepo, TestRepoBackend};
+use testutils::{create_random_commit, create_tree, CommitGraphBuilder, TestRepo, TestRepoBackend};
 
 fn get_git_backend(repo: &Arc<ReadonlyRepo>) -> &GitBackend {
     repo.store()
@@ -30,11 +37,11 @@ fn get_git_backend(repo: &Arc<ReadonlyRepo>) -> &GitBackend {
         .unwrap()
 }
 
-fn get_git_repo(repo: &Arc<ReadonlyRepo>) -> gix::Repository {
-    get_git_backend(repo).git_repo()
-}
-
-fn collect_no_gc_refs(git_repo: &gix::Repository) -> HashSet<CommitId> {
+fn collect_no_gc_refs(git_repo_path: &Path) -> HashSet<CommitId> {
+    // Load fresh git repo to isolate from false caching issue. Here we want to
+    // ensure that the underlying data is correct. We could test the in-memory
+    // data as well, but we don't have any special handling in our code.
+    let git_repo = gix::open(git_repo_path).unwrap();
     let git_refs = git_repo.references().unwrap();
     let no_gc_refs_iter = git_refs.prefixed("refs/jj/keep/").unwrap();
     no_gc_refs_iter
@@ -42,6 +49,46 @@ fn collect_no_gc_refs(git_repo: &gix::Repository) -> HashSet<CommitId> {
         .collect()
 }
 
+fn get_copy_records(
+    store: &Store,
+    paths: &[RepoPathBuf],
+    a: &Commit,
+    b: &Commit,
+) -> HashMap<String, Vec<String>> {
+    let stream = store
+        .get_copy_records(paths, &[a.id().clone()], &[b.id().clone()])
+        .unwrap();
+    let mut res: HashMap<String, Vec<String>> = HashMap::new();
+    for copy_record in block_on_stream(stream).map(|r| r.unwrap()) {
+        res.insert(
+            copy_record.target.as_internal_file_string().into(),
+            match copy_record.sources {
+                CopySources::Resolved(CopySource { path, .. }) => {
+                    vec![path.as_internal_file_string().into()]
+                }
+                CopySources::Conflict(conflicting) => conflicting
+                    .iter()
+                    .map(|s| s.path.as_internal_file_string().into())
+                    .collect(),
+            },
+        );
+    }
+    res
+}
+
+fn make_commit(
+    tx: &mut Transaction,
+    settings: &UserSettings,
+    parents: Vec<CommitId>,
+    content: &[(&RepoPath, &str)],
+) -> Commit {
+    let tree = create_tree(tx.base_repo(), content);
+    tx.mut_repo()
+        .new_commit(settings, parents, tree.id())
+        .write()
+        .unwrap()
+}
+
 #[test]
 fn test_gc() {
     // TODO: Better way to disable the test if git command couldn't be executed
@@ -53,7 +100,7 @@ fn test_gc() {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = test_repo.repo;
-    let git_repo = get_git_repo(&repo);
+    let git_repo_path = get_git_backend(&repo).git_repo_path();
     let base_index = repo.readonly_index();
 
     // Set up commits:
@@ -94,7 +141,7 @@ fn test_gc() {
 
     // At first, all commits have no-gc refs
     assert_eq!(
-        collect_no_gc_refs(&git_repo),
+        collect_no_gc_refs(git_repo_path),
         hashset! {
             commit_a.id().clone(),
             commit_b.id().clone(),
@@ -113,7 +160,7 @@ fn test_gc() {
         .gc(base_index.as_index(), SystemTime::UNIX_EPOCH)
         .unwrap();
     assert_eq!(
-        collect_no_gc_refs(&git_repo),
+        collect_no_gc_refs(git_repo_path),
         hashset! {
             commit_a.id().clone(),
             commit_b.id().clone(),
@@ -126,11 +173,14 @@ fn test_gc() {
         },
     );
 
+    // Don't rely on the exact system time because file modification time might
+    // have lower precision for example.
+    let now = || SystemTime::now() + Duration::from_secs(1);
+
     // All reachable: redundant no-gc refs will be removed
-    let now = SystemTime::now();
-    repo.store().gc(repo.index(), now).unwrap();
+    repo.store().gc(repo.index(), now()).unwrap();
     assert_eq!(
-        collect_no_gc_refs(&git_repo),
+        collect_no_gc_refs(git_repo_path),
         hashset! {
             commit_d.id().clone(),
             commit_g.id().clone(),
@@ -147,9 +197,9 @@ fn test_gc() {
     mut_index.add_commit(&commit_e);
     mut_index.add_commit(&commit_f);
     mut_index.add_commit(&commit_h);
-    repo.store().gc(mut_index.as_index(), now).unwrap();
+    repo.store().gc(mut_index.as_index(), now()).unwrap();
     assert_eq!(
-        collect_no_gc_refs(&git_repo),
+        collect_no_gc_refs(git_repo_path),
         hashset! {
             commit_d.id().clone(),
             commit_e.id().clone(),
@@ -163,9 +213,9 @@ fn test_gc() {
     mut_index.add_commit(&commit_b);
     mut_index.add_commit(&commit_c);
     mut_index.add_commit(&commit_f);
-    repo.store().gc(mut_index.as_index(), now).unwrap();
+    repo.store().gc(mut_index.as_index(), now()).unwrap();
     assert_eq!(
-        collect_no_gc_refs(&git_repo),
+        collect_no_gc_refs(git_repo_path),
         hashset! {
             commit_c.id().clone(),
             commit_f.id().clone(),
@@ -175,15 +225,70 @@ fn test_gc() {
     // B|C|F are no longer reachable
     let mut mut_index = base_index.start_modification();
     mut_index.add_commit(&commit_a);
-    repo.store().gc(mut_index.as_index(), now).unwrap();
+    repo.store().gc(mut_index.as_index(), now()).unwrap();
     assert_eq!(
-        collect_no_gc_refs(&git_repo),
+        collect_no_gc_refs(git_repo_path),
         hashset! {
             commit_a.id().clone(),
         },
     );
 
     // All unreachable
-    repo.store().gc(base_index.as_index(), now).unwrap();
-    assert_eq!(collect_no_gc_refs(&git_repo), hashset! {});
+    repo.store().gc(base_index.as_index(), now()).unwrap();
+    assert_eq!(collect_no_gc_refs(git_repo_path), hashset! {});
+}
+
+#[test]
+fn test_copy_detection() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
+    let repo = &test_repo.repo;
+
+    let paths = &[
+        RepoPathBuf::from_internal_string("file0"),
+        RepoPathBuf::from_internal_string("file1"),
+        RepoPathBuf::from_internal_string("file2"),
+    ];
+
+    let mut tx = repo.start_transaction(&settings);
+    let commit_a = make_commit(
+        &mut tx,
+        &settings,
+        vec![repo.store().root_commit_id().clone()],
+        &[(&paths[0], "content")],
+    );
+    let commit_b = make_commit(
+        &mut tx,
+        &settings,
+        vec![commit_a.id().clone()],
+        &[(&paths[1], "content")],
+    );
+    let commit_c = make_commit(
+        &mut tx,
+        &settings,
+        vec![commit_b.id().clone()],
+        &[(&paths[2], "content")],
+    );
+
+    let store = repo.store();
+    assert_eq!(
+        get_copy_records(store, paths, &commit_a, &commit_b),
+        HashMap::from([("file1".to_string(), vec!["file0".to_string()])])
+    );
+    assert_eq!(
+        get_copy_records(store, paths, &commit_b, &commit_c),
+        HashMap::from([("file2".to_string(), vec!["file1".to_string()])])
+    );
+    assert_eq!(
+        get_copy_records(store, paths, &commit_a, &commit_c),
+        HashMap::from([("file2".to_string(), vec!["file0".to_string()])])
+    );
+    assert_eq!(
+        get_copy_records(store, &[], &commit_a, &commit_c),
+        HashMap::default(),
+    );
+    assert_eq!(
+        get_copy_records(store, paths, &commit_c, &commit_c),
+        HashMap::default(),
+    );
 }
