@@ -60,7 +60,7 @@ fn test_same_type() {
                 );
             }
         }
-        let tree_id = tree_builder.write_tree();
+        let tree_id = tree_builder.write_tree().unwrap();
         store.get_tree(RepoPath::root(), &tree_id).unwrap()
     };
 
@@ -202,7 +202,7 @@ fn test_executable() {
                 testutils::write_normal_file(&mut tree_builder, repo_path, "contents");
             }
         }
-        let tree_id = tree_builder.write_tree();
+        let tree_id = tree_builder.write_tree().unwrap();
         store.get_tree(RepoPath::root(), &tree_id).unwrap()
     };
 
@@ -250,7 +250,7 @@ fn test_subtrees() {
                 &format!("contents of {path:?}"),
             );
         }
-        let tree_id = tree_builder.write_tree();
+        let tree_id = tree_builder.write_tree().unwrap();
         store.get_tree(RepoPath::root(), &tree_id).unwrap()
     };
 
@@ -304,7 +304,7 @@ fn test_subtree_becomes_empty() {
                 &format!("contents of {path:?}"),
             );
         }
-        let tree_id = tree_builder.write_tree();
+        let tree_id = tree_builder.write_tree().unwrap();
         store.get_tree(RepoPath::root(), &tree_id).unwrap()
     };
 
@@ -333,7 +333,7 @@ fn test_subtree_one_missing() {
                 &format!("contents of {path:?}"),
             );
         }
-        let tree_id = tree_builder.write_tree();
+        let tree_id = tree_builder.write_tree().unwrap();
         store.get_tree(RepoPath::root(), &tree_id).unwrap()
     };
 
@@ -400,11 +400,11 @@ fn test_types() {
         RepoPath::from_internal_string("tree_normal_symlink"),
         "contents",
     );
-    let base_tree_id = base_tree_builder.write_tree();
+    let base_tree_id = base_tree_builder.write_tree().unwrap();
     let base_tree = store.get_tree(RepoPath::root(), &base_tree_id).unwrap();
-    let side1_tree_id = side1_tree_builder.write_tree();
+    let side1_tree_id = side1_tree_builder.write_tree().unwrap();
     let side1_tree = store.get_tree(RepoPath::root(), &side1_tree_id).unwrap();
-    let side2_tree_id = side2_tree_builder.write_tree();
+    let side2_tree_id = side2_tree_builder.write_tree().unwrap();
     let side2_tree = store.get_tree(RepoPath::root(), &side2_tree_id).unwrap();
 
     // Created the merged tree
@@ -581,15 +581,26 @@ fn test_simplify_conflict_after_resolving_parent() {
         .write()
         .unwrap();
 
-    let commit_b2 = rebase_commit(&settings, tx.mut_repo(), &commit_b, &[commit_d]).unwrap();
-    let commit_c2 =
-        rebase_commit(&settings, tx.mut_repo(), &commit_c, &[commit_b2.clone()]).unwrap();
+    let commit_b2 = rebase_commit(
+        &settings,
+        tx.mut_repo(),
+        commit_b,
+        vec![commit_d.id().clone()],
+    )
+    .unwrap();
+    let commit_c2 = rebase_commit(
+        &settings,
+        tx.mut_repo(),
+        commit_c,
+        vec![commit_b2.id().clone()],
+    )
+    .unwrap();
 
     // Test the setup: Both B and C should have conflicts.
     let tree_b2 = commit_b2.tree().unwrap();
     let tree_c2 = commit_b2.tree().unwrap();
-    assert!(!tree_b2.path_value(path).is_resolved());
-    assert!(!tree_c2.path_value(path).is_resolved());
+    assert!(!tree_b2.path_value(path).unwrap().is_resolved());
+    assert!(!tree_c2.path_value(path).unwrap().is_resolved());
 
     // Create the resolved B and rebase C on top.
     let tree_b3 = create_tree(repo, &[(path, "AbC\ndef\nghi\n")]);
@@ -599,13 +610,19 @@ fn test_simplify_conflict_after_resolving_parent() {
         .set_tree_id(tree_b3.id())
         .write()
         .unwrap();
-    let commit_c3 = rebase_commit(&settings, tx.mut_repo(), &commit_c2, &[commit_b3]).unwrap();
+    let commit_c3 = rebase_commit(
+        &settings,
+        tx.mut_repo(),
+        commit_c2,
+        vec![commit_b3.id().clone()],
+    )
+    .unwrap();
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     let repo = tx.commit("test");
 
     // The conflict should now be resolved.
     let tree_c2 = commit_c3.tree().unwrap();
-    let resolved_value = tree_c2.path_value(path);
+    let resolved_value = tree_c2.path_value(path).unwrap();
     match resolved_value.into_resolved() {
         Ok(Some(TreeValue::File {
             id,
